IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[adfCompanyCreate]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure adfCompanyCreate'
		DROP PROCEDURE [dbo].[adfCompanyCreate]
	END
GO

PRINT 'Creating procedure adfCompanyCreate'
GO

/****** Object:  StoredProcedure [dbo].[adfCompanyCreate]    Script Date: 02/21/2013 10:46:08 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

/******************************************************************************
**		File: adfCompanyCreate.sql
**		Name: adfCompanyCreate
**		Desc: this procedure automatically generated by Avicom.Adf.Generator
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: Dmitry.Kolchev
**		Date: 10/04/2004 12:32:25
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		17.02.10	PAM			    LegalCode
**
**		6.05.10		KOV				   @ApplyVAT
** 
**		21.05.10	KOV			  @ExchangeBanned, @ExportFinanceOnce 
**  
**		23.06.11	MAN			  [Company].[Number]
**		30.11.11	PAM				  ""    
**		19.02.13	SAP			  [PlanProfitability],[PlanPurchases] 
**								[BIN]
**		28.02.13	SOV			   KPP, LegalCode
**		06.03.13	SAP			  OPENXML
**		09.03.14 	GNV 		  
**		13.03.14	GNV			   ,   
*******************************************************************************/
CREATE PROCEDURE [dbo].[adfCompanyCreate]
	@Xml XML,
	@ExtendedProperties XML,
	@CompanyReturnID INT = NULL OUTPUT
AS
	DECLARE	@result INT,			
			@State TState,
			@ID TIdentifier,
			@FileAs TName,
			@FoundationDate TDate,
			@LegalName TName,
			@KPP TCode,
			@LegalCode TCode,
			@BankAccountID TIdentifier,
			@ExecutiveEmployeeID TIdentifier,
			@AccountantEmployeeID TIdentifier,
			@Own TBool,
			@ManagerEmployeeID TIdentifier,
			@Comments TNotes,
			@Class TClass,
			@CurrentEmployeeID TIdentifier,
			@CurrentUserID TIdentifier,
			@ExternalID TExternalIdentifier,
			@Flag TFlag,
			@Private TBool,
			@CompanyStatusID TIdentifier,
			@value TVariant,
			@ApplyVAT TBool,
			@PaymentCurrencyPoint TBool,
			@ExchangeBanned TBool,
			@ExportFinanceOnce TBool,
			@Number TCode,
			@PlanProfitability TPercent,
			@PlanPurchases TBool = 0,
			@BIN TCode

	SET NOCOUNT ON
	
	SET @State = dbo.adfwfGetInitialState('5206f98c-4e57-44ea-8dc9-0ab6b81fd9c2')
	IF @State IS NULL
	BEGIN
		RAISERROR('#accessdenied', 18, -1)
		RETURN 1
	END

	EXEC adfParameterGet @Name='ApplyVAT', @Group='CommonBusiness', @Value = @value OUTPUT
		SET @ApplyVAT = ISNULL(CAST(@value as bit),	1)
	
	EXEC adfParameterGet @Name='PaymentCurrencyPoint', @Group='CommonBusiness', @Value = @value OUTPUT
		SET @PaymentCurrencyPoint = ISNULL(CAST(@value as bit),1)
	
	EXEC adfParameterGet @Name='ExportFinanceOnce', @Group='Pjm6', @Value = @value OUTPUT
		SET @ExportFinanceOnce = ISNULL(CAST(@value as bit),0)	
	
	EXEC adfParameterGet @Name='ExchangeBanned', @Group='Pjm6', @Value = @value OUTPUT
		SET @ExchangeBanned = ISNULL(CAST(@value as bit),1)

	SET @CurrentUserID = dbo.adfGetCurrentUserID()
	SELECT @value = [Value] FROM [UserParameter] WHERE [Name] = 'SetPrivatePropertyOnCreate' AND [UserID] = @CurrentUserID
	SET @Private = ISNULL(CAST(@value as bit), 1)

	SELECT
		@FileAs = T.c.value('FileAs[1]','nvarchar(256)'),
		@FoundationDate = CONVERT(datetime, T.c.value('FoundationDate[1]','varchar(19)'), 126),
		@LegalName = T.c.value('LegalName[1]','nvarchar(256)'),
		@KPP = T.c.value('KPP[1]','varchar(32)'),
		@LegalCode = CASE 
						WHEN T.c.value('LegalCode[1]','varchar(32)') = '' THEN NULL 
						ELSE T.c.value('LegalCode[1]','varchar(32)') 
					END,
		@BankAccountID = T.c.value('BankAccountID[1]','int'),
		@ExecutiveEmployeeID = T.c.value('ExecutiveEmployeeID[1]','int'),
		@AccountantEmployeeID = T.c.value('AccountantEmployeeID[1]','int'),
		@Own = (CASE T.c.value('Own[1]','varchar(5)') 
					WHEN 'false' THEN 0 
					ELSE 1 
				END),
		@ManagerEmployeeID = T.c.value('ManagerEmployeeID[1]','int'),
		@Comments = T.c.value('Comments[1]','nvarchar(max)'),
		@ExternalID = T.c.value('ExternalID[1]','nvarchar(256)'),
		@Flag = T.c.value('Flag[1]','int'),
		@CompanyStatusID = T.c.value('CompanyStatusID[1]','int'),
		@Number = T.c.value('Number[1]','varchar(32)'),
		@BIN = T.c.value('BIN[1]','varchar(32)')
	FROM
		@Xml.nodes('/adfCompanyGet/Company') AS T(c)
		
	BEGIN TRANSACTION
	
	IF @ManagerEmployeeID IS NULL
	BEGIN
		SET @ManagerEmployeeID = dbo.adfGetCurrentEmployeeID()
	END
	
	IF @Flag IS NULL
	BEGIN
		SET @Flag = 0
	END
	
	
	DECLARE @budgetPlanning bit
		SELECT
			@budgetPlanning = CONVERT(bit,[Value])
		FROM [F41AB0F8-FDF0-4FAD-9B71-8F17B4A51B53]
		WHERE [Name] = 'UseBudgetPlanning' AND [Group] = 'Pjm6'
		
	--         	
	IF @budgetPlanning = 1 AND @Own = 0
	BEGIN
		SELECT
			@PlanProfitability = CONVERT(decimal(18,4),[Value])
		FROM [F41AB0F8-FDF0-4FAD-9B71-8F17B4A51B53]
		WHERE [Name] = 'PlanProfitability' AND [Group] = 'Pjm7'
		
		SELECT
			@PlanPurchases = CONVERT(bit,[Value])
		FROM [F41AB0F8-FDF0-4FAD-9B71-8F17B4A51B53]
		WHERE [Name] = 'PlanPurchases' AND [Group] = 'Pjm7'					
	END
	
	INSERT INTO [Company]
		([Flag], [State], [FileAs], [FoundationDate], [LegalName], 
		 [LegalCode], [KPP], [BIN],
		 [BankAccountID], [ExecutiveEmployeeID], [AccountantEmployeeID], [Own],
		 [ManagerEmployeeID], [Comments], [CompanyStatusID],[ApplyVAT],
		 [PaymentCurrencyPoint],[ExchangeBanned], [ExportFinanceOnce],[Number], [PlanProfitability], [PlanPurchases])
	VALUES
		(@Flag, @State, @FileAs, @FoundationDate, @LegalName, 
		@LegalCode, @KPP, @BIN,
		@BankAccountID, @ExecutiveEmployeeID, @AccountantEmployeeID, @Own,
		@ManagerEmployeeID, @Comments, @CompanyStatusID, @ApplyVAT,
		@PaymentCurrencyPoint, @ExchangeBanned, @ExportFinanceOnce,@Number,@PlanProfitability, @PlanPurchases)

	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END

	SET @ID = SCOPE_IDENTITY()
	
	SELECT @Class = [Class] FROM [Company] WHERE [Company].[ID] = @ID

	INSERT INTO [ExtendedProperties] ([ID], [Class], [ExtendedProperties])
	VALUES(@ID, @Class, @ExtendedProperties)	

	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END

	EXEC @result = adfBusinessCardCreate @doc = @Xml, @XPathRoot = '/adfCompanyGet', @ID = @ID, 
		@Class = @Class, @State = @State, @FileAs = @FileAs, @Comments = @Comments,
		@CompanyID = @ID, @PositionID = NULL, @ExternalID = @ExternalID
	
	IF @@error != 0 OR @result != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END
	
	IF @Private = 1
	BEGIN
		INSERT INTO UserContact ([UserID], [BusinessCardClass], [BusinessCardID])
			VALUES(dbo.adfGetCurrentUserID(), @Class, @ID)
		IF @@ERROR != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END
	END

	INSERT INTO [CompanyIndustryBranch] ([CompanyID], [IndustryBranchID])
	SELECT 
		@ID, 
		T.c.value('IndustryBranchID[1]','int') 
	FROM
		@Xml.nodes('/adfCompanyGet/CompanyIndustryBranch') AS T(c)


	IF @@error != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END
	
	EXEC @result = adfCompanyCalendarCreate @ID, @Own
	IF @result != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END
	
	COMMIT TRANSACTION	
	
	SET @CompanyReturnID = @ID
	
	EXEC adfCompanyGet @ID
	
	RETURN 0
GO

GRANT EXECUTE ON adfCompanyCreate TO PUBLIC
GO
