----- FACTURE
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'Facture') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	/*==============================================================*/
	/* Table: Facture                                               */
	/*==============================================================*/
	create table Facture (
	   Class                TClass               not null,
	   ID                   TIdentifier          identity,
	   State                TState               not null,
	   Flag                 TFlag                not null,
	   FileAs               TName                not null,
	   Date                 TDate                not null,
	   Number               TCode                not null,
	   Advance              TBool                not null default 0,
	   CurrencyCode         TCode3               not null,
	   ProjectID            TIdentifier          not null,
	   SupplierCompanyID    TIdentifier          not null,
	   SupplierAddress      TNotes               null,
	   SupplierINN          TName                null,
	   SupplierKPP          TCode                null,
	   CustomerCompanyID    TIdentifier          not null,
	   CustomerAddress      TNotes               null,
	   CustomerINN          TName                null,
	   CustomerKPP          TCode                null,
	   Sender               TNotes               null,
	   CurrencyRate         TAmount              null default 1,
	   Scale                TCount               null,
	   TotalPC              TMoney               null,
	   TotalWithVATPC       TMoney               null,
	   VATPC                AS ([TotalWithVATPC] - [TotalPC]),
	   Total                TMoney               null,
	   TotalWithVAT         TMoney               null,
	   VAT                  AS ([TotalWithVAT] - [Total]),
	   Recipient            TNotes               null,
	   DocumentNumber       TName                null,
	   DocumentDate         TDate                null,
	   Chief                TName                null,
	   Accountant           TName                null,
	   Created              TSysDateTime         not null default GETDATE(),
	   CreatedBy            TUserID              not null,
	   Modified             TSysDateTime         not null default GETDATE(),
	   ModifiedBy           TUserID              not null,
	   Comments             TNotes               null,
	   Timestamp            timestamp            not null,
	   constraint FacturePK primary key  (ID)
	)
	/*==============================================================*/
	/* Table: FactureLine                                           */
	/*==============================================================*/
	create table FactureLine (
	   ID                   TIdentifier          identity,
	   FactureID            TIdentifier          not null,
	   DocumentClass        TClass               null,
	   DocumentID           TIdentifier          null,
	   Description          TNotes               null,
	   UOMID                TIdentifier          null,
	   Amount               TAmount              null,
	   VATIsIncluded        TBool                null,
	   VATRate              TAmount              null,
	   BasePrice            TCount               not null default 1,
	   SortKey              TCode                null,
	   OriginalPricePC      TMoney               null,
	   PricePC              TMoney               null,
	   ExcisePC             TMoney               null,
	   TotalPC              TMoney               null,
	   VATPC                TMoney               null,
	   TotalWithVATPC       TMoney               null,
	   OriginalPrice        TMoney               null,
	   Price                TMoney               null,
	   Total                TMoney               null,
	   Excise               TMoney               null,
	   VAT                  TMoney               null,
	   TotalWithVAT         TMoney               null,
	   Country              TName                null,
	   CustomDeclaration    TName                null,
	   constraint FactureLinePK primary key  (ID)
	)
END
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'Advance' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='Facture')
	EXEC sp_executesql N'ALTER TABLE [Facture] ADD [Advance] TBool not null default 0'
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'BasePrice' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='FactureLine')
	EXEC sp_executesql N'ALTER TABLE [FactureLine] ADD [BasePrice] TCount not null default 1'
GO
IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'SortKey' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='FactureLine')
	EXEC sp_executesql N'ALTER TABLE [FactureLine] ADD [SortKey] TCode null'
GO

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'FacturePayment') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	/*==============================================================*/
	/* Table: FacturePayment                                        */
	/*==============================================================*/
	create table FacturePayment (
	   PaymentID            TIdentifier          not null,
	   FactureID            TIdentifier          not null,
	   constraint FacturePaymentPK primary key  (PaymentID, FactureID)
	)
END
go

----- CALCULATION
IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'Scale' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='Calculation')
BEGIN
	EXEC sp_executesql N'ALTER TABLE [Calculation] ADD [Scale] TCount NULL'
	EXEC sp_executesql N'
		ALTER TABLE [Calculation] DISABLE TRIGGER ALL
		UPDATE [Calculation] SET [Scale] = 2
		ALTER TABLE [Calculation] ENABLE TRIGGER ALL'
END
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'ServiceTypeID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='Calculation')
	ALTER TABLE [Calculation] ADD [ServiceTypeID] TIdentifier NULL
GO

------ INVOICE
IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'Scale' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='Invoice')
BEGIN
	EXEC sp_executesql N'ALTER TABLE [Invoice] ADD [Scale] TCount NULL'
	EXEC sp_executesql N'
		ALTER TABLE [Invoice] DISABLE TRIGGER ALL
		UPDATE [Invoice] SET [Scale] = 2
		ALTER TABLE [Invoice] ENABLE TRIGGER ALL'
END
GO
IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'BasePrice' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='InvoiceLine')
	EXEC sp_executesql N'ALTER TABLE [InvoiceLine] ADD [BasePrice] TCount not null default 1'
GO
IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'SortKey' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='InvoiceLine')
	EXEC sp_executesql N'ALTER TABLE [InvoiceLine] ADD [SortKey] TCode null'
GO


IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'SupplierCompanyID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='Invoice')
BEGIN
	EXEC sp_executesql N'ALTER TABLE [Invoice] ADD [SupplierCompanyID] TIdentifier NULL'
	EXEC sp_executesql N'ALTER TABLE [Invoice] ADD [CustomerCompanyID] TIdentifier NULL'
	EXEC sp_executesql N'
		ALTER TABLE [Invoice] DISABLE TRIGGER ALL
		UPDATE [Invoice] 
		SET 
			[SupplierCompanyID] = [Project].[SupplierCompanyID],
			[CustomerCompanyID] = [Project].[CustomerCompanyID]
		FROM
			[Invoice]
			INNER JOIN [Project] ON [Project].[ID] = [Invoice].[ProjectID]
		ALTER TABLE [Invoice] ENABLE TRIGGER ALL'
	EXEC sp_executesql N'ALTER TABLE [Invoice] ALTER COLUMN [SupplierCompanyID] TIdentifier NOT NULL'
	EXEC sp_executesql N'ALTER TABLE [Invoice] ALTER COLUMN [CustomerCompanyID] TIdentifier NOT NULL'
END
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'ServiceTypeID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='InvoiceLine')
BEGIN
	EXEC sp_executesql N'ALTER TABLE [InvoiceLine] ADD [ServiceTypeID] TIdentifier NULL'
	EXEC sp_executesql N'
	ALTER TABLE [InvoiceLine] DISABLE TRIGGER ALL
	UPDATE [InvoiceLine]
	SET  [ServiceTypeID] = [Service].[ServiceTypeID]
	FROM
		[InvoiceLine]
		INNER JOIN [ServiceAssignment]
			ON [InvoiceLine].[DocumentClass] = [ServiceAssignment].[Class] AND
				[InvoiceLine].[DocumentID] = [ServiceAssignment].[ID]
		INNER JOIN [Service]
			ON [ServiceAssignment].[ServiceID] = [Service].[ID]
	ALTER TABLE [InvoiceLine] ENABLE TRIGGER ALL'
END
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'TotalBC' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='InvoiceLine')
BEGIN
	EXEC sp_executesql N'ALTER TABLE [InvoiceLine] ADD [TotalBC] TMoney NULL'
	EXEC sp_executesql N'ALTER TABLE [InvoiceLine] ADD [TotalWithVATBC] TMoney NULL'
	EXEC sp_executesql N'
	DECLARE @lines TABLE (
		   ID int,
		   InvoiceID int null,
		   VATIsIncluded bit null,
		   VATRate decimal(18,4) null,
		   Amount decimal(18,4) null,
		   PricePC money null,
		   VATPC money null,
		   Price money null,
		   VAT money null,
		   TotalBC money null,
		   TotalWithVATBC money null)
	INSERT INTO @lines (ID,InvoiceID,VATIsIncluded,VATRate,Amount,PricePC,VATPC,Price,VAT,TotalBC,TotalWithVATBC)
	SELECT 
		ID,InvoiceID,VATIsIncluded,VATRate,Amount,PricePC,VATPC,Price,VAT,TotalBC,TotalWithVATBC
	FROM 
		[InvoiceLine]
		
	DECLARE @BaseCurrency TName, @NationalCurrencyCode TCode3, @value TVariant
	EXEC adfParameterGet ''NationalCurrency'', ''CommonBusiness'', @value OUTPUT	
	SET @NationalCurrencyCode = CAST(@value AS varchar(3))

	EXEC adfParameterGet ''BaseCurrency'', ''CommonBusiness'', @value OUTPUT	
	SET @BaseCurrency = CAST(@value AS varchar(3))

	UPDATE @lines
	SET 
		TotalBC = ROUND(CASE 
						WHEN [Project].[CurrencyCode] = @BaseCurrency
							THEN l.PricePC * l.Amount - l.VATIsIncluded * l.VATPC
						WHEN [Invoice].[CurrencyCode] = @BaseCurrency
							THEN l.Price * l.Amount - l.VATIsIncluded * l.VAT
						WHEN [Project].[CurrencyCode] = @NationalCurrencyCode
							THEN (l.PricePC * l.Amount - l.VATIsIncluded * l.VATPC) / [Invoice].[BaseCurrencyRate]
						ELSE (l.PricePC * l.Amount - l.VATIsIncluded * l.VATPC) * [Invoice].[BaseCurrencyRate]
					END, 2),
		TotalWithVATBC = ROUND(CASE 
						WHEN [Project].[CurrencyCode] = @BaseCurrency
							THEN l.PricePC * l.Amount + (1 - l.VATIsIncluded) * l.VATPC
						WHEN [Invoice].[CurrencyCode] = @BaseCurrency
							THEN l.Price * l.Amount + (1 - l.VATIsIncluded) * l.VAT
						WHEN [Project].[CurrencyCode] = @NationalCurrencyCode
							THEN (l.PricePC * l.Amount + (1 - l.VATIsIncluded) * l.VATPC) / [Invoice].[BaseCurrencyRate]
						ELSE (l.PricePC * l.Amount + (1 - l.VATIsIncluded) * l.VATPC) * [Invoice].[BaseCurrencyRate]
					END, 2)
	FROM
		@lines l
		INNER JOIN [Invoice]
			ON [Invoice].[ID] = l.[InvoiceID]
		INNER JOIN [Project]
			ON [Project].[ID] = [Invoice].[ProjectID]

	ALTER TABLE [InvoiceLine] DISABLE TRIGGER ALL
	UPDATE
		[InvoiceLine]
	SET
		TotalBC = l.TotalBC,
		TotalWithVATBC = l.TotalWithVATBC
	FROM
		[InvoiceLine] 
		INNER JOIN @lines l 
			ON [InvoiceLine].[ID] = l.[ID]
	ALTER TABLE [InvoiceLine] ENABLE TRIGGER ALL'
END
GO

------ STATEMENT
IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'Paid' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='Statement')
BEGIN
	EXEC sp_executesql N'ALTER TABLE [Statement] ADD [Paid] TMoney NULL'
	EXEC sp_executesql N'ALTER TABLE [Statement] ADD [PaidWithVAT] TMoney NULL'
	EXEC sp_executesql N'ALTER TABLE [Statement] ADD [PaidVAT] AS ([PaidWithVAT] - [Paid])'
END

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'Scale' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='Statement')
BEGIN
	EXEC sp_executesql N'ALTER TABLE [Statement] ADD [Scale] TCount NULL'
	EXEC sp_executesql N'
		ALTER TABLE [Statement] DISABLE TRIGGER ALL
		UPDATE [Statement] SET [Scale] = 2
		ALTER TABLE [Statement] ENABLE TRIGGER ALL'
END
GO
IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'BasePrice' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='StatementLine')
	EXEC sp_executesql N'ALTER TABLE [StatementLine] ADD [BasePrice] TCount not null default 1'
GO
IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'SortKey' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='StatementLine')
	EXEC sp_executesql N'ALTER TABLE [StatementLine] ADD [SortKey] TCode null'
GO


IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'SupplierCompanyID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='Statement')
BEGIN
	EXEC sp_executesql N'ALTER TABLE [Statement] ADD [SupplierCompanyID] TIdentifier NULL'
	EXEC sp_executesql N'ALTER TABLE [Statement] ADD [CustomerCompanyID] TIdentifier NULL'
	EXEC sp_executesql N'
		ALTER TABLE [Statement] DISABLE TRIGGER ALL
		UPDATE [Statement] 
		SET 
			[SupplierCompanyID] = [Project].[SupplierCompanyID],
			[CustomerCompanyID] = [Project].[CustomerCompanyID]
		FROM
			[Statement]
			INNER JOIN [Project] ON [Project].[ID] = [Statement].[ProjectID]
		ALTER TABLE [Statement] ENABLE TRIGGER ALL'
	EXEC sp_executesql N'ALTER TABLE [Statement] ALTER COLUMN [SupplierCompanyID] TIdentifier NOT NULL'
	EXEC sp_executesql N'ALTER TABLE [Statement] ALTER COLUMN [CustomerCompanyID] TIdentifier NOT NULL'
END
GO


IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'ServiceTypeID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='StatementLine')
BEGIN
	EXEC sp_executesql N'ALTER TABLE [StatementLine] ADD [ServiceTypeID] TIdentifier NULL'
	
	EXEC sp_executesql N'
	ALTER TABLE [StatementLine] DISABLE TRIGGER ALL
	UPDATE [StatementLine]
	SET [ServiceTypeID] = [Service].[ServiceTypeID]
	FROM
		[StatementLine]
		INNER JOIN [ServiceAssignment]
			ON [StatementLine].[DocumentClass] = [ServiceAssignment].[Class] AND
				[StatementLine].[DocumentID] = [ServiceAssignment].[ID]
		INNER JOIN [Service]
			ON [ServiceAssignment].[ServiceID] = [Service].[ID]
	ALTER TABLE [StatementLine] ENABLE TRIGGER ALL'
END
GO

IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'StatementPaymentValue' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='StatementAdvanceInvoice')
BEGIN
	EXEC sp_executesql N'ALTER TABLE [StatementAdvanceInvoice] ADD [StatementPaymentValue] TMoney NULL'
	EXEC sp_executesql N'ALTER TABLE [StatementAdvanceInvoice] ADD [StatementPaymentVAT] TMoney NULL'
	EXEC sp_executesql N'ALTER TABLE [StatementAdvanceInvoice] ADD [StatementPaymentValuePC] TMoney NULL'
	EXEC sp_executesql N'ALTER TABLE [StatementAdvanceInvoice] ADD [StatementPaymentVATPC] TMoney NULL'
END
GO

--------- SERVICE ASSIGNMENT
IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'ServiceTypeID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='ServiceAssignment')
BEGIN
	EXEC sp_executesql N'ALTER TABLE [ServiceAssignment] ADD [ServiceTypeID] TIdentifier NULL'
	
	EXEC sp_executesql N'
	ALTER TABLE [ServiceAssignment] DISABLE TRIGGER ALL
	UPDATE [ServiceAssignment]
	SET [ServiceTypeID] = [Service].[ServiceTypeID]
	FROM
		[ServiceAssignment]
		INNER JOIN [Service]
			ON [ServiceAssignment].[ServiceID] = [Service].[ID]
	ALTER TABLE [ServiceAssignment] ENABLE TRIGGER ALL'
	EXEC sp_executesql N'ALTER TABLE [ServiceAssignment] ALTER COLUMN [ServiceTypeID] TIdentifier NOT NULL'
END
GO

-------- SERVICE REQUEST
IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'ServiceTypeID' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='ServiceRequest')
	ALTER TABLE [ServiceRequest] ADD [ServiceTypeID] TIdentifier NULL
GO
IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'Description' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='ServiceRequest')
	ALTER TABLE [ServiceRequest] ADD [Description] TNotes NULL
GO

-------- SERVICE Type
IF NOT EXISTS (SELECT * FROM syscolumns WHERE [name] = 'RatesDepended' AND OBJECTPROPERTY(id, N'IsUserTable') = 1 AND OBJECT_NAME(id)='ServiceType')
	ALTER TABLE [ServiceType] ADD [RatesDepended] TBool NOT NULL DEFAULT 0
GO

