﻿IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[fnPjm7GetWeeksListBetweenTwoDates]') AND xtype IN (N'FN', N'IF', N'TF'))
	BEGIN
		PRINT 'Dropping function fnPjm7GetWeeksListBetweenTwoDates'
		DROP FUNCTION [dbo].[fnPjm7GetWeeksListBetweenTwoDates]
	END

GO

PRINT 'Creating function fnPjm7GetWeeksListBetweenTwoDates'
GO
/******************************************************************************
**		File: fnPjm7GetWeeksListBetweenTwoDates.sql
**		Name: fnPjm7GetWeeksListBetweenTwoDates
**		Desc: Функция возвращает список дат (по неделям) между двумя датами
**
**		This template can be customized:
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: anton.paramonov
**		Date: 03.10.2011 14:55
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE FUNCTION fnPjm7GetWeeksListBetweenTwoDates
(
      @DtFrom DATETIME,
      @DtTo DATETIME
)

RETURNS @WeekList TABLE ([Date] datetime, [Week] int, [Month] int, [Year] int, [PeriodName] nvarchar(30))

AS
BEGIN
	DECLARE @Temp TABLE ([Date] datetime, [Week] int, [Month] int, [Year] int, [PeriodName] nvarchar(30))
    DECLARE 
		@TotDays INT,
		@CNT INT

    SET @TotDays =  DATEDIFF(DD,@DtFrom,@DtTo)

    SET @CNT = 0
    WHILE @TotDays >= @CNT
    BEGIN
	DECLARE @DayDate datetime
	SET @DayDate = (@DtTo - @CNT)

	INSERT INTO @Temp
	SELECT
		@DayDate AS [Date],
		dbo.fnPjm7GetMonthWeekNumber(@DayDate) AS [Week],
		MONTH(@DayDate) AS [Month],
		YEAR(@DayDate) AS [Year],
		(CASE MONTH(@DayDate)
					WHEN 1 THEN 'Январь'
					WHEN 2 THEN 'Февраль'
					WHEN 3 THEN 'Март'
					WHEN 4 THEN 'Апрель'
					WHEN 5 THEN 'Май'
					WHEN 6 THEN 'Июнь'
					WHEN 7 THEN 'Июль'
					WHEN 8 THEN 'Август'
					WHEN 9 THEN 'Сентябрь'
					WHEN 10 THEN 'Октябрь'
					WHEN 11 THEN 'Ноябрь'
					WHEN 12 THEN 'Декабрь'
				END)+ ', ' + CAST(YEAR(@DayDate) as nvarchar)
		
	SET @CNT = @CNT + 1
    END

	INSERT INTO @WeekList
	SELECT
		MIN([Date]),
		[Week],
		[Month],
		[Year],
		[PeriodName]
	FROM
		@Temp
	GROUP BY
		[Week],
		[Month],
		[Year],
		[PeriodName]
		
    RETURN
END

GO 
  