IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[fnPjm6GetNextPaymentDate]') AND xtype IN (N'FN', N'IF', N'TF'))
	BEGIN
		PRINT 'Dropping function fnPjm6GetNextPaymentDate'
		DROP FUNCTION [dbo].[fnPjm6GetNextPaymentDate]
	END

GO

PRINT 'Creating function fnPjm6GetNextPaymentDate'
GO
/******************************************************************************
**		File: 
**		Name: fnPjm6GetNextPaymentDate
**		Desc: 
**
**		This template can be customized:
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE FUNCTION fnPjm6GetNextPaymentDate(@PeriodType TCode, @DayType TCode, @Date TDate)
RETURNS TDate
AS
BEGIN
	IF @PeriodType IS NULL
		RETURN NULL
		
	IF @Date IS NULL
		SET @Date = CAST(CONVERT(varchar(8), GETDATE(), 112) as datetime)
	ELSE
		SET @Date = CAST(CONVERT(varchar(8), @Date, 112) as datetime)
		
	DECLARE @nextDate TDate
	SET @nextDate = dbo.fnPjm6GetNextFirstPeriodDate(@PeriodType, @DayType, @Date)
	IF @DayType IN ('LAST_CALENDAR', 'LAST_WORKING')
	BEGIN
		SET @nextDate = dbo.fnPjm6GetNextFirstPeriodDate(@PeriodType, @DayType, @nextDate)
		SET @nextDate = DATEADD(day, -1, @nextDate)
	END
	
	IF @DayType ='FIRST_WORKING'
	BEGIN
		WHILE dbo.fnPjm5IsWorkingDay(@nextDate) = 0
			SET @nextDate = DATEADD(day, 1, @nextDate)
	END
	ELSE IF @DayType ='LAST_WORKING'
	BEGIN
		WHILE dbo.fnPjm5IsWorkingDay(@nextDate) = 0
			SET @nextDate = DATEADD(day, -1, @nextDate)
	END
	
	RETURN @nextDate
END
GO

GRANT EXEC ON fnPjm6GetNextPaymentDate TO PUBLIC
GO

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[fnPjm6GetNextFirstPeriodDate]') AND xtype IN (N'FN', N'IF', N'TF'))
	BEGIN
		PRINT 'Dropping function fnPjm6GetNextFirstPeriodDate'
		DROP FUNCTION [dbo].[fnPjm6GetNextFirstPeriodDate]
	END

GO

PRINT 'Creating function fnPjm6GetNextFirstPeriodDate'
GO
/******************************************************************************
**		File: 
**		Name: fnPjm6GetNextFirstPeriodDate
**		Desc: 
**
**		This template can be customized:
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE FUNCTION fnPjm6GetNextFirstPeriodDate(@PeriodType TCode, @DayType TCode, @Date TDate)
RETURNS TDate
AS
BEGIN
	DECLARE @nextDate TDate, @day int, @month int
	DECLARE @portion int
	IF @PeriodType = '10'
	BEGIN
		SET @day = Day(@Date)
		SET @portion = (@day-1) / 10
		SET @nextDate = DATEADD(day, 1 - @day, @Date)
		IF @portion = 2
			SET @nextDate = DATEADD(month, 1, @nextDate)
		ELSE
			SET @nextDate = DATEADD(day, 10 * (@portion+1), @nextDate)
	END
	ELSE IF @PeriodType = '15'
	BEGIN
		SET @day = Day(@Date)
		SET @portion = (@day-1) / 15
		SET @nextDate = DATEADD(day, 1 - @day, @Date)
		IF @portion = 0
			SET @nextDate = DATEADD(day, 15, @nextDate)
		ELSE
			SET @nextDate = DATEADD(month, 1, @nextDate)
	END
	ELSE IF @PeriodType = '30'
	BEGIN
		SET @day = Day(@Date)
		SET @nextDate = DATEADD(day, 1 - @day, @Date)
		SET @nextDate = DATEADD(month, 1, @nextDate)
	END
	ELSE IF @PeriodType = '90'
	BEGIN
		SET @day = DATEPART(dayofyear, @Date)
		SET @month = MONTH(@Date)
		SET @portion = (@month-1) / 3
		SET @nextDate = DATEADD(day, 1 - @day, @Date)
		SET @nextDate = DATEADD(month, 3 * (@portion+1), @nextDate)
	END
	ELSE IF @PeriodType = '180'
	BEGIN
		SET @day = DATEPART(dayofyear, @Date)
		SET @month = MONTH(@Date)
		SET @portion = (@month-1) / 6
		SET @nextDate = DATEADD(day, 1 - @day, @Date)
		SET @nextDate = DATEADD(month, 6 * (@portion+1), @nextDate)
	END
	ELSE IF @PeriodType = '360'
	BEGIN
		SET @day = DATEPART(dayofyear, @Date)
		SET @nextDate = DATEADD(day, 1 - @day, @Date)
		SET @nextDate = DATEADD(year, 1, @nextDate)
	END
	RETURN @nextDate
END
GO

GRANT EXEC ON fnPjm6GetNextFirstPeriodDate TO PUBLIC
GO
  