IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[fnReportGetTimeCostSum]') AND xtype IN (N'FN', N'IF', N'TF'))
	BEGIN
		PRINT 'Dropping function fnReportGetTimeCostSum'
		DROP FUNCTION [dbo].[fnReportGetTimeCostSum]
	END
GO

PRINT 'Creating function fnReportGetTimeCostSum'
GO

/******************************************************************************
**		File: fnReportGetTimeCostSum.sql
**		Name: fnReportGetTimeCostSum
**
**              
**		Return values: 
**		
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: MAN
**		Date: 23/08/2010 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		23.08.2010	MAN				First Version
*******************************************************************************/
CREATE FUNCTION dbo.fnReportGetTimeCostSum 
(
	@PeriodStart		TDateTime,
	@PeriodEnd			TDateTime,
	@CurrencyCode		TCode3,
	@ProjectXML			XML
)
RETURNS @ResultTimeCostSum  TABLE 
(
	ProjectID				Tidentifier,
	TimeCostingBillable		TAmount,	--  :  
	TimeCostingRegistered	TAmount,	--  :  
	TimeCostingApproved		TAmount,	--  : 
	TimeRevenueSold			TAmount,	--  :  
	TimeRevenueApproved		TAmount,	--  : 
	TimeRevenueSigned		TAmount		--  : 
)
AS
BEGIN
	DECLARE
		@CalculationClass TClass,
		@StatementClass TClass,
		@InvoiceClass TClass,
		@TimeEntryClass TClass,
		@ExpenseClass TClass,
		@ApprovedState TState
	SET	@CalculationClass = dbo.adfGetClassByName('Pjm5.Calculation')
	SET	@StatementClass = dbo.adfGetClassByName('Pjm5.Statement')
	SET	@InvoiceClass = dbo.adfGetClassByName('Pjm5.Invoice')
	SET	@TimeEntryClass = dbo.adfGetClassByName('Pjm5.TimeEntry')
	SET	@ExpenseClass = dbo.adfGetClassByName('Pjm5.Expense')

	DECLARE @CostingBillable	TABLE( ProjectID Tidentifier, TimeID TIdentifier, Value TAmount )
	DECLARE @CostingRegistered	TABLE( ProjectID Tidentifier, TimeID TIdentifier, Value TAmount )
	DECLARE @CostingApproved	TABLE( ProjectID Tidentifier, TimeID TIdentifier, Value TAmount )
	DECLARE @RevenueSold		TABLE( ProjectID Tidentifier, TimeID TIdentifier, Value TAmount )
	DECLARE @RevenueApproved	TABLE( ProjectID Tidentifier, TimeID TIdentifier, Value TAmount )
	DECLARE @RevenueSigned		TABLE( ProjectID Tidentifier, TimeID TIdentifier, Value TAmount )

	DECLARE @DocumentInFinance TABLE(DocumentID TIdentifier,LinkClass TClass, State	TState, Date TDateTime,ThroughCalculation TFlag	)
	INSERT INTO @DocumentInFinance( DocumentID, LinkClass, State, Date, ThroughCalculation )
	SELECT 
	[F].[DocumentID], [F].[LinkClass], [F].[State], [F].[Date],	[F].[ThroughCalculation]
	FROM
	fnReportGetDocumentInFinance(@TimeEntryClass,@PeriodStart, @PeriodEnd,	@ProjectXML) AS [F]

	INSERT INTO	@CostingBillable 
	(
		[ProjectID],
		[TimeID],
		[Value]
	)
	SELECT
	DISTINCT
		[vwTimeEntry].[ProjectID],
		[vwTimeEntry].[ID],
		SUM
		( DISTINCT
			CASE 
				WHEN [Project].[ApplyVAT] = 0 AND [Project].[RatesWithVAT] = 1 
					THEN [vwTimeEntry].[RoundedDuration] / 60.0 * [vwTimeEntry].[Rate] / ( 1 + [vwTimeEntry].[VATRate] )
				WHEN
					( [Project].[ApplyVAT] = 0 AND [Project].[RatesWithVAT] = 0  )
					OR
					( [Project].[ApplyVAT] = 1 AND [Project].[RatesWithVAT] = 1  )
					THEN [vwTimeEntry].[RoundedDuration] / 60.0 * [vwTimeEntry].[Rate]
				WHEN [Project].[ApplyVAT] = 1 AND [Project].[RatesWithVAT] = 0 
					THEN [vwTimeEntry].[RoundedDuration] / 60.0 * [vwTimeEntry].[Rate] * ( 1 + [vwTimeEntry].[VATRate] )
			END *
			CASE [vwTimeEntry].[State]
				WHEN 6 
					THEN dbo.fnReportGetFinDocumentCurrency([vwTimeEntry].[CurrencyCode],[vwTimeEntry].[ID],@TimeEntryClass,@CurrencyCode)
				ELSE
					dbo.pjm6fnCurrencyGetRate
						(
							[vwTimeEntry].[CurrencyCode],
							[vwTimeEntry].[StartTime], 
							@CurrencyCode
						)
			END
		) AS [TimeCostingBillable]
	FROM
	[vwTimeEntry]
	INNER JOIN [Project]
		ON [Project].[ID] = [vwTimeEntry].[ProjectID]
	LEFT OUTER JOIN @DocumentInFinance
		ON [@DocumentInFinance].[DocumentID] = [vwTimeEntry].[ID]
	WHERE 
		[vwTimeEntry].[Billable] = 1 --   
		AND
		[vwTimeEntry].[State] IN (3, 6) -- , 
		AND
		[vwTimeEntry].[StartTime] >= @PeriodStart
		AND
		[vwTimeEntry].[StartTime] < @PeriodEnd
		AND
		[vwTimeEntry].[ProjectID] IN 
		(	
			SELECT
				ROW.value('@ProjectID','TIdentifier') AS [ProjectID]
			FROM
				@ProjectXML.nodes('/row') AS ROW(ROW)
		)
				GROUP BY 
			[vwTimeEntry].[ID],
			[vwTimeEntry].[ProjectID]
		
	INSERT INTO	@CostingRegistered
	(
		[ProjectID],
		[TimeID],
		[Value]
	)
	SELECT
		[vwTimeEntry].[ProjectID],
		[vwTimeEntry].[ID],
		SUM
		( 
			DISTINCT
			CASE 
				WHEN [Project].[ApplyVAT] = 0 AND [Project].[RatesWithVAT] = 1 
					THEN [vwTimeEntry].[BillableHour] * [vwTimeEntry].[Rate] / ( 1 + [vwTimeEntry].[VATRate] )
				WHEN
					( [Project].[ApplyVAT] = 0 AND [Project].[RatesWithVAT] = 0  )
					OR
					( [Project].[ApplyVAT] = 1 AND [Project].[RatesWithVAT] = 1  )
					THEN [vwTimeEntry].[BillableHour] * [vwTimeEntry].[Rate]
				WHEN [Project].[ApplyVAT] = 1 AND [Project].[RatesWithVAT] = 0 
					THEN [vwTimeEntry].[BillableHour] * [vwTimeEntry].[Rate] * ( 1 + [vwTimeEntry].[VATRate] )
			END *
			dbo.fnReportGetFinDocumentCurrency([vwTimeEntry].[CurrencyCode],[vwTimeEntry].[ID],@TimeEntryClass,@CurrencyCode)
		) 
	FROM
			[vwTimeEntry]
			INNER JOIN @DocumentInFinance
				ON [vwTimeEntry].[ID] = [@DocumentInFinance].[DocumentID]
			INNER JOIN [Project]
				ON [Project].[ID] = [vwTimeEntry].[ProjectID]
	WHERE
		[@DocumentInFinance].[ThroughCalculation] = 0
		AND
		(
			(
				[@DocumentInFinance].[LinkClass] = @CalculationClass
				AND
				[@DocumentInFinance].[State] IN (1,2,3,5)
			)
			OR
			(
				[@DocumentInFinance].[LinkClass] = @StatementClass
				AND
				[@DocumentInFinance].[State] IN (1,2,3)
			) 
			OR
			(
				[@DocumentInFinance].[LinkClass] = @InvoiceClass
				AND
				[@DocumentInFinance].[State] IN (1,2,4,5,6,7)	
			)
		)
	GROUP BY 
		[vwTimeEntry].[ID],
		[vwTimeEntry].[ProjectID]

	INSERT INTO	@CostingApproved
	(
		[ProjectID],
		[TimeID],
		[Value]
	)
	SELECT

		[vwTimeEntry].[ProjectID],
		[vwTimeEntry].[ID],
		SUM
		(
			DISTINCT 
			CASE 
				WHEN [Project].[ApplyVAT] = 0 AND [Project].[RatesWithVAT] = 1 
					THEN [vwTimeEntry].[BillableHour] * [vwTimeEntry].[Rate] / ( 1 + [vwTimeEntry].[VATRate] )
				WHEN
					( [Project].[ApplyVAT] = 0 AND [Project].[RatesWithVAT] = 0  )
					OR
					( [Project].[ApplyVAT] = 1 AND [Project].[RatesWithVAT] = 1  )
				THEN [vwTimeEntry].[BillableHour] * [vwTimeEntry].[Rate]
					WHEN [Project].[ApplyVAT] = 1 AND [Project].[RatesWithVAT] = 0 
				THEN [vwTimeEntry].[BillableHour] * [vwTimeEntry].[Rate] * ( 1 + [vwTimeEntry].[VATRate] )
			END *
			dbo.fnReportGetFinDocumentCurrency([vwTimeEntry].[CurrencyCode],[vwTimeEntry].[ID],@TimeEntryClass,@CurrencyCode)
		)
	FROM
		[vwTimeEntry]
		INNER JOIN @DocumentInFinance
			ON [vwTimeEntry].[ID] = [@DocumentInFinance].[DocumentID]
		INNER JOIN [Project]
			ON [Project].[ID] = [vwTimeEntry].[ProjectID]
	WHERE
		(
			[@DocumentInFinance].[LinkClass] = @CalculationClass
			AND
			[@DocumentInFinance].[State] IN (2,3)
		)
		OR
		(
			[@DocumentInFinance].[ThroughCalculation] = 0
			AND 
			[@DocumentInFinance].[LinkClass] = @StatementClass
			AND
			[@DocumentInFinance].[State] IN (2,3)
		) 
		OR
		(	
			[@DocumentInFinance].[ThroughCalculation] = 0
			AND 
			[@DocumentInFinance].[LinkClass] = @InvoiceClass
			AND
			[@DocumentInFinance].[State] IN (2,5)	
		)
	GROUP BY 
		[vwTimeEntry].[ID],
		[vwTimeEntry].[ProjectID]

	INSERT INTO	@RevenueSold
	(
		[ProjectID],
		[TimeID],
		[Value]
	)
	SELECT
		DISTINCT
		[vwTimeEntry].[ProjectID],
		[vwTimeEntry].[ID],
		SUM
		( 
			DISTINCT
			CASE 
				WHEN [Project].[ApplyVAT] = 0 AND [Project].[RatesWithVAT] = 1 
					THEN [vwTimeEntry].[BillableHour] * [vwTimeEntry].[Rate] / ( 1 + [vwTimeEntry].[VATRate] )
				WHEN
					( [Project].[ApplyVAT] = 0 AND [Project].[RatesWithVAT] = 0  )
					OR
					( [Project].[ApplyVAT] = 1 AND [Project].[RatesWithVAT] = 1  )
					THEN [vwTimeEntry].[BillableHour] * [vwTimeEntry].[Rate]
				WHEN [Project].[ApplyVAT] = 1 AND [Project].[RatesWithVAT] = 0 
					THEN [vwTimeEntry].[BillableHour] * [vwTimeEntry].[Rate] * ( 1 + [vwTimeEntry].[VATRate] )
			END *
			dbo.fnReportGetFinDocumentCurrency([vwTimeEntry].[CurrencyCode],[vwTimeEntry].[ID],@TimeEntryClass,@CurrencyCode)
		)
	FROM
		[vwTimeEntry]
		INNER JOIN @DocumentInFinance
			ON [vwTimeEntry].[ID] = [@DocumentInFinance].[DocumentID]
		INNER JOIN [Project]
			ON [Project].[ID] = [vwTimeEntry].[ProjectID]
	WHERE 
		(
			[@DocumentInFinance].[LinkClass] = @StatementClass
			AND
			[@DocumentInFinance].[State] IN (1,2,3)
		) 
		OR
		(
			[@DocumentInFinance].[LinkClass] = @InvoiceClass
			AND
			[@DocumentInFinance].[State] IN (1,2,4,5,6,7)
		)	
	GROUP BY 
		[vwTimeEntry].[ID],
		[vwTimeEntry].[ProjectID]
		
	INSERT INTO	@RevenueApproved
	(
		[ProjectID],
		[TimeID],
		[Value]
	)
	SELECT
		DISTINCT
		[vwTimeEntry].[ProjectID],
		[vwTimeEntry].[ID],	
		SUM
		(
			DISTINCT
			CASE 
				WHEN [Project].[ApplyVAT] = 0 AND [Project].[RatesWithVAT] = 1 
					THEN [vwTimeEntry].[BillableHour] * [vwTimeEntry].[Rate] / ( 1 + [vwTimeEntry].[VATRate] )
				WHEN
					( [Project].[ApplyVAT] = 0 AND [Project].[RatesWithVAT] = 0  )
					OR
					( [Project].[ApplyVAT] = 1 AND [Project].[RatesWithVAT] = 1  )
					THEN [vwTimeEntry].[BillableHour] * [vwTimeEntry].[Rate]
				WHEN [Project].[ApplyVAT] = 1 AND [Project].[RatesWithVAT] = 0 
					THEN [vwTimeEntry].[BillableHour] * [vwTimeEntry].[Rate] * ( 1 + [vwTimeEntry].[VATRate] )
			END *
			dbo.fnReportGetFinDocumentCurrency([vwTimeEntry].[CurrencyCode],[vwTimeEntry].[ID],@TimeEntryClass,@CurrencyCode)
		)
	FROM
		[vwTimeEntry]
		INNER JOIN @DocumentInFinance
			ON [vwTimeEntry].[ID] = [@DocumentInFinance].[DocumentID]
		INNER JOIN [Project]
			ON [Project].[ID] = [vwTimeEntry].[ProjectID]
	WHERE
		(
			[@DocumentInFinance].[LinkClass] = @StatementClass
			AND
			[@DocumentInFinance].[State] IN (2,3)
		) 
		OR
		(
			[@DocumentInFinance].[LinkClass] = @InvoiceClass
			AND
			[@DocumentInFinance].[State] IN (2,5)	
		)
	GROUP BY 
		[vwTimeEntry].[ID],
		[vwTimeEntry].[ProjectID]
			
	INSERT INTO	@RevenueSigned
	(
		[ProjectID],
		[TimeID],
		[Value]
	)
	SELECT
		DISTINCT
		[vwTimeEntry].[ProjectID],
		[vwTimeEntry].[ID],	
		SUM
		( 
			DISTINCT
			CASE 
				WHEN [Project].[ApplyVAT] = 0 AND [Project].[RatesWithVAT] = 1 
					THEN [vwTimeEntry].[BillableHour] * [vwTimeEntry].[Rate] / ( 1 + [vwTimeEntry].[VATRate] )
				WHEN
					( [Project].[ApplyVAT] = 0 AND [Project].[RatesWithVAT] = 0  )
					OR
					( [Project].[ApplyVAT] = 1 AND [Project].[RatesWithVAT] = 1  )
					THEN [vwTimeEntry].[BillableHour] * [vwTimeEntry].[Rate]
				WHEN [Project].[ApplyVAT] = 1 AND [Project].[RatesWithVAT] = 0 
					THEN [vwTimeEntry].[BillableHour] * [vwTimeEntry].[Rate] * ( 1 + [vwTimeEntry].[VATRate] )
			END *
			dbo.fnReportGetFinDocumentCurrency([vwTimeEntry].[CurrencyCode],[vwTimeEntry].[ID],@TimeEntryClass,@CurrencyCode)
		)
	FROM
		[vwTimeEntry]
		INNER JOIN @DocumentInFinance
			ON [vwTimeEntry].[ID] = [@DocumentInFinance].[DocumentID]
		INNER JOIN [Project]
			ON [Project].[ID] = [vwTimeEntry].[ProjectID]
	WHERE
		(
			[@DocumentInFinance].[LinkClass] = @StatementClass
			AND
			[@DocumentInFinance].[State] IN ( 3 )
		) 
		OR
		(
			[@DocumentInFinance].[LinkClass] = @InvoiceClass
			AND
			[@DocumentInFinance].[State] IN ( 5 )	
		)
	GROUP BY 
		[vwTimeEntry].[ID],
		[vwTimeEntry].[ProjectID]
			
	INSERT INTO @ResultTimeCostSum
	(
		ProjectID,
		TimeCostingBillable,	--  :  
		TimeCostingRegistered,	--  :  
		TimeCostingApproved,	--  : 
		TimeRevenueSold,		--  :  
		TimeRevenueApproved,	--  : 
		TimeRevenueSigned		--  : 
	)
	SELECT 
		[@CostingBillable].[ProjectID],
		SUM([@CostingBillable].[Value]),
		SUM([@CostingRegistered].[Value]),
		SUM([@CostingApproved].[Value]),
		SUM([@RevenueSold].[Value]),
		SUM([@RevenueApproved].[Value]),
		SUM([@RevenueSigned].[Value])
	FROM
		@CostingBillable
		LEFT OUTER JOIN @CostingRegistered
			ON [@CostingRegistered].[TimeID] = [@CostingBillable].[TimeID]
		LEFT OUTER JOIN @CostingApproved
			ON [@CostingApproved].[TimeID] = [@CostingBillable].[TimeID]
		LEFT OUTER JOIN @RevenueSold
			ON [@RevenueSold].[TimeID] = [@CostingBillable].[TimeID]
		LEFT OUTER JOIN @RevenueApproved
			ON [@RevenueApproved].[TimeID] = [@CostingBillable].[TimeID]
		LEFT OUTER JOIN @RevenueSigned
			ON [@RevenueSigned].[TimeID] = [@CostingBillable].[TimeID]
	GROUP BY
		[@CostingBillable].[ProjectID]
	RETURN
END
GO