IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[fnReportSplitPeriod]') AND xtype IN (N'FN', N'IF', N'TF'))
	BEGIN
		PRINT 'Dropping function fnReportSplitPeriod'
		DROP FUNCTION [dbo].[fnReportSplitPeriod]
	END

GO

PRINT 'Creating function fnReportSplitPeriod'
GO

/******************************************************************************
**		File: fnReportSplitPeriod.sql
**		Name: fnReportSplitPeriod
**
**              
**		Return values: 
**		   ,   
**		Called by:   
**              
**		Parameters:
**		Input								Output
**      ----------							-----------
**		
**		
**		Auth: avgusta.bekhtina
**		Date: 10.12.14 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
*******************************************************************************/
CREATE FUNCTION [dbo].[fnReportSplitPeriod]
(
	@PeriodStart date,
	@PeriodEnd date,
	@LangCode TCode3,
	@Short bit,
	@WithYears bit,
	@PeriodType INT --  (0 - , 1 - , 2 - )
)
RETURNS @PeriodSplits TABLE
(
	PeriodStartDate date,
	PeriodEndDate date,
	PeriodName TName	
)
AS
BEGIN
	DECLARE @period int = 0,
			@dateDiff int

	IF(@PeriodType = 0)
	BEGIN
		SET @dateDiff = DATEDIFF(MONTH, @PeriodStart, @PeriodEnd)
	END
	ELSE IF(@PeriodType = 1)
	BEGIN
		SET @dateDiff = DATEDIFF(QUARTER, @PeriodStart, @PeriodEnd)
	END
	ELSE
	BEGIN
		SET @dateDiff = DATEDIFF(YEAR, @PeriodStart, @PeriodEnd)
	END

	DECLARE @periodTab table ( [Number] int )
	
	WHILE @period <= @dateDiff
	BEGIN
		INSERT INTO @periodTab VALUES(@period)
		SET @period = @period + 1 
	END

	INSERT INTO @PeriodSplits
	SELECT 
		CASE
			WHEN @PeriodType = 0
				THEN DATEADD(MONTH, n.Number, @PeriodStart)
			WHEN  @PeriodType = 1
				THEN DATEADD(QUARTER, n.Number, @PeriodStart)
			WHEN @PeriodType = 2
				THEN DATEADD(YEAR, n.Number, @PeriodStart)
		END,
		CASE
			WHEN @PeriodType = 0
				THEN DATEADD(MONTH, n.Number+1, @PeriodStart)
			WHEN  @PeriodType = 1
				THEN DATEADD(QUARTER, n.Number+1, @PeriodStart)
			WHEN @PeriodType = 2
				THEN DATEADD(YEAR, n.Number+1, @PeriodStart)
		END,
		CASE
			WHEN @PeriodType = 0
				THEN dbo.fnReportGetPeriodName(DATEADD(MONTH, n.Number, @PeriodStart), NULL,'ru',@Short,@WithYears,@PeriodType)
			WHEN  @PeriodType = 1
				THEN dbo.fnReportGetPeriodName(DATEADD(QUARTER, n.Number, @PeriodStart), NULL,'ru',@Short,@WithYears,@PeriodType)
			WHEN @PeriodType = 2
				THEN dbo.fnReportGetPeriodName(DATEADD(YEAR, n.Number, @PeriodStart), NULL,'ru',@Short,@WithYears,@PeriodType)
		END
	FROM 
		@periodTab AS n
	WHERE 
		n.Number <= @dateDiff	


RETURN
END
