﻿IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[fnReportSplitPeriodOnYears]') AND xtype IN (N'FN', N'IF', N'TF'))
	BEGIN
		PRINT 'Dropping function fnReportSplitPeriodOnYears'
		DROP FUNCTION [dbo].[fnReportSplitPeriodOnYears]
	END
GO

PRINT 'Creating function fnReportSplitPeriodOnYears'
GO

/******************************************************************************
**		File: fnReportSplitPeriodOnYears.sql
**		Name: fnReportSplitPeriodOnYears
**
**              
**		Return values: 
**		Разбивает период на года
**		Called by:   
**              
**		Parameters:
**		Input								Output
**      ----------							-----------
**		Дата начала периода
**		Дата конца периода
**		
**		
**		Auth: Sergey Pelevin
**		Date: 05.08.14 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:		Description:
**		--------	--------	-------------------------------------------
**		06.08.14	SAP			Оптимизация
*******************************************************************************/
CREATE FUNCTION fnReportSplitPeriodOnYears
(
	@PeriodStart date,
	@PeriodEnd date
)
RETURNS @Years TABLE
(
	[StartDate] date,
	[EndDate] date
)
AS
BEGIN
	DECLARE @Numbers TABLE
    (
		[NumberOfYear] tinyint
	)

	INSERT INTO @Numbers
        SELECT 0 UNION ALL
        SELECT 1 UNION ALL
        SELECT 2 UNION ALL
        SELECT 3 UNION ALL
        SELECT 4 UNION ALL
		SELECT 5 UNION ALL
		SELECT 6 UNION ALL
		SELECT 7 UNION ALL
		SELECT 8 UNION ALL
		SELECT 9 UNION ALL
		SELECT 10 UNION ALL
		SELECT 11 UNION ALL
		SELECT 12 UNION ALL
		SELECT 13 UNION ALL
		SELECT 14 UNION ALL
		SELECT 15 UNION ALL
		SELECT 16 UNION ALL
		SELECT 17 UNION ALL
		SELECT 18 UNION ALL
		SELECT 19 UNION ALL
		SELECT 20 UNION ALL
		SELECT 21 UNION ALL
		SELECT 22 UNION ALL
		SELECT 23 UNION ALL
		SELECT 24 UNION ALL
		SELECT 25 UNION ALL
		SELECT 26 UNION ALL
		SELECT 27 UNION ALL
		SELECT 28 UNION ALL
		SELECT 29 UNION ALL
		SELECT 30 


	INSERT INTO @Years	
	SELECT	
		CASE WHEN (DateAdd(Year, DateDiff(Year, 0, @PeriodStart) + [NumberOfYear] ,0)) <= @PeriodStart
			THEN @PeriodStart
			ELSE DateAdd(Year, DateDiff(Year, 0, @PeriodStart) + [NumberOfYear] ,0)
		END AS [StartDate],

		CASE WHEN (DateAdd(Day, -1, DateAdd(Year, DateDiff(Year, 0, @PeriodStart) + [NumberOfYear] + 1 ,0))) > @PeriodEnd
			THEN @PeriodEnd
			ELSE DateAdd(Day, -1, DateAdd(Year, DateDiff(Year, 0, @PeriodStart) + [NumberOfYear] + 1 ,0))
		END AS [EndDate]	
	FROM		 
		@Numbers
	WHERE
		datediff(Y, @PeriodStart, @PeriodEnd) >= [NumberOfYear]
	ORDER BY 
		[StartDate], 
		[NumberOfYear]

	DELETE FROM @Years
	WHERE 
		[StartDate] >= [EndDate]


RETURN
END
GO