IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm5trBusyEntryInsert')
	BEGIN
		PRINT 'Dropping Trigger pjm5trBusyEntryInsert'
		DROP  Trigger pjm5trBusyEntryInsert
	END
GO

/******************************************************************************
**		File: 
**		Name: pjm5trBusyEntryInsert
**		Desc: 
**
**		This template can be customized:
**              
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**		--------		--------				-------------------------------------------
**    
*******************************************************************************/

PRINT 'Creating Trigger pjm5trBusyEntryInsert'
GO
CREATE Trigger pjm5trBusyEntryInsert 
ON [BusyEntry] FOR INSERT, UPDATE
AS
BEGIN
	DECLARE @value TVariant, @p TCount
	EXEC adfParameterGet @Name='WorkTime', @Group='Pjm5', @Value = @value OUTPUT
	SET @p = CAST(@value as int)
	
	INSERT INTO [EmployeeCalendar] ([EmployeeID], [Date], [Working], [Planned], [Approved], [Actual])
	SELECT DISTINCT
		inserted.[EmployeeID],
		inserted.[Date],
		@p,
		0,
		0,
		0
	FROM
		inserted
		INNER JOIN [Task]
			ON [Task].[ID] = inserted.[TaskID]
	WHERE
		[Task].[State] != 7 AND
		inserted.[EmployeeID] IS NOT NULL AND
		NOT EXISTS (
			SELECT * FROM [EmployeeCalendar]
			WHERE [EmployeeID] = inserted.[EmployeeID] AND [Date] = inserted.[Date]
			)
	GROUP BY
		inserted.[EmployeeID],
		inserted.[Date],
		[Task].[State]

	IF @@error != 0
	BEGIN
		RAISERROR('#cannotchangeemployeecalendar', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	;WITH [DateBusyEntry] ([EmployeeID], [Date], [Planned], [Approved]) AS
	(
		SELECT
			inserted.[EmployeeID],
			inserted.[Date],
			SUM(CASE 
					WHEN [BusyEntry].[Planned] > [BusyEntry].[Canceled] 
						THEN [BusyEntry].[Planned] - [BusyEntry].[Canceled]
					ELSE 0
				END), 
			SUM(CASE 
				WHEN [Task].[State] IN (2, 3, 5, 6) AND [BusyEntry].[Planned] > [BusyEntry].[Canceled]
					THEN ([BusyEntry].[Planned] - [BusyEntry].[Canceled])
				ELSE 0
			END)
		FROM
			inserted
			INNER JOIN [BusyEntry]
				ON [BusyEntry].[EmployeeID] = inserted.[EmployeeID] AND
					[BusyEntry].[Date] = inserted.[Date]
			INNER JOIN [Task]
				ON [Task].[ID] = [BusyEntry].[TaskID]
		GROUP BY
			inserted.[EmployeeID],
			inserted.[Date]
	)
	UPDATE
		[EmployeeCalendar]
	SET
		[Planned] = ISNULL([DateBusyEntry].[Planned], 0),
		[Approved] = ISNULL([DateBusyEntry].[Approved], 0)
	FROM
		[EmployeeCalendar]
		INNER JOIN [DateBusyEntry]
			ON ([EmployeeCalendar].[EmployeeID] = [DateBusyEntry].[EmployeeID] AND 
				[EmployeeCalendar].[Date] = [DateBusyEntry].[Date])
	IF @@error != 0
	BEGIN
		RAISERROR('#cannotchangeemployeecalendar', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	DELETE [BusyEntry]
	FROM
		[BusyEntry]
		INNER JOIN inserted
			ON inserted.[ProjectID] = [BusyEntry].[ProjectID] AND
				inserted.[TaskID] = [BusyEntry].[TaskID] AND
				inserted.[AssignmentID] = [BusyEntry].[AssignmentID] AND
				inserted.[EmployeeID] = [BusyEntry].[EmployeeID] AND
				inserted.[Date] = [BusyEntry].[Date]		
	WHERE
		inserted.[Planned] <= 0 AND inserted.[ActualWork] = 0 AND inserted.[ActualOvertimeWork] = 0 
	IF @@error != 0
	BEGIN
		RAISERROR('#cannotchangeemployeecalendar', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END
END

GO

IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm5trBusyEntryUpdate')
	BEGIN
		PRINT 'Dropping Trigger pjm5trBusyEntryUpdate'
		DROP  Trigger pjm5trBusyEntryUpdate
	END
GO
/******************************************************************************
**		File: 
**		Name: pjm5trBusyEntryUpdate
**		Desc: 
**
**		This template can be customized:
**              
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**		--------		--------				-------------------------------------------
**    
*******************************************************************************/
PRINT 'Creating Trigger pjm5trBusyEntryUpdate'
GO
CREATE Trigger pjm5trBusyEntryUpdate
ON [BusyEntry] FOR UPDATE
AS
BEGIN
	
	DELETE [BusyEntry]
	FROM
		[BusyEntry]
		INNER JOIN inserted
			ON inserted.[ProjectID] = [BusyEntry].[ProjectID] AND
				inserted.[TaskID] = [BusyEntry].[TaskID] AND
				inserted.[AssignmentID] = [BusyEntry].[AssignmentID] AND
				inserted.[EmployeeID] = [BusyEntry].[EmployeeID] AND
				inserted.[Date] = [BusyEntry].[Date]		
	WHERE
		inserted.[Planned] = 0 AND 
		inserted.[ActualWork] = 0 AND 
		inserted.[ActualOvertimeWork] = 0 
	
	;WITH [DateBusyEntry] ([EmployeeID], [Date], [Planned], [Approved]) AS
	(
		SELECT
			inserted.[EmployeeID],
			inserted.[Date],
			SUM(CASE 
					WHEN [BusyEntry].[Planned] > [BusyEntry].[Canceled] 
						THEN [BusyEntry].[Planned] - [BusyEntry].[Canceled]
					ELSE 0
				END), 
			SUM(CASE 
				WHEN [Task].[State] IN (2, 3, 5, 6) AND [BusyEntry].[Planned] > [BusyEntry].[Canceled]
					THEN ([BusyEntry].[Planned] - [BusyEntry].[Canceled])
				ELSE 0
			END)
		FROM
			inserted
			INNER JOIN [BusyEntry]
				ON [BusyEntry].[EmployeeID] = inserted.[EmployeeID] AND
					[BusyEntry].[Date] = inserted.[Date]
			INNER JOIN [Task]
				ON [Task].[ID] = [BusyEntry].[TaskID]
		GROUP BY
			inserted.[EmployeeID],
			inserted.[Date]
		UNION
		SELECT
			deleted.[EmployeeID],
			deleted.[Date],
			SUM(CASE 
					WHEN [BusyEntry].[Planned] > [BusyEntry].[Canceled] 
						THEN [BusyEntry].[Planned] - [BusyEntry].[Canceled]
					ELSE 0
				END), 
			SUM(CASE 
				WHEN [Task].[State] IN (2, 3, 5, 6) AND [BusyEntry].[Planned] > [BusyEntry].[Canceled]
					THEN ([BusyEntry].[Planned] - [BusyEntry].[Canceled])
				ELSE 0
			END)
		FROM
			deleted
			INNER JOIN [BusyEntry]
				ON [BusyEntry].[EmployeeID] = deleted.[EmployeeID] AND
					[BusyEntry].[Date] = deleted.[Date]
			INNER JOIN [Task]
				ON [Task].[ID] = [BusyEntry].[TaskID]
		GROUP BY
			deleted.[EmployeeID],
			deleted.[Date]
	)
	UPDATE
		[EmployeeCalendar]
	SET
		[Planned] = ISNULL([DateBusyEntry].[Planned], 0),
		[Approved] = ISNULL([DateBusyEntry].[Approved], 0)
	FROM
		[EmployeeCalendar]
		INNER JOIN [DateBusyEntry]
			ON ([EmployeeCalendar].[EmployeeID] = [DateBusyEntry].[EmployeeID] AND 
				[EmployeeCalendar].[Date] = [DateBusyEntry].[Date])
	IF @@error != 0
	BEGIN
		RAISERROR('#cannotchangeemployeecalendar', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END	
		
END
GO

IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm5trBusyEntryDelete')
	BEGIN
		PRINT 'Dropping Trigger pjm5trBusyEntryDelete'
		DROP  Trigger pjm5trBusyEntryDelete
	END
GO

/******************************************************************************
**		File: 
**		Name: pjm5trBusyEntryDelete
**		Desc: 
**
**		This template can be customized:
**              
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**		--------		--------				-------------------------------------------
**    
*******************************************************************************/

PRINT 'Creating Trigger pjm5trBusyEntryDelete'
GO
CREATE Trigger pjm5trBusyEntryDelete 
ON [BusyEntry] FOR DELETE
AS
BEGIN
	UPDATE	
		[EmployeeCalendar]
	SET
		[Approved] = CASE 
			WHEN [EmployeeCalendar].[Approved] - (deleted.[Planned] - deleted.[Canceled]) > 0
				THEN [EmployeeCalendar].[Approved] - (deleted.[Planned] - deleted.[Canceled]) 
			ELSE 0
		END
	FROM
		[EmployeeCalendar]
		INNER JOIN deleted
			ON ([EmployeeCalendar].[EmployeeID] = deleted.[EmployeeID] AND 
				[EmployeeCalendar].[Date] = deleted.[Date])
		INNER JOIN [Task]
			ON [Task].[ID] = deleted.[TaskID]
	WHERE
		[Task].[State] NOT IN (7, 8)
	IF @@error != 0
	BEGIN
		RAISERROR('#cannotchangeemployeecalendar', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	UPDATE	
		[EmployeeCalendar]
	SET
		[Planned] = CASE 
			WHEN [EmployeeCalendar].[Planned] - (deleted.[Planned] - deleted.[Canceled]) > 0
				THEN [EmployeeCalendar].[Planned] - (deleted.[Planned] - deleted.[Canceled])
			ELSE 0
		END
	FROM
		[EmployeeCalendar]
		INNER JOIN deleted
			ON ([EmployeeCalendar].[EmployeeID] = deleted.[EmployeeID] AND 
				[EmployeeCalendar].[Date] = deleted.[Date])
		INNER JOIN [Task]
			ON [Task].[ID] = deleted.[TaskID]
	WHERE
		[Task].[State] != 7		
	IF @@error != 0
	BEGIN
		RAISERROR('#cannotchangeemployeecalendar', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END
END

GO

