﻿IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm7DocumentChangeBillableFlag]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm7DocumentChangeBillableFlag'
		DROP PROCEDURE [dbo].[pjm7DocumentChangeBillableFlag]
	END
GO

PRINT 'Creating procedure pjm7DocumentChangeBillableFlag'
GO

/******************************************************************************
**		File: pjm7DocumentChangeBillableFlags.sql
**		Name: pjm7DocumentChangeBillableFlags
**		Desc: 
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: Sergey Pelevin
**		Date: 25.06.14
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		01.07.14	SAP				Добавлена проверка на состояние документа
*******************************************************************************/
CREATE PROCEDURE pjm7DocumentChangeBillableFlag
	@DocumentClass TClass,
	@DocumentID TIdentifier,
	@BillableFlagNewValue bit
AS
	SET NOCOUNT ON

	DECLARE 
		@TimeEntryClass [TClass] = [dbo].[adfGetClassByName]('Pjm5.TimeEntry'),
		@ExpenseClass [TClass] = [dbo].[adfGetClassByName]('Pjm5.Expense'),
		@State TState
		
		IF @DocumentClass <> @TimeEntryClass AND @DocumentClass <> @ExpenseClass
		BEGIN
			RAISERROR('#DocumentHasNoBillableFlag', 18, -1)
			RETURN 1
		END

		SELECT
			@State = [State]
		FROM
			[Document]
		WHERE 
			[Class] = @DocumentClass
			AND
			[ID] = @DocumentID

		IF @State IN (4,6)
		BEGIN
			RAISERROR('#CannotEditDocumentInCurrentState', 18, -1)
			RETURN 1
		END

		BEGIN TRANSACTION
		
		--добавление необходимых прав
		INSERT INTO [Right] ([Name],[DisplayName],[Group]) VALUES('RemoveBillableFlag', '', '')
		EXEC pjm6SetRightForMasters 'RemoveBillableFlag'

		IF @DocumentClass = @TimeEntryClass
		BEGIN
			UPDATE [TimeEntry]
			SET [Billable] = @BillableFlagNewValue
			WHERE [ID] = @DocumentID
		END
		ELSE IF @DocumentClass = @ExpenseClass
		BEGIN
			UPDATE [Expense]
			SET [Billable] = @BillableFlagNewValue
			WHERE [ID] = @DocumentID
		END

		--удаление дарованных в ходе процедуры прав
		EXEC pjm6DelRightForMasters 'RemoveBillableFlag'
		DELETE [Right] WHERE [Name] = 'RemoveBillableFlag'

		IF @@error <> 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END

		COMMIT TRANSACTION

	RETURN 0
GO

GRANT EXECUTE ON pjm7DocumentChangeBillableFlag TO PUBLIC
GO