IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm5CaseUpdate]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm5CaseUpdate'
		DROP PROCEDURE [dbo].[pjm5CaseUpdate]
	END
GO

PRINT 'Creating procedure pjm5CaseUpdate'
GO

SET QUOTED_IDENTIFIER ON
GO
/******************************************************************************
**		File: pjm5CaseUpdate.sql
**		Name: pjm5CaseUpdate
**		Desc: 
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: MAV
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		27.03.13	SAP				  OPENXML	
*******************************************************************************/
CREATE PROCEDURE pjm5CaseUpdate
	@Xml XML
AS
	SET NOCOUNT ON
	DECLARE
		@ID TIdentifier
	
	DECLARE 
		@FileAs TName,
		@Code TCode,
		@CompanyID TIdentifier,
		@PrevCompanyID TIdentifier,
		@EmployeeID TIdentifier,
		@StartDate TDate,
		@EndDate TDate,
		@NumberTemplate TName,
		@CurrentNumber TCount,
		@Increment TCount,
		@AllowInternal TBool,
		@AllowIncoming TBool,
		@AllowOutgoing TBool,
		@AllowFolder TBool,
		@HasToBeSentBack TBool,
		@Comments TNotes,
		@DocumentCount int,
		@WordTemplatesFolderID TIdentifier,
		@WordTemplatesPath TNotes
		
	SELECT
		@ID = T.c.value('ID[1]','int'),
		@FileAs = T.c.value('FileAs[1]', 'nvarchar(512)'),
		@Code = T.c.value('Code[1]','nvarchar(32)'),
		@CompanyID = T.c.value('CompanyID[1]','int'),
		@EmployeeID = T.c.value('EmployeeID[1]','int'),
		@StartDate = CAST(T.c.value('StartDate[1]','varchar(19)') as datetime),
		@EndDate = CAST(T.c.value('EndDate[1]','varchar(19)') as datetime),
		@NumberTemplate  = T.c.value('NumberTemplate[1]','nvarchar(512)'),
		@CurrentNumber = T.c.value('CurrentNumber[1]','int'),
		@Increment = T.c.value('Increment[1]','int'),
		@AllowInternal = CASE WHEN T.c.value('AllowInternal[1]','varchar(5)') = 'false' THEN 0 ELSE 1 END,
		@AllowIncoming = CASE WHEN T.c.value('AllowIncoming[1]','varchar(5)') = 'false' THEN 0 ELSE 1 END,
		@AllowOutgoing = CASE WHEN T.c.value('AllowOutgoing[1]','varchar(5)') = 'false' THEN 0 ELSE 1 END,
		@AllowFolder = CASE WHEN T.c.value('AllowFolder[1]','varchar(5)') = 'false' THEN 0 ELSE 1 END,
		@HasToBeSentBack  = CASE WHEN T.c.value('HasToBeSentBack[1]','varchar(5)') = 'false' THEN 0 ELSE 1 END,
		@Comments = T.c.value('Comments[1]','nvarchar(max)'),
		@WordTemplatesFolderID = T.c.value('WordTemplatesFolderID[1]','int'),
		@WordTemplatesPath = T.c.value('WordTemplatesPath[1]','nvarchar(max)'),
		@PrevCompanyID = [Case].[CompanyID]		
	FROM 
		@Xml.nodes('/pjm5CaseGet/Case') AS T(c)		
		INNER JOIN [Case]
			ON [Case].[ID] = T.c.value('ID[1]','int')
	
	SET @DocumentCount = 0
	IF EXISTS(SELECT * FROM [DocumentCard] WHERE [CaseID] = @ID)
		SET @DocumentCount = 1
	
	BEGIN TRANSACTION
	UPDATE 
		[Case]
	SET
		[Code] = @Code,
		[FileAs] = @FileAs,
		[CompanyID] = @CompanyID,
		[EmployeeID] = @EmployeeID,
		[StartDate] = @StartDate,
		[EndDate] = @EndDate,
		[NumberTemplate] = @NumberTemplate,
		[CurrentNumber] = @CurrentNumber,
		[Increment] = @Increment,
		[AllowInternal] = CASE @DocumentCount WHEN 1 THEN [AllowInternal] ELSE @AllowInternal END,
		[AllowIncoming] = CASE @DocumentCount WHEN 1 THEN [AllowIncoming] ELSE @AllowIncoming END,
		[AllowOutgoing] = CASE @DocumentCount WHEN 1 THEN [AllowOutgoing] ELSE @AllowOutgoing END,
		[AllowFolder] = @AllowFolder,
		[HasToBeSentBack] = @HasToBeSentBack,
		[Comments] = @Comments,
		[WordTemplatesFolderID] = @WordTemplatesFolderID,
		[WordTemplatesPath] = @WordTemplatesPath
	WHERE
		[ID] = @ID
	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END
	
	IF @AllowFolder = 0
	BEGIN
		UPDATE
			[CaseOfFolder]
		SET
			[Allowed] = 0
		WHERE
			[CaseID] = @ID AND
			[Allowed] = 1
		IF @@ERROR != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END
	END
	
	IF @PrevCompanyID != @CompanyID
	BEGIN
		UPDATE
			[CaseOfFolder]
		SET
			[FolderID] = [Folder].[ID],
			[Allowed] = CASE WHEN ISNULL(T.C.value('.', 'int'), 0) = 0 THEN 1 ELSE [CaseOfFolder].[Allowed] END
		FROM
			[CaseOfFolder]
			INNER JOIN [Case]
				ON [Case].[ID] = [CaseOfFolder].[CaseID]
			INNER JOIN [Folder]
				ON [Folder].[CompanyID] = [Case].[CompanyID]
			OUTER APPLY [Folder].[Parameters].nodes('/Parameters/AllowClassFlag') T(C)
		WHERE
			[CaseOfFolder].[CaseID] = @ID
		IF @@ERROR != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END
	END
	
	COMMIT TRANSACTION
	EXEC pjm5CaseGet @ID

	RETURN 0
GO

GRANT EXECUTE ON pjm5CaseUpdate TO PUBLIC
GO
