﻿IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm7FactureBrowseByCompany]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm7FactureBrowseByCompany'
		DROP PROCEDURE [dbo].[pjm7FactureBrowseByCompany]
	END
GO

PRINT 'Creating procedure pjm7FactureBrowseByCompany'
GO

/******************************************************************************
**		File: pjm7FactureBrowseByCompany.sql
**		Name: pjm7FactureBrowseByCompany
**		Desc: 
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: SAI
**		Date: 02/05/2013 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/

CREATE PROCEDURE [dbo].[pjm7FactureBrowseByCompany] 
	@ID int,
	@StateSelector int,
	@DateSelector int,
	@PeriodStart datetime,
	@PeriodEnd datetime	
AS		
BEGIN
	DECLARE
		@EmployeeID TIdentifier,
		@ViewAllProjects TBool

	SET @ViewAllProjects = dbo.adfHasAccessRight(dbo.adfGetRightID('ViewAllProjects'))
	SET @EmployeeID = dbo.adfGetCurrentEmployeeID()
	
	IF @DateSelector != 0
		EXEC BillingFilterPeriodGet @DateSelector, @PeriodStart OUTPUT, @PeriodEnd OUTPUT
		
	SELECT
		[Facture].[Class],
		[Facture].[ID],
		[Facture].[State],
		[Facture].[Flag],
		[Facture].[FileAs],
		[Facture].[Date],
		[Facture].[Number],
		[Facture].[ProjectID],
		[Facture].[SupplierCompanyID],
		[Facture].[SupplierAddress],
		[Facture].[SupplierINN],
		[Facture].[SupplierKPP],
		[Facture].[CustomerCompanyID],
		[Facture].[CustomerAddress],
		[Facture].[CustomerINN],
		[Facture].[CustomerKPP],
		[Facture].[Sender],
		[Facture].[Recipient],
		[Facture].[DocumentNumber],
		[Facture].[DocumentDate],
		[Facture].[Chief],
		[Facture].[Accountant],
		[Facture].[Created],
		[Facture].[CreatedBy],
		[Facture].[Modified],
		[Facture].[ModifiedBy],
		[Facture].[Comments],
		[Facture].[Timestamp],
		[Facture].[Advance],
		[Facture].[CurrencyCode],
		[Facture].[Total],
		[Facture].[VAT],
		[Facture].[TotalWithVAT],
		[Facture].[TotalWithVATPC] AS [ProjectTotal],
		[Project].[CurrencyCode] AS [ProjectCurrencyCode],
		[Project].[ProjectManagerEmployeeID],
		[Project].[CostCenterID],
		[Company].[ManagerEmployeeID],
		[Facture].[ExchangeBanned]
	FROM
		[Facture]
		INNER JOIN [Project]
			ON [Project].[ID] = [Facture].[ProjectID]
		LEFT OUTER JOIN [Company]
			ON [Project].[CustomerCompanyID] = [Company].[ID]
	WHERE
		([Project].[SupplierCompanyID] = @ID OR
			[Project].[CustomerCompanyID] = @ID) AND
		(@DateSelector = 0 OR [Facture].[Date] BETWEEN @PeriodStart AND @PeriodEnd) AND
		(
			(@StateSelector = 0) OR 
			(@StateSelector = 4 AND [Facture].[State] != 3) OR
			(@StateSelector != 0 AND [Facture].[State] = @StateSelector)
		) AND
		(dbo.fnPjm5HasFinansDataAccess([Project].[ID], @EmployeeID, NULL, NULL) = 1)
		
	RETURN 0
END

GO

GRANT EXECUTE ON [pjm7FactureBrowseByCompany] TO PUBLIC
GO
