IF  EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'pjm5trFolderInsert'))
BEGIN
	PRINT 'Dropping Trigger [pjm5trFolderInsert]'
	DROP TRIGGER [pjm5trFolderInsert]
END
GO

PRINT 'Creating Trigger [pjm5trFolderInsert]'
GO

CREATE TRIGGER [pjm5trFolderInsert] 
ON [Folder] FOR INSERT
AS
	SET NOCOUNT ON
	;WITH [SubfolderCount] ([FolderID], [Count]) AS (
		SELECT 
			[FolderID], COUNT(*)
		FROM 
			[inserted] 
		WHERE 
			[FolderID] IS NOT NULL 
		GROUP BY 
			[FolderID]
	)
	UPDATE [Folder]
	SET
		[SubfolderCount] = [SubfolderCount] + [SubfolderCount].[Count],
		[Modified] = DEFAULT,
		[ModifiedBy] = DEFAULT
	FROM
		[Folder] 
		INNER JOIN [SubfolderCount]
			ON [Folder].[ID] = [SubfolderCount].[FolderID]
	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN
	END
	RETURN
GO

IF  EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'pjm5trFolderDelete'))
BEGIN
	PRINT 'Dropping Trigger [pjm5trFolderDelete]'
	DROP TRIGGER [pjm5trFolderDelete]
END
GO

PRINT 'Creating Trigger [pjm5trFolderDelete]'
GO

CREATE TRIGGER [pjm5trFolderDelete] 
ON [Folder] FOR DELETE
AS
	SET NOCOUNT ON
	;WITH [SubfolderCount] ([FolderID], [Count]) AS (
		SELECT 
			[FolderID], COUNT(*)
		FROM 
			[deleted] 
		WHERE 
			[FolderID] IS NOT NULL 
		GROUP BY 
			[FolderID]
	)
	UPDATE [Folder]
	SET
		[SubfolderCount] = [SubfolderCount] - [SubfolderCount].[Count],
		[Modified] = DEFAULT,
		[ModifiedBy] = DEFAULT
	FROM
		[Folder] 
		INNER JOIN [SubfolderCount]
			ON [Folder].[ID] = [SubfolderCount].[FolderID]
	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN
	END
	RETURN
GO 

IF  EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'pjm5trFolderUpdate'))
BEGIN
	PRINT 'Dropping Trigger [pjm5trFolderUpdate]'
	DROP TRIGGER [pjm5trFolderUpdate]
END
GO

PRINT 'Creating Trigger [pjm5trFolderUpdate]'
GO

CREATE TRIGGER [pjm5trFolderUpdate] 
ON [Folder] FOR UPDATE
AS
	SET NOCOUNT ON
	IF (UPDATE(FolderID))
	BEGIN
		;WITH [SubfolderCount] ([FolderID], [Count]) AS (
			SELECT 
				[FolderID], COUNT(*)
			FROM 
				[deleted] 
			WHERE 
				[FolderID] IS NOT NULL 
			GROUP BY 
				[FolderID]
		)
		UPDATE [Folder]
		SET
			[SubfolderCount] = [SubfolderCount] - [SubfolderCount].[Count],
			[Modified] = DEFAULT,
			[ModifiedBy] = DEFAULT
		FROM
			[Folder] 
			INNER JOIN [SubfolderCount]
				ON [Folder].[ID] = [SubfolderCount].[FolderID]
		IF @@ERROR != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN
		END

		;WITH [SubfolderCount] ([FolderID], [Count]) AS (
			SELECT 
				[FolderID], COUNT(*)
			FROM 
				[inserted] 
			WHERE 
				[FolderID] IS NOT NULL 
			GROUP BY 
				[FolderID]
		)
		UPDATE [Folder]
		SET
			[SubfolderCount] = [SubfolderCount] + [SubfolderCount].[Count],
			[Modified] = DEFAULT,
			[ModifiedBy] = DEFAULT
		FROM
			[Folder] 
			INNER JOIN [SubfolderCount]
				ON [Folder].[ID] = [SubfolderCount].[FolderID]
		IF @@ERROR != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN
		END
	END
	RETURN
GO  