IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm6trMeetingRequestAttendeeDelete')
	BEGIN
		PRINT 'Dropping Trigger pjm6trMeetingRequestAttendeeDelete'
		DROP  Trigger pjm6trMeetingRequestAttendeeDelete
	END
GO

/******************************************************************************
**		File: 
**		Name: pjm6trMeetingRequestAttendeeDelete
**		Desc: 
**
**		This template can be customized:
**              
**
**		Auth: PAM
**		Date: 22.12.09
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**		--------		--------				-------------------------------------------
**    
*******************************************************************************/

PRINT 'Creating Trigger pjm6trMeetingRequestAttendeeDelete'
GO
CREATE TRIGGER pjm6trMeetingRequestAttendeeDelete 
ON [Attendee] FOR DELETE
AS
BEGIN
    IF @@rowcount = 0
       RETURN
	SET NOCOUNT ON
	DECLARE @CurrentUserID TIdentifier
	SET @CurrentUserID = dbo.adfGetCurrentUserID()
	
	DECLARE @user TIdentifier, @meetingrequest TIdentifier, @rv int
	DECLARE users CURSOR FAST_FORWARD FOR
		SELECT
			[deleted].[MeetingRequestID], [Employee].[UserID]
		FROM
			[deleted]
			INNER JOIN [MeetingRequest]
				ON [MeetingRequest].[ID] = [deleted].[MeetingRequestID]
			INNER JOIN [Employee]
				ON [deleted].[EmployeeID] = [Employee].[ID]
		WHERE
			[MeetingRequest].[State] = 2 AND
			[Employee].[UserID] IS NOT NULL

	OPEN users
	FETCH NEXT FROM users INTO @meetingrequest, @user
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXEC pjm6MeetingRequestAttendeeDeleteNotification @meetingrequest, @user
		FETCH NEXT FROM users INTO @meetingrequest, @user
	END
	CLOSE users
	DEALLOCATE users
END
GO

IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm6trMeetingRequestAttendeeInsert')
	BEGIN
		PRINT 'Dropping Trigger pjm6trMeetingRequestAttendeeInsert'
		DROP  Trigger pjm6trMeetingRequestAttendeeInsert
	END
GO

/******************************************************************************
**		File: 
**		Name: pjm6trMeetingRequestAttendeeInsert
**		Desc: 
**
**		This template can be customized:
**              
**
**		Auth: PAM
**		Date: 22.12.09
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**		--------		--------				-------------------------------------------
**		25.07.13	MAB					      
*******************************************************************************/

PRINT 'Creating Trigger pjm6trMeetingRequestAttendeeInsert'
GO
CREATE TRIGGER pjm6trMeetingRequestAttendeeInsert 
ON [Attendee] FOR INSERT
AS
BEGIN
    IF @@rowcount = 0
       RETURN
	SET NOCOUNT ON
	DECLARE @CurrentUserID TIdentifier
	SET @CurrentUserID = dbo.adfGetCurrentUserID()

	DECLARE 
		@value							TVariant,
		@ExchangeSyncMeetingRequests	TBool,
		@LocalNowDateTime				TDate,
		@UserSyncMeetings				bit,
		@UserSyncMeetingsInFuture		int,
		@OriginatorEmployeeID			TIdentifier,
		@OriginatorUserID				TIdentifier

	--   
	EXEC adfParameterGet @Name='ExchangeSyncMeetingRequests', @Group='Pjm6', @Value = @value OUTPUT
	SET @ExchangeSyncMeetingRequests = CAST(@value as bit)

	SET @LocalNowDateTime = dbo.fnPjm7GetLocalDate()
	
	DECLARE 
		@user						TIdentifier, 
		@meetingrequest				TIdentifier,
		@rv							int,
		@EmployeeID					TIdentifier,
		@ClassMeetingRequest		TClass,
		@StartTime					TDatetime,
		@MeetingRequestID			TIdentifier,
		@SyncForEmployee			TBool,
		@MeetingRequestUserID		TIdentifier

	SELECT
		@OriginatorEmployeeID	= [MeetingRequest].[OriginatorEmployeeID],
		@OriginatorUserID		= [Employee].[UserID],
		@ClassMeetingRequest	= [MeetingRequest].[Class],
		@StartTime				= [MeetingRequest].[StartTime],
		@MeetingRequestID		= [MeetingRequest].[ID]
	FROM
		[MeetingRequest]
		INNER JOIN [inserted]
			ON	[MeetingRequest].[ID] = [inserted].[MeetingRequestID]
		INNER JOIN [Employee]
			ON [MeetingRequest].[OriginatorEmployeeID] = [Employee].[ID]

	IF @ExchangeSyncMeetingRequests = 1 AND @StartTime > @LocalNowDateTime AND EXISTS (	SELECT 
																						*	
																					FROM
																						[ExchangeDocumentSync]
																					WHERE
																						[DocumentClass]	= @ClassMeetingRequest AND
																						[DocumentID] = @MeetingRequestID AND
																						[UserID] = @OriginatorUserID)
		SET @SyncForEmployee = 0
	ELSE
		SET @SyncForEmployee = 1															

	DECLARE users CURSOR FAST_FORWARD FOR
		SELECT
			[inserted].[MeetingRequestID], 
			[Employee].[UserID],
			[inserted].[EmployeeID]
		FROM
			[inserted]
			INNER JOIN [MeetingRequest]
				ON [MeetingRequest].[ID] = [inserted].[MeetingRequestID]
			INNER JOIN [Employee]
				ON [inserted].[EmployeeID] = [Employee].[ID]
		WHERE
			[MeetingRequest].[State] = 2 AND
			[Employee].[UserID] IS NOT NULL

	OPEN users
	FETCH NEXT FROM users INTO @meetingrequest, @user, @EmployeeID
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXEC pjm6MeetingRequestAttendeeAppointNotification @meetingrequest, @user

		IF @SyncForEmployee = 1
		BEGIN
			EXEC pjm7MeetingRequestAutoSyncExchange @meetingrequest, @ClassMeetingRequest, @user, @EmployeeID
		END
		FETCH NEXT FROM users INTO @meetingrequest, @user, @EmployeeID
	END
	CLOSE users
	DEALLOCATE users
END
GO