IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm5NotificationEventGet]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm5NotificationEventGet'
		DROP PROCEDURE [dbo].[pjm5NotificationEventGet]
	END
GO

PRINT 'Creating procedure pjm5NotificationEventGet'
GO

/******************************************************************************
**		File: pjm5NotificationEventGet.sql
**		Name: pjm5NotificationEventGet
**		Desc: 
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 20/12/2005 13:00:00
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    13.05.10		SYV				   
**									  pjm6DismissOldEvents
**    24.08.10		SYV				  pjm6DismissOldEvents .. 
**									    
**    08.11.13      SNP               pjm6DismissOldEvents
*******************************************************************************/
CREATE PROCEDURE [pjm5NotificationEventGet]
	@UserID TIdentifier,
	@Now TDate
AS
	SET NOCOUNT ON
	
	IF @UserID IS NULL
	BEGIN
		SET @UserID = dbo.adfGetCurrentUserID()
	END
	
	IF @UserID IS NULL
	BEGIN
		RETURN 0
	END
	
	EXEC pjm6DismissOldEvents @UserID
	
	DECLARE
		@IssueClass				TClass,
		@ServiceRequestClass	TClass,
		@ProjectClass			TClass,
		@InvoiceClass			TClass,
		@StatementClass			TClass,
		@TaskClass				TClass,
		@TaskRequestClass		TClass,
		@EmployeeClass			TClass,
		@InitialEventState		TState,
		@AlertedEventState		TState
		
	SELECT
		@IssueClass = dbo.adfGetClassByName('Pjm5.Issue'),
		@ServiceRequestClass = dbo.adfGetClassByName('Pjm5.ServiceRequest'),
		@ProjectClass = dbo.adfGetClassByName('Pjm5.Project'),
		@InvoiceClass = dbo.adfGetClassByName('Pjm5.Invoice'),
		@StatementClass = dbo.adfGetClassByName('Pjm5.Statement'),
		@TaskClass = dbo.adfGetClassByName('Pjm5.Task'),
		@TaskRequestClass = dbo.adfGetClassByName('Pjm5.TaskRequest'),
		@EmployeeClass = dbo.adfGetClassByName('Adf.Employee'),
		@InitialEventState = dbo.fnPjm5GetInitialEventState(),
		@AlertedEventState = dbo.fnPjm5GetAlertedEventState()
	
	SELECT
		CAST(1 as int) AS [EventType],
		NULL AS [TypeName],
		[Event].[ID],
		[Event].[SubscriptionTypeID],
		[SubscriptionType].[Aggregatable],
		[Event].[DocumentClass],
		[Event].[DocumentID],
		[Event].[ContainerClass],
		[Event].[ContainerID],
		[Event].[AlertTime],
		[SubscriptionRuleNotificationWay].[SubscriptionRecurrencePatternID],
		[Document].[FileAs] AS [DocumentFileAs],
		NULL AS [Description],
		CASE
			WHEN [Document].[Class] = @IssueClass THEN
			(
				SELECT [Number] FROM [Issue]
				WHERE [Issue].[ID] = [Document].[ID]
			)
			WHEN [Document].[Class] = @ServiceRequestClass THEN
			(
				SELECT [Number] FROM [ServiceRequest]
				WHERE [ServiceRequest].[ID] = [Document].[ID]
			)
			WHEN [Document].[Class] = @ProjectClass THEN
			(
				SELECT [Number] FROM [Project]
				WHERE [Project].[ID] = [Document].[ID]
			)
			WHEN [Document].[Class] = @InvoiceClass THEN
			(
				SELECT [Number] FROM [Invoice]
				WHERE [Invoice].[ID] = [Document].[ID]
			)
			WHEN [Document].[Class] = @StatementClass THEN
			(
				SELECT [Number] FROM [Statement]
				WHERE [Statement].[ID] = [Document].[ID]
			)
			ELSE NULL
		END AS [DocumentNumber],
		CASE
			WHEN [ContainerTemplate].[ID] IS NULL THEN
				dbo.fnPjm5GetLocalSimpleTheme([GlobalTemplate].[ID], NULL)
			ELSE
				dbo.fnPjm5GetLocalSimpleTheme([ContainerTemplate].[ID], NULL)
		END AS [SimpleMessageTheme],
		CASE
			WHEN [ContainerTemplate].[ID] IS NULL THEN
				dbo.fnPjm5GetLocalAggregatedTheme([GlobalTemplate].[ID], NULL)
			ELSE
				dbo.fnPjm5GetLocalAggregatedTheme([ContainerTemplate].[ID], NULL)
		END AS [AggregatedMessageTheme],
		CASE
			WHEN [Event].[State] = @AlertedEventState THEN CAST(1 as bit)
			ELSE CAST(0 as bit)
		END AS [Alerted]
	FROM
		[Event]
		INNER JOIN [SubscriptionType]
			ON [SubscriptionType].[ID] = [Event].[SubscriptionTypeID]
		INNER JOIN [SubscriptionRule]
			ON [SubscriptionRule].[ID] = [Event].[SubscriptionRuleID]
		INNER JOIN [SubscriptionRuleNotificationWay]
			ON ([SubscriptionRuleNotificationWay].[SubscriptionRuleID] = [SubscriptionRule].[ID] AND
				[SubscriptionRuleNotificationWay].[NotificationWayID] = [Event].[NotificationWayID])
		INNER JOIN [Document]
			ON ([Document].[Class] = [Event].[DocumentClass] AND
				[Document].[ID] = [Event].[DocumentID])
		LEFT JOIN [NotificationMessageTemplate] AS [ContainerTemplate]
			ON ([ContainerTemplate].[SubscriptionTypeID] = [Event].[SubscriptionTypeID] AND
				[ContainerTemplate].[NotificationWayID] = [Event].[NotificationWayID] AND
				[ContainerTemplate].[DocumentClass] = [Event].[ContainerClass] AND
				[ContainerTemplate].[DocumentID] = [Event].[ContainerID])
		LEFT JOIN [NotificationMessageTemplate] AS [GlobalTemplate]
			ON ([GlobalTemplate].[SubscriptionTypeID] = [Event].[SubscriptionTypeID] AND
				[GlobalTemplate].[NotificationWayID] = [Event].[NotificationWayID] AND
				[GlobalTemplate].[DocumentClass] IS NULL AND
				[GlobalTemplate].[DocumentID] IS NULL)
	WHERE
		([SubscriptionRule].[UserID] = @UserID) AND
		([Event].[State] = @InitialEventState OR [Event].[State] = @AlertedEventState) AND
		([Event].[NotificationWayID] = 1)
		
	UNION ALL
	
	SELECT
		CAST(2 as int) AS [EventType],
		[ReminderType].[Name] AS [TypeName],
		[Reminder].[ID],
		NULL AS [SubscriptionTypeID],
		NULL AS [Aggregatable],
		[Reminder].[DocumentClass],
		[Reminder].[DocumentID],
		NULL AS [ContainerClass],
		NULL AS [ContainerID],
		CASE
			WHEN [ReminderType].[IsTimeFormat] = 1 THEN
				DATEADD(minute, -[UserReminder].[OriginalInterval], [Reminder].[EventTime])
			ELSE [Reminder].[EventTime]
		END AS [AlertTime],
		NULL AS [SubscriptionRecurrencePatternID],
		[Document].[FileAs] AS [DocumentFileAs],
		[UserReminder].[Description] AS [Description],
		CASE
			WHEN [Document].[Class] = @IssueClass THEN
			(
				SELECT [Number] FROM [Issue]
				WHERE [Issue].[ID] = [Document].[ID]
			)
			WHEN [Document].[Class] = @ServiceRequestClass THEN
			(
				SELECT [Number] FROM [ServiceRequest]
				WHERE [ServiceRequest].[ID] = [Document].[ID]
			)
			WHEN [Document].[Class] = @ProjectClass THEN
			(
				SELECT [Number] FROM [Project]
				WHERE [Project].[ID] = [Document].[ID]
			)
			WHEN [Document].[Class] = @InvoiceClass THEN
			(
				SELECT [Number] FROM [Invoice]
				WHERE [Invoice].[ID] = [Document].[ID]
			)
			WHEN [Document].[Class] = @StatementClass THEN
			(
				SELECT [Number] FROM [Statement]
				WHERE [Statement].[ID] = [Document].[ID]
			)
			ELSE NULL
		END AS [DocumentNumber],
		NULL AS [SimpleMessageTheme],
		NULL AS [AggregatedMessageTheme],
		CAST(0 as bit) AS [Alerted]
	FROM
		[UserReminder]
		INNER JOIN [Reminder]
			ON [Reminder].[ID] = [UserReminder].[ReminderID]
		INNER JOIN [ReminderType]
			ON [ReminderType].[Subtype] = [Reminder].[Subtype]
		INNER JOIN [Document]
			ON ([Document].[Class] = [Reminder].[DocumentClass] AND
				[Document].[ID] = [Reminder].[DocumentID])
	WHERE
		([UserReminder].[UserID] = @UserID AND [UserReminder].[State] = 1) AND
		([Reminder].[EventTime] IS NOT NULL) AND
		(
			[Document].[Class] != @InvoiceClass 
			OR
			([ReminderType].[Subtype] = 'VP' AND [Document].[State] = 2) 
			OR
			([ReminderType].[Subtype] = 'VA' AND [Document].[State] = 6)
			OR
			[ReminderType].[Subtype] NOT IN ('VP', 'VA')
		 ) 
		 AND
		(DATEADD(minute, -[UserReminder].[NextInterval], [Reminder].[EventTime]) <= @Now)
	
	RETURN 0
GO

GRANT EXECUTE ON [pjm5NotificationEventGet] TO PUBLIC
GO 