IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm5SubscriptionEventGet]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm5SubscriptionEventGet'
		DROP PROCEDURE [dbo].[pjm5SubscriptionEventGet]
	END
GO

PRINT 'Creating procedure pjm5SubscriptionEventGet'
GO

/******************************************************************************
**		File: pjm5SubscriptionEventGet.sql
**		Name: pjm5SubscriptionEventGet
**		Desc: 
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 20/12/2005 13:00:00
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE PROCEDURE [pjm5SubscriptionEventGet]
	@UserID TIdentifier,
	@NotificationWayID TIdentifier,
	@DefaultLang TLanguage
AS
	SET NOCOUNT ON
	
	DECLARE
		@IssueClass TClass,
		@ServiceRequestClass TClass,
		@ProjectClass TClass,
		@InvoiceClass TClass,
		@StatementClass TClass,
		@TaskRequestClass TClass,
		@InitialEventState TState,
		@AlertedEventState TState
		
	SELECT
		@IssueClass = dbo.adfGetClassByName('Pjm5.Issue'),
		@ServiceRequestClass = dbo.adfGetClassByName('Pjm5.ServiceRequest'),
		@ProjectClass = dbo.adfGetClassByName('Pjm5.Project'),
		@InvoiceClass = dbo.adfGetClassByName('Pjm5.Invoice'),
		@StatementClass = dbo.adfGetClassByName('Pjm5.Statement'),
		@TaskRequestClass = dbo.adfGetClassByName('Pjm5.TaskRequest'),
		@InitialEventState = dbo.fnPjm5GetInitialEventState(),
		@AlertedEventState = dbo.fnPjm5GetAlertedEventState()
		
	IF (@DefaultLang IS NULL OR (@DefaultLang != 'ru' AND @DefaultLang != 'en'))
	BEGIN
		SET @DefaultLang = dbo.adfGetLanguage()
	END
	
	SELECT
		[Event].[ID],
		[Event].[SubscriptionTypeID],
		[SubscriptionType].[Aggregatable],
		[Event].[DocumentClass],
		[Event].[DocumentID],
		[Event].[ContainerClass],
		[Event].[ContainerID],
		[Event].[AlertTime],
		[SubscriptionRuleNotificationWay].[SubscriptionRecurrencePatternID],
		[Document].[FileAs] AS [DocumentFileAs],
		CASE
			WHEN [Document].[Class] = @IssueClass THEN
			(
				SELECT [Number] FROM [Issue]
				WHERE [Issue].[ID] = [Document].[ID]
			)
			WHEN [Document].[Class] = @ServiceRequestClass THEN
			(
				SELECT [Number] FROM [ServiceRequest]
				WHERE [ServiceRequest].[ID] = [Document].[ID]
			)
			WHEN [Document].[Class] = @ProjectClass THEN
			(
				SELECT [Number] FROM [Project]
				WHERE [Project].[ID] = [Document].[ID]
			)
			WHEN [Document].[Class] = @InvoiceClass THEN
			(
				SELECT [Number] FROM [Invoice]
				WHERE [Invoice].[ID] = [Document].[ID]
			)
			WHEN [Document].[Class] = @StatementClass THEN
			(
				SELECT [Number] FROM [Statement]
				WHERE [Statement].[ID] = [Document].[ID]
			)
			ELSE NULL
		END AS [DocumentNumber],
		CASE
			WHEN [ContainerTemplate].[ID] IS NULL THEN
				dbo.fnPjm5GetLocalSimpleTheme([GlobalTemplate].[ID],
				dbo.fnPjm5GetEmployeeLanguage([Employee].[ID], @DefaultLang, [SubscriptionRule].[ExternalInitiator]))
			ELSE
				dbo.fnPjm5GetLocalSimpleTheme([ContainerTemplate].[ID],
				dbo.fnPjm5GetEmployeeLanguage([Employee].[ID], @DefaultLang, [SubscriptionRule].[ExternalInitiator]))
		END AS [SimpleMessageTheme],
		CASE
			WHEN [ContainerTemplate].[ID] IS NULL THEN
				dbo.fnPjm5GetLocalSimpleBody([GlobalTemplate].[ID],
				dbo.fnPjm5GetEmployeeLanguage([Employee].[ID], @DefaultLang, [SubscriptionRule].[ExternalInitiator]))
			ELSE
				dbo.fnPjm5GetLocalSimpleBody([ContainerTemplate].[ID],
				dbo.fnPjm5GetEmployeeLanguage([Employee].[ID], @DefaultLang, [SubscriptionRule].[ExternalInitiator]))
		END AS [SimpleMessageBody],
		CASE
			WHEN [ContainerTemplate].[ID] IS NULL THEN
				dbo.fnPjm5GetLocalAggregatedTheme([GlobalTemplate].[ID],
				dbo.fnPjm5GetEmployeeLanguage([Employee].[ID], @DefaultLang, [SubscriptionRule].[ExternalInitiator]))
			ELSE
				dbo.fnPjm5GetLocalAggregatedTheme([ContainerTemplate].[ID],
				dbo.fnPjm5GetEmployeeLanguage([Employee].[ID], @DefaultLang, [SubscriptionRule].[ExternalInitiator]))
		END AS [AggregatedMessageTheme],
		CASE
			WHEN [ContainerTemplate].[ID] IS NULL THEN
				dbo.fnPjm5GetLocalAggregatedBody([GlobalTemplate].[ID],
				dbo.fnPjm5GetEmployeeLanguage([Employee].[ID], @DefaultLang, [SubscriptionRule].[ExternalInitiator]))
			ELSE
				dbo.fnPjm5GetLocalAggregatedBody([ContainerTemplate].[ID],
				dbo.fnPjm5GetEmployeeLanguage([Employee].[ID], @DefaultLang, [SubscriptionRule].[ExternalInitiator]))
		END AS [AggregatedMessageBody],
		ISNULL((CASE 
			WHEN [SubscriptionRule].[ExternalInitiator] = 0 THEN
			(
				SELECT TOP 1 
					[Email] 
				FROM 
					[Email]
				WHERE 
					[Email].[BusinessCardID] = [Employee].[ID] AND
					[Email].[BusinessCardClass] = [Employee].[Class]
			)
			ELSE
			(
				SELECT 
					CAST([PropertyValue].[Value] as varchar(512)) 
				FROM 
					[PropertyValue]
					INNER JOIN [Property] 
						ON	(	[PropertyValue].[PropertyID] = [Property].[ID] AND
								[Property].[Guid] = CAST('{00000001-0000-0000-0000-000000000000}' as uniqueidentifier)
							)
				WHERE 
					[PropertyValue].[DocumentID] = [Document].[ID] AND
					[PropertyValue].[DocumentClass] = [Document].[Class]
			)
		END),'') AS [Email]
	FROM
		[Event]
		INNER JOIN [SubscriptionType]
			ON [SubscriptionType].[ID] = [Event].[SubscriptionTypeID]
		INNER JOIN [SubscriptionRule]
			ON [SubscriptionRule].[ID] = [Event].[SubscriptionRuleID]
		INNER JOIN [Employee]
			ON [Employee].[UserID] = [SubscriptionRule].[UserID]
		INNER JOIN [SubscriptionRuleNotificationWay]
			ON ([SubscriptionRuleNotificationWay].[SubscriptionRuleID] = [SubscriptionRule].[ID] AND
				[SubscriptionRuleNotificationWay].[NotificationWayID] = [Event].[NotificationWayID])
		INNER JOIN [Document]
			ON ([Document].[Class] = [Event].[DocumentClass] AND
				[Document].[ID] = [Event].[DocumentID])
		LEFT JOIN [NotificationMessageTemplate] AS [ContainerTemplate]
			ON ([ContainerTemplate].[SubscriptionTypeID] = [Event].[SubscriptionTypeID] AND
				[ContainerTemplate].[NotificationWayID] = [Event].[NotificationWayID] AND
				[ContainerTemplate].[DocumentClass] = [Event].[ContainerClass] AND
				[ContainerTemplate].[DocumentID] = [Event].[ContainerID])
		LEFT JOIN [NotificationMessageTemplate] AS [GlobalTemplate]
			ON ([GlobalTemplate].[SubscriptionTypeID] = [Event].[SubscriptionTypeID] AND
				[GlobalTemplate].[NotificationWayID] = [Event].[NotificationWayID] AND
				[GlobalTemplate].[DocumentClass] IS NULL AND
				[GlobalTemplate].[DocumentID] IS NULL)
	WHERE
		([SubscriptionRule].[UserID] = @UserID OR @UserID IS NULL) AND
		([Event].[State] = @InitialEventState OR [Event].[State] = @AlertedEventState) AND
		([Event].[NotificationWayID] = @NotificationWayID)
	
	RETURN 0
GO

GRANT EXECUTE ON [pjm5SubscriptionEventGet] TO PUBLIC
GO