IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'pjm5PaymentCheckImported')
	BEGIN
		PRINT 'Dropping Procedure pjm5PaymentCheckImported'
		DROP  Procedure  pjm5PaymentCheckImported
	END

GO

PRINT 'Creating Procedure pjm5PaymentCheckImported'
GO

--------------------------------------------------------------------------------------
CREATE Procedure pjm5PaymentCheckImported
	@Date TDate,
	@Number TCode,
	@PayerCompanyID TIdentifier,
	@BankAccountID TIdentifier,
	@Value TMoney,
	@CurrencyCode TCode

AS
	SET NOCOUNT ON

------------------------------------------------
	SELECT
		[Payment].[ID] AS [PaymentID]
	FROM 
		[Payment]
	WHERE
		[Payment].[Date]=@Date AND
		[Payment].[Number]=@Number AND
		[Payment].[PayerCompanyID]=@PayerCompanyID AND
		([Payment].[PayeeBankAccountID]=@BankAccountID OR
		[Payment].[PayeeBankAccountID] IS NULL AND @BankAccountID IS NULL)
		
	
	DECLARE @CurrencyRate TABLE
	(
		CurrencyRate TMoney,
		CurrencyCode TCode
	)
	
	IF(@CurrencyCode = dbo.fnPjm5GetNationalCurrency())
	BEGIN
	INSERT INTO @CurrencyRate
	(
		CurrencyRate,
		CurrencyCode
	)
	SELECT 
		[CurrencyRate].[Rate],
		[CurrencyRate].[FromCurrencyCode]
	FROM
		[CurrencyRate]
	WHERE
		[CurrencyRate].[ToCurrencyCode] = @CurrencyCode
		AND
		(
			[ValidFrom] = @Date
		OR
			[ValidTo] = @Date
		OR
			(@Date BETWEEN [ValidFrom] AND [ValidTo])
		)	
	END		
	ELSE
	BEGIN
	INSERT INTO @CurrencyRate
	(
		CurrencyRate,
		CurrencyCode
	)
	SELECT 
		[CurrencyRate].[Rate],
		[CurrencyRate].[ToCurrencyCode]
	FROM
		[CurrencyRate]
	WHERE
		[CurrencyRate].[FromCurrencyCode] = @CurrencyCode
		AND
		(
			[ValidFrom] = @Date
		OR
			[ValidTo] = @Date
		OR
			(@Date BETWEEN [ValidFrom] AND [ValidTo])
		)	
	END
	
	DECLARE @BaseCurrencyCode TCode3,
			@val TVariant
	EXEC adfParameterGet @Name='BaseCurrency', @Group='CommonBusiness', @Value = @val OUTPUT
	SET @BaseCurrencyCode = CAST(@val as varchar(3))
	
	IF (@BaseCurrencyCode = dbo.fnPjm5GetNationalCurrency() AND
		EXISTS
	(
		SELECT 
			[Rate]
		FROM
			[CurrencyRate]
		WHERE
			[FromCurrencyCode] = @CurrencyCode AND
			[ToCurrencyCode] = @BaseCurrencyCode AND
			(@Date BETWEEN [ValidFrom] AND [ValidTo])
		)
			OR
		@BaseCurrencyCode != dbo.fnPjm5GetNationalCurrency() AND
		EXISTS (
		SELECT 
			[Rate]
		FROM
			[CurrencyRate]
		WHERE
			[FromCurrencyCode] = @BaseCurrencyCode AND
			[ToCurrencyCode] = @CurrencyCode AND
			(@Date BETWEEN [ValidFrom] AND [ValidTo])
	) OR @BaseCurrencyCode = @CurrencyCode )
	BEGIN
	SELECT
	TOP(1) 
		[Invoice].[ID] AS [InvoiceID],
		[Invoice].[VAT] AS [VAT],
		CASE WHEN [Invoice].[CurrencyCode]=@CurrencyCode THEN 1.0 
			ELSE CASE WHEN [Invoice].[PaymentCurrencyPoint] =1 THEN [Invoice].[CurrencyRate] 
					ELSE [Currency].[CurrencyRate] END END AS [CurrencyRate]
	FROM [Invoice]
		LEFT OUTER JOIN @CurrencyRate AS [Currency]
		ON [Invoice].[CurrencyCode]=[Currency].[CurrencyCode]
	WHERE
		[Invoice].[State] =2 AND
		[Invoice].[CustomerCompanyID] = @PayerCompanyID AND
		[Invoice].[SupplierBankAccountID] = @BankAccountID AND
		[Invoice].[TotalWithVAT] = @Value
	ORDER BY [Invoice].[Date] ASC
	END

	RETURN 0
GO

GRANT EXEC ON pjm5PaymentCheckImported TO PUBLIC
GO   