IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm5trPaymentDetailInsert')
	BEGIN
		PRINT 'Dropping Trigger pjm5trPaymentDetailInsert'
		DROP  Trigger pjm5trPaymentDetailInsert
	END
GO

/******************************************************************************
**		File: 
**		Name: pjm5trPaymentDetailInsert
**		Desc: 
**
**		This template can be customized:
**              
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**		--------		--------				-------------------------------------------
**    
*******************************************************************************/

PRINT 'Creating Trigger pjm5trPaymentDetailInsert'
GO
CREATE Trigger pjm5trPaymentDetailInsert 
ON [PaymentDetail] FOR INSERT
AS
BEGIN
	IF @@ROWCOUNT = 0
		RETURN
		
	IF EXISTS(
		SELECT *
		FROM [Inserted]
			INNER JOIN [Invoice]
				ON [Inserted].[InvoiceID] = [Invoice].[ID]
			INNER JOIN [Payment]
				ON [Inserted].[PaymentID] = [Payment].[ID]
		WHERE [Invoice].[CustomerCompanyID] != [Payment].[PayerCompanyID]
	)
	BEGIN	
		RAISERROR ('#wrongpayercomanyid', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END 
		
	DECLARE
		@invoiceClass TClass,
		@paymentClass TClass
	SET @invoiceClass = dbo.adfGetClassByName('Pjm5.Invoice')
	SET @paymentClass = dbo.adfGetClassByName('Adf.Payment')
	
	INSERT INTO [Attachment]
		(
		[DocumentClass], [DocumentID], [Name],
		[LinkDocumentClass], [LinkDocumentID], [AttachmentTypeID]
		)
	SELECT
		@invoiceClass, [inserted].[InvoiceID], '',
		@paymentClass, [inserted].[PaymentID], 3
	FROM
		[inserted]
	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN
	END
END

GO
IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm5trPaymentDetailUpdate')
	BEGIN
		PRINT 'Dropping Trigger pjm5trPaymentDetailUpdate'
		DROP  Trigger pjm5trPaymentDetailUpdate
	END
GO

/******************************************************************************
**		File: 
**		Name: pjm5trPaymentDetailUpdate
**		Desc: 
**
**		This template can be customized:
**              
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**		--------		--------				-------------------------------------------
**    
*******************************************************************************/

PRINT 'Creating Trigger pjm5trPaymentDetailUpdate'
GO
CREATE Trigger pjm5trPaymentDetailUpdate 
ON [PaymentDetail] FOR UPDATE
AS
BEGIN
	DECLARE
		@invoiceClass TClass,
		@paymentClass TClass
	SET @invoiceClass = dbo.adfGetClassByName('Pjm5.Invoice')
	SET @paymentClass = dbo.adfGetClassByName('Adf.Payment')
END

GO

IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm5trPaymentDetailDelete')
	BEGIN
		PRINT 'Dropping Trigger pjm5trPaymentDetailDelete'
		DROP  Trigger pjm5trPaymentDetailDelete
	END
GO

/******************************************************************************
**		File: 
**		Name: pjm5trPaymentDetailDelete
**		Desc: 
**
**		This template can be customized:
**              
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**		--------		--------				-------------------------------------------
**    
*******************************************************************************/

PRINT 'Creating Trigger pjm5trPaymentDetailDelete'
GO
CREATE Trigger pjm5trPaymentDetailDelete 
ON [PaymentDetail] FOR DELETE
AS
BEGIN
	IF @@ROWCOUNT = 0
		RETURN
	DECLARE
		@invoiceClass TClass,
		@paymentClass TClass
	SET @invoiceClass = dbo.adfGetClassByName('Pjm5.Invoice')
	SET @paymentClass = dbo.adfGetClassByName('Adf.Payment')

	DELETE [Attachment]
	FROM
		[Attachment]
		INNER JOIN [deleted]
			ON ([Attachment].[DocumentClass] = @invoiceClass  AND 
				[Attachment].[DocumentID] = [deleted].[InvoiceID] AND
				[Attachment].[LinkDocumentClass] = @paymentClass AND 
				[Attachment].[LinkDocumentID] = [deleted].[PaymentID])
	WHERE
		[Attachment].[AttachmentTypeID] = 3
	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN
	END
	
	
	/*********Change State Of Payment When One Of The Invoices was unbinded***************/
	DECLARE
		@ID TIdentifier,
		@State TState,
		@Count TCount
		
	DECLARE payments CURSOR LOCAL FAST_FORWARD READ_ONLY FOR
	SELECT
		[Payment].[ID],
		[Payment].[State],
		(SELECT COUNT(*) FROM [PaymentDetail] WHERE [PaymentDetail].[ID] = [deleted].[ID]) as [Count]
	FROM
		[deleted]
		INNER JOIN [Payment]
			ON [deleted].[PaymentID] = [Payment].[ID]
		
	OPEN payments
	
	FETCH NEXT FROM payments
	INTO @ID, @State, @Count
	
	WHILE @@FETCH_STATUS = 0
	BEGIN
		
		IF @State = 3
			exec adfPaymentChangeState @ID, 2
		
		FETCH NEXT FROM payments
		INTO @ID, @State, @Count
		
	END
	
	CLOSE payments
	DEALLOCATE payments
	/************************************/
	
	
END

GO
