IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[PjmSyncTimeEntryExport]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure PjmSyncTimeEntryExport'
		DROP PROCEDURE [dbo].[PjmSyncTimeEntryExport]
	END
GO

PRINT 'Creating procedure PjmSyncTimeEntryExport'
GO

CREATE PROCEDURE [PjmSyncTimeEntryExport]
	@TimeStamp bigint OUTPUT,
	@DeviceID varchar(40),
	@ItemID varchar(32),
	@FileAs TName,
	@StartTime TDateTime,
	@EndTime TDateTime,
	@Reminder TBool,
	@Categories TNotes,
	@Comments TNotes	
AS	
	SET NOCOUNT ON
		
	DECLARE 
			@State				TState,
			@ProjectID			TIdentifier,
			@AssignmentID		TIdentifier,
			@ActivityTypeID		TIdentifier,
			@RoundedDuration	TCount,
			@CurrencyCode		TCode3,
			@Rate				TMoney,
			@VATRate			TPercent,
			@EmployeeID			TIdentifier,
			@UserID				TIdentifier,	
			@Class				TClass,
			@ID					TIdentifier,
			@Code				TCode,
			@Subject			TName,
			@Index				int,
			@Duration			TCount
			
			
	IF EXISTS(SELECT * FROM [ExternalLink] WHERE [DeviceID] = @DeviceID AND [ItemID] = @ItemID)
		RETURN 0

	SET @EmployeeID = dbo.adfGetCurrentEmployeeID() 
	SET @UserID = dbo.adfGetCurrentUserID()
	SET @VATRate = dbo.fnPjm5GetVATRate()
	SET @Rate = dbo.fnPjm5GetEmployeeRate(@EmployeeID, @ProjectID, 0, NULL, 0)
	SET @Class = dbo.adfGetClassByName('Pjm5.TimeEntry')
	SET @Duration = DATEDIFF(minute, @StartTime, @EndTime)

	SET @Index = PATINDEX('%(%)', @FileAs)

	IF @Index  > 0
	BEGIN
		SET @Code = RTRIM(LTRIM(SUBSTRING(@FileAs, @index + 1, LEN(@FileAs) - @index - 1)))
		SET @FileAs = RTRIM(LTRIM(SUBSTRING(@FileAs, 1, @index - 1)))
		IF LEN(@FileAs) = 0
		BEGIN
			SET @FileAs = @Code
			SET @Code = NULL
		END
	END
	
	IF @Code IS NOT NULL AND EXISTS(SELECT * FROM [Project] INNER JOIN [vwAssignment]
				ON [Project].[ID] = [vwAssignment].[ProjectID] WHERE [Number]=@Code AND 
				[vwAssignment].[EmployeeID] = @EmployeeID AND [vwAssignment].[State]=1)
	BEGIN
		SELECT
			@ProjectID = [Project].[ID],
			@CurrencyCode = [Project].[CurrencyCode],
			@RoundedDuration = dbo.fnPjm5RoundedDuration(@Duration, [RoundingInterval], [RoundingMode]),
			@AssignmentID = [vwAssignment].[ID]
		FROM
			[Project]
			INNER JOIN [vwAssignment]
				ON [Project].[ID] = [vwAssignment].[ProjectID]
		WHERE
			[Project].[Number] = @Code AND 
			[vwAssignment].[EmployeeID] = @EmployeeID AND
			[vwAssignment].[State] = 1

		DECLARE @var sql_variant, @ActivityTypeIsMandatory BIT

		EXEC adfParameterGet 'ActivityTypeIsMandatory', 'Pjm5', @var OUTPUT
		IF @var IS NULL
			SET @ActivityTypeIsMandatory = 1
		ELSE
			SET @ActivityTypeIsMandatory = CAST(@var as BIT)
	END
	ELSE
	BEGIN
		SET @CurrencyCode = 'RUB'
		SET @RoundedDuration = @Duration
	END

	BEGIN TRANSACTION
	
	INSERT INTO [TimeEntry]
	(
			[State],
			[Flag],
			[FileAs],
			[ActivityTypeID],
			[EmployeeID],
			[ProjectID],
			[StartTime],
			[Duration],
			[RoundedDuration],
			[Billable],
			[Overtime],
			[Rate],
			[CurrencyCode],
			[VATRate],
			[Comments]
	)
		VALUES 
	(
		1,
		1,
		ISNULL(@FileAs, '( )'),
		@ActivityTypeID,
		@EmployeeID,
		@ProjectID,
		@StartTime,
		@Duration,
		@RoundedDuration,
		1,
		0,
		0,
		@CurrencyCode,
		@VATRate,
		@Comments
	)
	IF @@ERROR !=0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END

	SET @ID = SCOPE_IDENTITY()

	INSERT INTO [ExternalLink]
	(
		[DeviceID],
		[ItemID],
		[DocumentClass],
		[DocumentID]
	)
	VALUES
	(
		@DeviceID,
		@ItemID,
		@Class,
		@ID
	)

	IF @@ERROR !=0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END

	SELECT @TimeStamp = CAST([TimeEntry].[Timestamp] as bigint)
	FROM [TimeEntry]
	WHERE [TimeEntry].[ID]=@ID 

	COMMIT TRANSACTION

	RETURN 0
GO

GRANT EXECUTE ON PjmSyncTimeEntryExport TO PUBLIC
GO
   