IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm5ProjectBrowseActive]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm5ProjectBrowseActive'
		DROP PROCEDURE [dbo].[pjm5ProjectBrowseActive]
	END
GO

PRINT 'Creating procedure pjm5ProjectBrowseActive'
GO

/******************************************************************************
**		File: pjm5ProjectBrowseActive.sql
**		Name: pjm5ProjectBrowseActive
**		Desc:    ,    
**		       \  \ 
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: Andrew.Maklakov
**		Date: 01/20/2005 11:01:32
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:		Description:
**		--------	--------	-----------------------
**    	24.03.09	ENB			   ,
**								     , 
**								   .  
**								  
**
**   	26.03.09	ENB			   ,   
**								    
**								    
**								    
**		14.02.13	SOV			     
**		29.04.13	SOV			    
**		07.05.13	SOV			  ID . -  . - 
**		05.05.14	BAO				    
*******************************************************************************/
CREATE PROCEDURE [pjm5ProjectBrowseActive]

AS

	SET NOCOUNT ON
	
	IF dbo.adfHasAccessRight(dbo.adfGetRightID('ViewAllProjects')) = 0 BEGIN
		RAISERROR('#NotEnoughPermissions', 18, -1)
		RETURN 1
	END

	DECLARE 
		@PeriodStart	TDate,
		@PeriodEnd	TDate,
		@ViewAllFinance TBool
		
	SET @ViewAllFinance = dbo.adfHasAccessRight(dbo.adfGetRightID('ViewAllFinance'))
	
	SELECT
		@PeriodStart = CAST(CONVERT(varchar(8), GETDATE(), 112) AS datetime),
		@PeriodEnd   = DATEADD(day, 1, @PeriodStart)
	
	SELECT
		[Project].[Class],
		[Project].[ID],
		[Project].[State],
		[Project].[Flag],
		[Project].[Date],
		[Project].[Number],
		[Project].[FileAs],
		[Project].[ProjectTypeID],
		[Project].[SupplierCompanyID],
		[Project].[ProjectManagerEmployeeID],
		[Project].[CustomerCompanyID],
		[Project].[CustomerFinanceManagerID],
		[Project].[ResponsiblePersonEmployeeID],
		[Project].[PriorityID],
		[ProjectDetail].[DueDate],
		[Project].[EndDate],
		[Project].[CurrencyCode],
		[Project].[CostCenterID],
		[Project].[ProjectStageID],
		[Budgets].[PlannedManHour] / 60.0 AS [ManHour],
		CASE
			WHEN @ViewAllFinance = 1 
			THEN [Project].[Budget]
			ELSE -1
		END AS [Budget],
		[Budgets].[PlannedManHour],
		CASE
			WHEN @ViewAllFinance = 1 
			THEN [Budgets].[PlannedBudget]
			ELSE -1
		END AS [PlannedBudget],
		[Project].[Billable],
		[Project].[RoundingInterval],
		[Project].[RoundingMode],
		[Project].[ContractNumber],
		[Project].[ContractDate],
		[Project].[ContractEndDate],
		[Project].[ContractSubject],
		[Project].[ServiceLevelID],
		[Project].[ApplyVAT],
		[Project].[InvoiceExpirationPeriod],
		[Project].[InvoiceExpirationPeriodType],
		[Project].[Comments],
		[Project].[Created],
		[Project].[CreatedBy],
		[Project].[Modified],
		[Project].[ModifiedBy],
		[Project].[Timestamp],
		[Project].[ExchangeSystemName],
		[ProjectDetail].[PercentComplete],
		(CAST([ProjectDetail].[PercentComplete] * 100 as int) / 25) * 25 as PC,
		[Project].[PaymentCurrencyPoint],
		[Project].[IsConsignor],
		[BusinessCard].[FileAs] as 'Consignor',
		[Project].[ConsignorContract],
		CASE 
			WHEN NOT EXISTS(SELECT *FROM [Project] 
									WHERE 
										([ID] = [SubProjects].[ProjectID] AND [SubProjects].[ProjectID] IS NOT NULL) AND
										(([Project].[State] = 2 AND [Project].[Date] < @PeriodEnd) 
										OR
										([Project].[State] = 4 AND 
										[Project].[EndDate] >= @PeriodStart AND [Project].[EndDate] < @PeriodEnd)
										))
			THEN NULL
			ELSE [SubProjects].[ProjectID]
		END AS [ParentProjectID],
		[Project].[ManagerAssistantEmployeeID],
		[Project].[FinanceManagerEmployeeID]
	FROM
		[Project]
		INNER JOIN [ProjectDetail]
			ON [ProjectDetail].[ID] = [Project].[ID]
		LEFT OUTER JOIN [BusinessCard]
			ON [BusinessCard].[Class] = [Project].[ConsignorClass] AND [BusinessCard].[ID] = [Project].[ConsignorID]
		LEFT OUTER JOIN fnpjm6GetSubprojectsForBrowse() AS [SubProjects]
			ON [SubProjects].[SubprojectID] = [Project].[ID]
		INNER JOIN dbo.fnPjm7GetProjectsBudgets() AS [Budgets]
			ON [Budgets].[ProjectID] = [Project].[ID]
	WHERE
		[Project].[State] = 2 AND [Project].[Date] < @PeriodEnd
		
	RETURN 0
GO

GRANT EXECUTE ON [pjm5ProjectBrowseActive] TO PUBLIC
GO 

