﻿IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[ReportExcelBudget]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure ReportExcelBudget'
		DROP PROCEDURE [dbo].[ReportExcelBudget]
	END
GO

PRINT 'Creating procedure ReportExcelBudget'
GO

/******************************************************************************
**		File: ReportExcelBudget.sql
**		Name: ReportExcelBudget
**		Desc: this procedure automatically generated by Avicom.Adf.Generator
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: Anton.Paramonov
**		Date: 22.09.2011 16:20:00
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		12.03.13	SOV				Для диаграммы Ганта по дням добавлена проверка, 
**									чтобы число дней планирования по работам  не превышало 249
**    	29.03.13	MAB				Выборка данных по плановым закупкам и 
									плановой рентабельности
									происходить из таблиц линии затрат и бюджета
*******************************************************************************/
CREATE PROCEDURE ReportExcelBudget
	@ID TIdentifier,
	@ReportName TName = NULL
AS
	DECLARE
		@VATRate				TPercent,
		@VATIsIncluded			TBool,
		@BudgetCurrency			TCode3,
		@nationalCurrency		TCode3,
		@value					TVariant,
		@DateFirstBefore		int,
		@BudgetExpenseLineClass TClass,
		@ProjectClass			TClass,
		@CompanyClass			TClass,
		@EmployeeClass			TClass

	
	SET @DateFirstBefore = @@datefirst
	SET @BudgetExpenseLineClass = dbo.adfGetClassByName('Pjm6.BudgetLineExpense')
	SET @ProjectClass =  dbo.adfGetClassByName('Pjm5.Project')
	SET @CompanyClass = dbo.adfGetClassByName('Adf.Company')
	SET	@EmployeeClass = dbo.adfGetClassByName('Adf.Employee')
	SET DATEFIRST 1

	EXEC adfParameterGet 'NationalCurrency', 'CommonBusiness', @value OUTPUT
	SET @nationalCurrency = CAST(@value as varchar(3))

	SELECT
		@VATRate = [Budget].[VATRate],
		@VATIsIncluded = [Budget].[VATIsIncluded],
		@BudgetCurrency = [Budget].[CurrencyCode]
	FROM
		[Budget]
	WHERE
		[Budget].[ID] = @ID

		
		
	IF EXISTS(
		SELECT *
		FROM
			[Budget]
			INNER JOIN [Project]
				ON [Project].[ID] = [Budget].[DocumentID]
			INNER JOIN [Employee] AS [ManagerEmployee]
				ON [Project].[ProjectManagerEmployeeID] = [ManagerEmployee].[ID]
			LEFT OUTER JOIN [Company] AS [CustomerCompany]	
				ON [Project].[CustomerCompanyID] = [CustomerCompany].[ID]
			LEFT OUTER JOIN [Company] AS [SupplierCompany]	
				ON [Project].[SupplierCompanyID] = [SupplierCompany].[ID]
		WHERE
			[Budget].[ID] = @ID
	)
	BEGIN
		SELECT
			[Budget].[ID],
			[Budget].[FileAs],
			[Budget].[BudgetTypeID],
			[Budget].[CurrencyCode],
			[Budget].[VATRate],
			[Budget].[VATIsIncluded],
			dbo.fnPjm6GetBudgetPeriodName([Budget].[StartDate], [Budget].[EndDate], [Budget].[BudgetTypeID], 'ru', 1) AS [Period],
			[Budget].[UseWorkPlanning],
			[Budget].[UseExpensePlanning],
			(CASE
				WHEN [Budget].[UseExpensePlanning] = 1 AND [Budget].[PlanPurchases] = 1 THEN
					1
				ELSE
					0
			END) AS [UsePurchasePlanning],
			[Budget].[LastCalculationDate],
			[PlanningActivityForWork],
			[PlanningEmployeeForWork],
			[PlanningQualificationForWork],
			[PlanningExpenseTypeForExpense],
			[PlanningEmployeeForExpense],
			[Project].[FileAs] AS [ProjectFileAs],
			[Project].[Number] AS [ProjectCode],

			(SELECT 
				ISNULL([PropertyValue].[Value],'') 
			FROM [PropertyValue] 
				JOIN [Property] ON [PropertyValue].[PropertyID] = [Property].[ID]
			WHERE [Property].[Class] = @ProjectClass
				AND [PropertyValue].[DocumentID] = [Project].[ID]
				AND [Property].[Guid] = '00000003-0000-0000-0000-000000000000') AS [ProjectFileAsOtherLang],

			[Budget].[PlanProfitability] AS [PlannedProfitability],
			
			[ManagerEmployee].[FileAs] AS [ProjectManagerName],
			(SELECT 
				ISNULL([PropertyValue].[Value],'') 
			FROM [PropertyValue] 
				JOIN [Property] ON [PropertyValue].[PropertyID] = [Property].[ID]
			WHERE [Property].[Class] = @EmployeeClass
				AND [PropertyValue].[DocumentID] = [ManagerEmployee].[ID]
				AND [Property].[Guid] = '00000008-0000-0000-0000-000000000000') AS [ProjectManagerNameOtherLang],

			[CustomerCompany].[LegalName] AS [CustomerCompanyLegalName],

			(SELECT 
				ISNULL([PropertyValue].[Value],'') 
			FROM [PropertyValue] 
				JOIN [Property] ON [PropertyValue].[PropertyID] = [Property].[ID]
			WHERE [Property].[Class] = @CompanyClass
				AND [PropertyValue].[DocumentID] = [CustomerCompany].[ID]
				AND [Property].[Guid] = '00000001-0000-0000-0000-000000000000') AS [CustomerCompanyLegalNameOtherLang],

			[SupplierCompany].[LegalName] AS [SupplierCompanyLegalName],
			(SELECT 
				ISNULL([PropertyValue].[Value],'') 
			FROM [PropertyValue] 
				JOIN [Property] ON [PropertyValue].[PropertyID] = [Property].[ID]
			WHERE [Property].[Class] = @CompanyClass
				AND [PropertyValue].[DocumentID] = [SupplierCompany].[ID]
				AND [Property].[Guid] = '00000001-0000-0000-0000-000000000000')	AS [SupplierCompanyLegalNameOtherLang],

			CASE WHEN [DiscountPercentWork] > 0 OR [DiscountTotalWork] > 0 THEN 2
				WHEN [DiscountPercentWork] < 0 OR [DiscountTotalWork] < 0 THEN 1
				WHEN [DiscountPercentWork] IS NULL AND [DiscountTotalWork] IS NULL THEN 0 END  AS [DiscountTypeWork],
			CASE WHEN [DiscountPercentExpense] > 0 OR [DiscountTotalExpense] > 0 THEN 2
				WHEN [DiscountPercentExpense] < 0 OR [DiscountTotalExpense] < 0 THEN 1
				WHEN [DiscountPercentExpense] IS NULL AND [DiscountTotalExpense] IS NULL THEN 0 END AS [DiscountTypeExpense],
			[Budget].[PlannedManHour],
			[Budget].[ActualManHour],
			--Без вложенности--------------------------------------------------------------------------------------------------------------------------------------------------
			--Стоимость работ (план)
			dbo.fnPjm6GetBudgetTotal([VATRate], [VATIsIncluded], [TotalPCPlannedManHour], [TotalWithVATPCPlannedManHour], 'ManHour') 
				AS [TotalPlannedManHour],
			--Сумма скидки (план)
			dbo.fnPjm6GetBudgetTotalDiscount([DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], dbo.fnPjm6GetBudgetTotal([VATRate], [VATIsIncluded], [TotalPCPlannedManHour], [TotalWithVATPCPlannedManHour], 'ManHour'))
				AS [DiscountTotalPlannedManHour],
			--Итого со скидкой (план)
			dbo.fnPjm6GetBudgetTotalWithDiscount([VATRate], [VATIsIncluded], [DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [TotalPCPlannedManHour], [TotalWithVATPCPlannedManHour])
				AS [TotalWDPlannedManHour],
			--НДС от итого со скидкой (план)
			dbo.fnPjm6GetBudgetVATForTotal([VATRate], [VATIsIncluded], [DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [TotalPCPlannedManHour], [TotalWithVATPCPlannedManHour])
				AS [VATWDPlannedManHour],
			--Итого со скидкой и НДС (план)
			dbo.fnPjm6GetBudgetTotalWithVATWD([VATRate], [VATIsIncluded], [DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [TotalPCPlannedManHour], [TotalWithVATPCPlannedManHour])
				AS [TotalWithVATPCWDPlannedManHour],
			--Стоимость работ (факт)
			dbo.fnPjm6GetBudgetTotal([VATRate], [VATIsIncluded], [TotalPCActualManHour], [TotalWithVATPCActualManHour], 'ManHour')
				AS [TotalActualManHour],
			--Сумма скидки (факт)
			dbo.fnPjm6GetBudgetTotalDiscount([DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], dbo.fnPjm6GetBudgetTotal([VATRate], [VATIsIncluded], [TotalPCActualManHour], [TotalWithVATPCActualManHour], 'ManHour')) 
				AS [DiscountTotalActualManHour],
			--Итого со скидкой (факт)
			dbo.fnPjm6GetBudgetTotalWithDiscount([VATRate], [VATIsIncluded], [DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [TotalPCActualManHour], [TotalWithVATPCActualManHour])
				AS [TotalWDActualManHour],
			--НДС от итого со скидкой (факт)
			dbo.fnPjm6GetBudgetVATForTotal([VATRate], [VATIsIncluded], [DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [TotalPCActualManHour], [TotalWithVATPCActualManHour])
				AS [VATWDActualManHour],
			--Итого со скидкой и НДС (факт)
			dbo.fnPjm6GetBudgetTotalWithVATWD([VATRate], [VATIsIncluded], [DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [TotalPCActualManHour], [TotalWithVATPCActualManHour])
				AS [TotalWithVATPCWDActualManHour],
			------------------------------------------------------------------------------------------------------------------------------------------------------------------
			--Стоимость затрат (план)
			dbo.fnPjm6GetBudgetTotal([VATRate], [VATIsIncluded], [TotalPCPlannedExpense], [TotalWithVATPCPlannedExpense], 'Expense') 
				AS [TotalPlannedExpense],
			--Сумма скидки (план)
			dbo.fnPjm6GetBudgetTotalDiscount([DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], dbo.fnPjm6GetBudgetTotal([VATRate], [VATIsIncluded], [TotalPCPlannedExpense], [TotalWithVATPCPlannedExpense], 'Expense'))
				AS [DiscountTotalPlannedExpense],
			--Итого со скидкой (план)
			dbo.fnPjm6GetBudgetTotalWithDiscount([VATRate], [VATIsIncluded], [DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [TotalPCPlannedExpense], [TotalPCPlannedExpense] + dbo.fnPjm6GetVATForTotal([TotalPCPlannedExpense], [VATRate], 0, 1))
				AS [TotalWDPlannedExpense],
			--НДС от итого со скидкой (план)
			dbo.fnPjm6GetBudgetVATForTotal([VATRate], [VATIsIncluded], [DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [TotalPCPlannedExpense], [TotalPCPlannedExpense] + dbo.fnPjm6GetVATForTotal([TotalPCPlannedExpense], [VATRate], 0, 1))
				AS [VATWDPlannedExpense],
			--Итого со скидкой и НДС (план)
			dbo.fnPjm6GetBudgetTotalWithVATWD([VATRate], [VATIsIncluded], [DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [TotalPCPlannedExpense], [TotalPCPlannedExpense] + dbo.fnPjm6GetVATForTotal([TotalPCPlannedExpense], [VATRate], 0, 1))
				AS [TotalWithVATPCWDPlannedExpense],
			--Стоимость затрат (факт)
			dbo.fnPjm6GetBudgetTotal([VATRate], [VATIsIncluded], [TotalPCActualExpense], [TotalWithVATPCActualExpense], 'Expense') 
				AS [TotalActualExpense],
			--Сумма скидки (факт)
			dbo.fnPjm6GetBudgetTotalDiscount([DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], dbo.fnPjm6GetBudgetTotal([VATRate], [VATIsIncluded], [TotalPCActualExpense], [TotalWithVATPCActualExpense], 'Expense'))
				AS [DiscountTotalActualExpense],
			--Итого со скидкой (факт)
			dbo.fnPjm6GetBudgetTotalWithDiscount([VATRate], [VATIsIncluded], [DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [TotalPCActualExpense], [TotalPCActualExpense] + dbo.fnPjm6GetVATForTotal([TotalPCActualExpense], [VATRate], 0, 1))
				AS [TotalWDActualExpense],
			--НДС от итого со скидкой (факт)
			dbo.fnPjm6GetBudgetVATForTotal([VATRate], [VATIsIncluded], [DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [TotalPCActualExpense], [TotalPCActualExpense] + dbo.fnPjm6GetVATForTotal([TotalPCActualExpense], [VATRate], 0, 1))
				AS [VATWDActualExpense],
			--Итого со скидкой и НДС (факт)
			dbo.fnPjm6GetBudgetTotalWithVATWD([VATRate], [VATIsIncluded], [DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [TotalPCActualExpense], [TotalPCActualExpense] + dbo.fnPjm6GetVATForTotal([TotalPCActualExpense], [VATRate], 0, 1))
			AS [TotalWithVATPCWDActualExpense],

			--Сумма без НДС
			[Budget].[TotalPCPlannedManHour] AS [TotalWithoutVATPCPlannedManHour],
			[Budget].[TotalWithVATPCPlannedManHour],
			[Budget].[VATPCPlannedManHour],
			[Budget].[TotalPCActualManHour] AS [TotalWithoutVATPCActualManHour],
			[Budget].[TotalWithVATPCActualManHour],
			[Budget].[VATPCActualManHour],
			[Budget].[TotalPCPlannedExpense] AS [TotalWithoutVATPCPlannedExpense],
			[Budget].[TotalWithVATPCPlannedExpense],
			[Budget].[VATPCPlannedExpense],
			[Budget].[TotalPCActualExpense] AS [TotalWithoutVATPCActualExpense],
			[Budget].[TotalWithVATPCActualExpense],
			[Budget].[VATPCActualExpense]
		FROM
			[Budget]
			INNER JOIN [Project]
				ON [Project].[ID] = [Budget].[DocumentID]
			INNER JOIN [Employee] AS [ManagerEmployee]
				ON [Project].[ProjectManagerEmployeeID] = [ManagerEmployee].[ID]
			LEFT OUTER JOIN [Company] AS [CustomerCompany]	
				ON [Project].[CustomerCompanyID] = [CustomerCompany].[ID]
			LEFT OUTER JOIN [Company] AS [SupplierCompany]	
				ON [Project].[SupplierCompanyID] = [SupplierCompany].[ID]
		WHERE
			[Budget].[ID] = @ID
	END
	ELSE
	BEGIN
			SELECT
			[Budget].[ID],
			[Budget].[FileAs],
			[Budget].[BudgetTypeID],
			[Budget].[CurrencyCode],
			[Budget].[VATRate],
			[Budget].[VATIsIncluded],
			dbo.fnPjm6GetBudgetPeriodName([Budget].[StartDate], [Budget].[EndDate], [Budget].[BudgetTypeID], 'ru', 1) AS [Period],
			[Budget].[UseWorkPlanning],
			[Budget].[UseExpensePlanning],
			(CASE
				WHEN [Budget].[UseExpensePlanning] = 1 AND [Budget].[PlanPurchases] = 1 THEN
					1
				ELSE
					0
			END) AS [UsePurchasePlanning],
			[Budget].[LastCalculationDate],
			[PlanningActivityForWork],
			[PlanningEmployeeForWork],
			[PlanningQualificationForWork],
			[PlanningExpenseTypeForExpense],
			[PlanningEmployeeForExpense],
			[Project].[FileAs] AS [ProjectFileAs],
			[Project].[Number] AS [ProjectCode],

			(SELECT 
				ISNULL([PropertyValue].[Value],'') 
			FROM [PropertyValue] 
				JOIN [Property] ON [PropertyValue].[PropertyID] = [Property].[ID]
			WHERE [Property].[Class] = @ProjectClass
				AND [PropertyValue].[DocumentID] = [Project].[ID]
				AND [Property].[Guid] = '00000003-0000-0000-0000-000000000000') AS [ProjectFileAsOtherLang],

			[Budget].[PlanProfitability] AS [PlannedProfitability],

			CASE WHEN [DiscountPercentWork] > 0 OR [DiscountTotalWork] > 0 THEN 2
				WHEN [DiscountPercentWork] < 0 OR [DiscountTotalWork] < 0 THEN 1
				WHEN [DiscountPercentWork] IS NULL AND [DiscountTotalWork] IS NULL THEN 0 END  AS [DiscountTypeWork],
			CASE WHEN [DiscountPercentExpense] > 0 OR [DiscountTotalExpense] > 0 THEN 2
				WHEN [DiscountPercentExpense] < 0 OR [DiscountTotalExpense] < 0 THEN 1
				WHEN [DiscountPercentExpense] IS NULL AND [DiscountTotalExpense] IS NULL THEN 0 END AS [DiscountTypeExpense],
			[Budget].[PlannedManHour],
			[Budget].[ActualManHour],
			--Без вложенности--------------------------------------------------------------------------------------------------------------------------------------------------
			--Стоимость работ (план)
			dbo.fnPjm6GetBudgetTotal([VATRate], [VATIsIncluded], [TotalPCPlannedManHour], [TotalWithVATPCPlannedManHour], 'ManHour') 
				AS [TotalPlannedManHour],
			--Сумма скидки (план)
			dbo.fnPjm6GetBudgetTotalDiscount([DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], dbo.fnPjm6GetBudgetTotal([VATRate], [VATIsIncluded], [TotalPCPlannedManHour], [TotalWithVATPCPlannedManHour], 'ManHour'))
				AS [DiscountTotalPlannedManHour],
			--Итого со скидкой (план)
			dbo.fnPjm6GetBudgetTotalWithDiscount([VATRate], [VATIsIncluded], [DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [TotalPCPlannedManHour], [TotalWithVATPCPlannedManHour])
				AS [TotalWDPlannedManHour],
			--НДС от итого со скидкой (план)
			dbo.fnPjm6GetBudgetVATForTotal([VATRate], [VATIsIncluded], [DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [TotalPCPlannedManHour], [TotalWithVATPCPlannedManHour])
				AS [VATWDPlannedManHour],
			--Итого со скидкой и НДС (план)
			dbo.fnPjm6GetBudgetTotalWithVATWD([VATRate], [VATIsIncluded], [DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [TotalPCPlannedManHour], [TotalWithVATPCPlannedManHour])
				AS [TotalWithVATPCWDPlannedManHour],
			--Стоимость работ (факт)
			dbo.fnPjm6GetBudgetTotal([VATRate], [VATIsIncluded], [TotalPCActualManHour], [TotalWithVATPCActualManHour], 'ManHour')
				AS [TotalActualManHour],
			--Сумма скидки (факт)
			dbo.fnPjm6GetBudgetTotalDiscount([DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], dbo.fnPjm6GetBudgetTotal([VATRate], [VATIsIncluded], [TotalPCActualManHour], [TotalWithVATPCActualManHour], 'ManHour')) 
				AS [DiscountTotalActualManHour],
			--Итого со скидкой (факт)
			dbo.fnPjm6GetBudgetTotalWithDiscount([VATRate], [VATIsIncluded], [DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [TotalPCActualManHour], [TotalWithVATPCActualManHour])
				AS [TotalWDActualManHour],
			--НДС от итого со скидкой (факт)
			dbo.fnPjm6GetBudgetVATForTotal([VATRate], [VATIsIncluded], [DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [TotalPCActualManHour], [TotalWithVATPCActualManHour])
				AS [VATWDActualManHour],
			--Итого со скидкой и НДС (факт)
			dbo.fnPjm6GetBudgetTotalWithVATWD([VATRate], [VATIsIncluded], [DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [TotalPCActualManHour], [TotalWithVATPCActualManHour])
				AS [TotalWithVATPCWDActualManHour],
			------------------------------------------------------------------------------------------------------------------------------------------------------------------
			--Стоимость затрат (план)
			dbo.fnPjm6GetBudgetTotal([VATRate], [VATIsIncluded], [TotalPCPlannedExpense], [TotalWithVATPCPlannedExpense], 'Expense') 
				AS [TotalPlannedExpense],
			--Сумма скидки (план)
			dbo.fnPjm6GetBudgetTotalDiscount([DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], dbo.fnPjm6GetBudgetTotal([VATRate], [VATIsIncluded], [TotalPCPlannedExpense], [TotalWithVATPCPlannedExpense], 'Expense'))
				AS [DiscountTotalPlannedExpense],
			--Итого со скидкой (план)
			dbo.fnPjm6GetBudgetTotalWithDiscount([VATRate], [VATIsIncluded], [DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [TotalPCPlannedExpense], [TotalPCPlannedExpense] + dbo.fnPjm6GetVATForTotal([TotalPCPlannedExpense], [VATRate], 0, 1))
				AS [TotalWDPlannedExpense],
			--НДС от итого со скидкой (план)
			dbo.fnPjm6GetBudgetVATForTotal([VATRate], [VATIsIncluded], [DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [TotalPCPlannedExpense], [TotalPCPlannedExpense] + dbo.fnPjm6GetVATForTotal([TotalPCPlannedExpense], [VATRate], 0, 1))
				AS [VATWDPlannedExpense],
			--Итого со скидкой и НДС (план)
			dbo.fnPjm6GetBudgetTotalWithVATWD([VATRate], [VATIsIncluded], [DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [TotalPCPlannedExpense], [TotalPCPlannedExpense] + dbo.fnPjm6GetVATForTotal([TotalPCPlannedExpense], [VATRate], 0, 1))
				AS [TotalWithVATPCWDPlannedExpense],
			--Стоимость затрат (факт)
			dbo.fnPjm6GetBudgetTotal([VATRate], [VATIsIncluded], [TotalPCActualExpense], [TotalWithVATPCActualExpense], 'Expense') 
				AS [TotalActualExpense],
			--Сумма скидки (факт)
			dbo.fnPjm6GetBudgetTotalDiscount([DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], dbo.fnPjm6GetBudgetTotal([VATRate], [VATIsIncluded], [TotalPCActualExpense], [TotalWithVATPCActualExpense], 'Expense'))
				AS [DiscountTotalActualExpense],
			--Итого со скидкой (факт)
			dbo.fnPjm6GetBudgetTotalWithDiscount([VATRate], [VATIsIncluded], [DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [TotalPCActualExpense], [TotalPCActualExpense] + dbo.fnPjm6GetVATForTotal([TotalPCActualExpense], [VATRate], 0, 1))
				AS [TotalWDActualExpense],
			--НДС от итого со скидкой (факт)
			dbo.fnPjm6GetBudgetVATForTotal([VATRate], [VATIsIncluded], [DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [TotalPCActualExpense], [TotalPCActualExpense] + dbo.fnPjm6GetVATForTotal([TotalPCActualExpense], [VATRate], 0, 1))
				AS [VATWDActualExpense],
			--Итого со скидкой и НДС (факт)
			dbo.fnPjm6GetBudgetTotalWithVATWD([VATRate], [VATIsIncluded], [DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [TotalPCActualExpense], [TotalPCActualExpense] + dbo.fnPjm6GetVATForTotal([TotalPCActualExpense], [VATRate], 0, 1))
			AS [TotalWithVATPCWDActualExpense],

			--Сумма без НДС
			[Budget].[TotalPCPlannedManHour] AS [TotalWithoutVATPCPlannedManHour],
			[Budget].[TotalWithVATPCPlannedManHour],
			[Budget].[VATPCPlannedManHour],
			[Budget].[TotalPCActualManHour] AS [TotalWithoutVATPCActualManHour],
			[Budget].[TotalWithVATPCActualManHour],
			[Budget].[VATPCActualManHour],
			[Budget].[TotalPCPlannedExpense] AS [TotalWithoutVATPCPlannedExpense],
			[Budget].[TotalWithVATPCPlannedExpense],
			[Budget].[VATPCPlannedExpense],
			[Budget].[TotalPCActualExpense] AS [TotalWithoutVATPCActualExpense],
			[Budget].[TotalWithVATPCActualExpense],
			[Budget].[VATPCActualExpense]
		FROM
			[Budget]
			INNER JOIN [Project]
				ON [Project].[ID] = [Budget].[DocumentID]
		WHERE
			[Budget].[ID] = @ID
	END
	--Выгрузка групп, которые имеют хотя бы 1 линию, и псевдосоздание групп для тех линий, 
	--в которых группа не указана
	SELECT
		[BudgetLineGroup].[ID],
		[BudgetLineGroup].[State],
		[BudgetLineGroup].[BudgetID],
		[BudgetLineGroup].[FileAs],
		[BudgetLineGroup].[Order],
		[BudgetLineGroup].[GroupTypeID]
	FROM
		[BudgetLineGroup]
	WHERE
		[BudgetLineGroup].[BudgetID] = @ID AND
		(([BudgetLineGroup].[GroupTypeID] = 1 
			AND EXISTS(SELECT *FROM [BudgetLineWork] WHERE [BudgetLineWork].[BudgetLineGroupID] = [BudgetLineGroup].[ID])
		)
		OR
		([BudgetLineGroup].[GroupTypeID] = 2
			AND EXISTS(SELECT *FROM [BudgetLineExpense] WHERE [BudgetLineExpense].[BudgetLineGroupID] = [BudgetLineGroup].[ID])
		))
	UNION
	SELECT
		-1,
		1,
		@ID,
		'Группа не указана',
		0,
		1
	WHERE
		EXISTS(SELECT *FROM [BudgetLineWork] WHERE [BudgetLineWork].[BudgetLineGroupID] IS NULL AND [BudgetLineWork].[BudgetID] = @ID)
	UNION
		SELECT
		-2,
		1,
		@ID,
		'Группа не указана',
		0,
		2
	WHERE
		EXISTS(SELECT *FROM [BudgetLineExpense] WHERE [BudgetLineExpense].[BudgetLineGroupID] IS NULL AND [BudgetLineExpense].[BudgetID] = @ID)

	DECLARE @WorkLines TABLE
	(
		[ID]									TIdentifier,
		[QualificationID]						TIdentifier,
		[PlanningQualificationFileAs]			TName,
		[PlanningQualificationFileAsOtherLang]	TName,
		[RateWithoutVAT]						TMoney
	)

	DECLARE @ExpenseLines TABLE
	(
		[ID]									TIdentifier,
		[ExpenseCodeID]							TIdentifier,
		[PlanningExpenseCodeFileAs]				TName,
		[PlanningExpenseCodeFileAsOtherLang]	TName,
		[PriceWithoutVAT]						TMoney
	)


	--Выгрузка линий работ
	INSERT INTO @WorkLines
	SELECT
		[BudgetLineWork].[ID],
		[BudgetLineWork].[QualificationID],
		[Qualification].[FileAs] AS [PlanningQualificationFileAs],	
		(	
			SELECT 
				ISNULL([PropertyValue].[Value],'') 
			FROM 
				[PropertyValue] 
				JOIN [Property] 
					ON [PropertyValue].[PropertyID] = [Property].[ID]
			WHERE 
				[Property].[Class] = [Qualification].[Class] AND 
				[PropertyValue].[DocumentID] = [Qualification].[ID] AND 
				[Property].[Guid] = '00000001-0000-0000-0000-000000000000'
		) AS [PlanningQualificationFileAsOtherLang],
		(CASE 
			WHEN [ActualManHour] > 0 THEN
				[TotalPCActualManHour] / [ActualManHour]
			ELSE
				0
		END) AS [RateWithoutVAT]
	FROM
		[BudgetLineWork]
		LEFT OUTER JOIN [Qualification]
			ON [BudgetLineWork].[QualificationID] = [Qualification].[ID]
	WHERE
		[BudgetLineWork].[BudgetID] = @ID

	SELECT
		[BudgetLineWork].[ID],
		[BudgetLineWork].[State],
		[BudgetLineWork].[BudgetID],
		[BudgetLineWork].[FileAs],
		(CASE WHEN [BudgetLineWork].[BudgetLineGroupID] IS NULL THEN -1 ELSE [BudgetLineWork].[BudgetLineGroupID] END) AS [BudgetLineGroupID],
		[BudgetLineWork].[Order],
		[BudgetLineWork].[StartDate],
		[BudgetLineWork].[EndDate],
		[BudgetLineWork].[Rate],
		[Employee].[FileAs] AS [PlanningEmployeeName],
		[ActivityType].[FileAs] AS [PlanningActivityTypeFileAs],
		[Lines].[PlanningQualificationFileAs] AS [PlanningQualificationFileAs],
		[Lines].[PlanningQualificationFileAsOtherLang] AS [PlanningQualificationFileAsOtherLang],
		[BudgetLineWork].[QualificationID],
		[BudgetLineWork].[PlannedManHour],
		[BudgetLineWork].[ActualManHour],
		(CASE WHEN @VATRate IS NOT NULL AND @VATIsIncluded = 1 THEN
		[BudgetLineWork].[TotalWithVATPCPlannedManHour] ELSE
		[BudgetLineWork].[TotalPCPlannedManHour] END) AS [TotalPlanned],
		(CASE WHEN @VATRate IS NOT NULL AND @VATIsIncluded = 1 THEN
		[BudgetLineWork].[TotalWithVATPCActualManHour] ELSE
		[BudgetLineWork].[TotalPCActualManHour] END) AS [TotalActual],
		
		--Суммы без НДС
		[Lines].[RateWithoutVAT] AS [RateWithoutVAT],
		[BudgetLineWork].[TotalPCPlannedManHour] AS [TotalWithoutVATPCPlanned]
	FROM
		@WorkLines AS [Lines]
		INNER JOIN [BudgetLineWork]
			ON [Lines].[ID] = [BudgetLineWork].[ID]
		LEFT OUTER JOIN [Employee]
			ON [BudgetLineWork].[EmployeeID] = [Employee].[ID]
		LEFT OUTER JOIN [ActivityType]
			ON [BudgetLineWork].[ActivityTypeID] = [ActivityType].[ID]
		LEFT OUTER JOIN [Qualification]
			ON [BudgetLineWork].[QualificationID] = [Qualification].[ID]
	WHERE
		[BudgetLineWork].[BudgetID] = @ID
	ORDER BY
		[BudgetLineWork].[Order] ASC
	
	SELECT
		[QualificationID] AS [ID],
		[PlanningQualificationFileAs] AS [Name],
		[PlanningQualificationFileAsOtherLang] AS [NameOtherLang],
		MAX([RateWithoutVAT]) AS [MaxRate],
		@ID AS [BudgetID]
	FROM
		@WorkLines AS [Lines]
	GROUP BY 
		[QualificationID],
		[PlanningQualificationFileAs],
		[PlanningQualificationFileAsOtherLang]
	ORDER BY
		[MaxRate] DESC
		


	--Выгрузка линий затрат
	INSERT INTO @ExpenseLines
	SELECT
		[BudgetLineExpense].[ID],
		[BudgetLineExpense].[ExpenseCodeID],
		[ExpenseCode].[FileAs],
		(SELECT 
			ISNULL([PropertyValue].[Value],'') 
		FROM [PropertyValue] 
			JOIN [Property] ON [PropertyValue].[PropertyID] = [Property].[ID]
		WHERE [Property].[Class] = [ExpenseCode].[Class]
			AND [PropertyValue].[DocumentID] = [ExpenseCode].[ID]
			AND [Property].[Guid] = '00000001-0000-0000-0000-000000000000') AS [PlanningExpenseCodeFileAsOtherLang],
		(CASE 
			WHEN @BudgetCurrency <> [BudgetLineExpense].[CurrencyCode] THEN
			(CASE
				WHEN @nationalCurrency = [BudgetLineExpense].[CurrencyCode] THEN
					[BudgetLineExpense].[Price] / [BudgetLineExpense].[CurrencyRate]
				ELSE
					[BudgetLineExpense]. [Price] * [BudgetLineExpense].[CurrencyRate]
			END)
			ELSE
				[BudgetLineExpense]. [Price]
		END)
	FROM 
		[BudgetLineExpense]
		LEFT OUTER JOIN [ExpenseCode]
			ON [BudgetLineExpense].[ExpenseCodeID] = [ExpenseCode].[ID]
	WHERE
		[BudgetLineExpense].[BudgetID] = @ID


	SELECT
		[BudgetLineExpense].[ID],
		[BudgetLineExpense].[State],
		[BudgetLineExpense].[BudgetID],
		[BudgetLineExpense].[FileAs],
		(CASE WHEN [BudgetLineExpense].[BudgetLineGroupID] IS NULL THEN -2 ELSE [BudgetLineExpense].[BudgetLineGroupID] END) AS [BudgetLineGroupID],
		[BudgetLineExpense].[Order],
		[BudgetLineExpense].[StartDate],
		[BudgetLineExpense].[EndDate],
		(CASE 
			WHEN @BudgetCurrency <> [BudgetLineExpense].[CurrencyCode] THEN
			(CASE
				WHEN @nationalCurrency = [BudgetLineExpense].[CurrencyCode] THEN
					[BudgetLineExpense].[Price] / [BudgetLineExpense].[CurrencyRate]
				ELSE
					[BudgetLineExpense]. [Price] * [BudgetLineExpense].[CurrencyRate]
			END)
			ELSE
				[BudgetLineExpense]. [Price]
		END) AS [Price],
		[UOM].[Code] AS [UOMCode],
		[Employee].[FileAs] AS [PlanningEmployeeName],
		[Lines].[PlanningExpenseCodeFileAs] AS [PlanningExpenseCodeFileAs],
		[Lines].[PlanningExpenseCodeFileAsOtherLang] AS [PlanningExpenseCodeFileAsOtherLang],
		[Lines].[ExpenseCodeID] AS [ExpenseCodeID],
		[BudgetLineExpense].[PlannedAmount],
		[BudgetLineExpense].[ActualAmount],
		(CASE WHEN @VATRate IS NOT NULL THEN
		[BudgetLineExpense].[TotalWithVATPCPlannedExpense] ELSE
		[BudgetLineExpense].[TotalPCPlannedExpense] END) AS [TotalPlanned],
		(CASE WHEN @VATRate IS NOT NULL THEN
		[BudgetLineExpense].[TotalWithVATPCActualExpense] ELSE
		[BudgetLineExpense].[TotalPCActualExpense] END) AS [TotalActual],
		
		--Суммы без НДС
		[Lines].[PriceWithoutVAT] AS [PriceWithoutVAT],
		[BudgetLineExpense].[TotalPCPlannedExpense] AS [TotalWithoutVATPCPlanned],
		
		--Данные по плановым закупкам
		(CASE
			WHEN @BudgetCurrency <> [BudgetLineExpense].[CurrencyCode] THEN
			(CASE
				WHEN @nationalCurrency = [BudgetLineExpense].[CurrencyCode] THEN
					[BudgetLineExpense].[PlanPurchasePrice] / [BudgetLineExpense].[CurrencyRate]
				ELSE
					[BudgetLineExpense]. [PlanPurchasePrice] * [BudgetLineExpense].[CurrencyRate]
			END)
			ELSE
				[BudgetLineExpense]. [PlanPurchasePrice]
		END) AS [PriceWithoutVATPCPlannedPurchase],
		[BudgetLineExpense].[PlanPurchaseAmount] AS [PlannedPurchaseAmount]		

	FROM
		@ExpenseLines AS [Lines]
		INNER JOIN [BudgetLineExpense]
			ON [Lines].[ID] = [BudgetLineExpense].[ID]
		LEFT OUTER JOIN [Employee]
			ON [BudgetLineExpense].[EmployeeID] = [Employee].[ID]
		LEFT OUTER JOIN [ExpenseCode]
			ON [BudgetLineExpense].[ExpenseCodeID] = [ExpenseCode].[ID]
		LEFT OUTER JOIN [UOM]
			ON [BudgetLineExpense].[UOMID] = [UOM].[ID]
	ORDER BY
		[BudgetLineExpense].[Order] ASC
	
	SELECT
		[ExpenseCodeID] AS [ID],
		[PlanningExpenseCodeFileAs] AS [Name],
		[PlanningExpenseCodeFileAsOtherLang] AS [NameOtherLang],
		MAX([PriceWithoutVAT]) AS [MaxPrice],
		@ID AS [BudgetID]
	FROM
		@ExpenseLines AS [Lines]
	GROUP BY 
		[ExpenseCodeID],
		[PlanningExpenseCodeFileAs],
		[PlanningExpenseCodeFileAsOtherLang]
	ORDER BY
		[MaxPrice] DESC

	DECLARE
		@MinDate	TDate,
		@MaxDate	TDate
	SELECT
		@MinDate = MIN([BudgetLineWork].[StartDate]),
		@MaxDate = MAX([BudgetLineWork].[EndDate])
	FROM
		[BudgetLineWork]
	WHERE
		[BudgetLineWork].[BudgetID] = @ID

	--Таблицы для диаграммы Ганта по дням---------------------------------------------------------
	--Сформировать таблицу с днями
	DECLARE @DatesTable TABLE
	(
		[Date]			TDate,
		[PeriodName]	nvarchar(30)
	)
	INSERT INTO @DatesTable ([Date], [PeriodName])
	SELECT
		[Date],
		[PeriodName]
	FROM
		dbo.fnPjm7GetDatesListBetweenTwoDates(@MinDate,@MaxDate) AS [DatesByDay]
	ORDER BY
		[Date] ASC
	
	-- Количество дней для Диаграммы Ганта не должно превышать 249
	IF (@ReportName IS NOT NULL AND @ReportName LIKE '%GanttChartByDays%' 
		AND (SELECT Count([Date]) FROM @DatesTable) > 249)
	BEGIN
		RAISERROR('#BudgetTooManyDaysForGantt', 18, -1)
		RETURN 1
	END	

	--Выгрузка таблицы с датами
	SELECT 
		[Date], 
		[PeriodName],
		DAY([Date]) AS [Day]
	FROM 
		@DatesTable AS [DatesByDay]
	ORDER BY
		[Date] ASC

	--Выгрузка диаграммы Ганта по работам
	SELECT 
		[Date],
		(CASE WHEN [Date] BETWEEN [BudgetLineWork].[StartDate] AND [BudgetLineWork].[EndDate] THEN 1 ELSE 0 END) AS [DayHasWork],
		[BudgetLineWork].[ID] AS [BudgetLineWorkID]
	FROM 
		@DatesTable AS [BudgetLineWorkGanttByDay]
		INNER JOIN [BudgetLineWork]
			ON [BudgetLineWork].[BudgetID] = @ID
	Order By [Date] ASC
	
	--Вспомогательная таблица для разметки
	SELECT
		[Date],
		0 as [DayHasWork],
		[ID] as [BudgetLineGroupID]
	FROM
		@DatesTable AS [BudgetLineGroupGanttByDay]
		INNER JOIN [BudgetLineGroup]
			ON [BudgetLineGroup].[BudgetID] = @ID AND [GroupTypeID] = 1

	--Таблицы для диаграммы Ганта по неделям---------------------------------------------------------
	--Сформировать таблицу с неделями
	DECLARE @WeeksTable TABLE
	(
		[Date]			datetime,
		[Week]			int, 
		[Month]			int, 
		[Year]			int, 
		[PeriodName]	nvarchar(30)
	)
	INSERT INTO @WeeksTable ([Date], [Week], [Month], [Year], [PeriodName])
	SELECT
		[Date],
		[Week],
		[Month],
		[Year],
		[PeriodName]
	FROM
		dbo.fnPjm7GetWeeksListBetweenTwoDates(@MinDate,@MaxDate) AS [DatesByWeek]
	ORDER BY
		[Year] ASC,
		[Month] ASC,
		[Week] ASC

	--Выгрузка таблицы с неделями
	SELECT 
		[Week],
		[Month],
		[Year],
		[PeriodName]
	FROM 
		@WeeksTable AS [DatesByWeek]
	ORDER BY
		[Year] ASC,
		[Month] ASC,
		[Week] ASC

	--Выгрузка диаграммы Ганта по работам
	SELECT 
		[Week],
		[Month],
		[Year],
		(CASE WHEN ([Date] BETWEEN [BudgetLineWork].[StartDate] AND [BudgetLineWork].[EndDate]) OR
			([Date] < [BudgetLineWork].[StartDate] AND [Year] = YEAR([BudgetLineWork].[StartDate]) AND [Month] = MONTH([BudgetLineWork].[StartDate]) AND  [Week] = dbo.fnPjm7GetMonthWeekNumber([BudgetLineWork].[StartDate])) THEN 1 ELSE 0 END) AS [DayHasWork],
		[BudgetLineWork].[ID] AS [BudgetLineWorkID]
	FROM 
		@WeeksTable AS [BudgetLineWorkGanttByWeek]
		INNER JOIN [BudgetLineWork]
			ON [BudgetLineWork].[BudgetID] = @ID
	Order By [Date] ASC
	
	--Вспомогательная таблица для разметки
	SELECT
		[Week],
		[Month],
		[Year],
		0 as [DayHasWork],
		[ID] as [BudgetLineGroupID]
	FROM
		@WeeksTable AS [BudgetLineGroupGanttByWeek]
		INNER JOIN [BudgetLineGroup]
			ON [BudgetLineGroup].[BudgetID] = @ID AND [GroupTypeID] = 1

	SET DATEFIRST @DateFirstBefore
	RETURN 0
GO

GRANT EXECUTE ON ReportExcelBudget TO PUBLIC
GO