IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'Pjm6GetTimeIntervalFormatted')
	BEGIN
		PRINT 'Dropping Procedure Pjm6GetTimeIntervalFormatted'
		DROP  Procedure  Pjm6GetTimeIntervalFormatted
	END

GO

PRINT 'Creating Procedure Pjm6GetTimeIntervalFormatted'
GO
/******************************************************************************
**		File:	Pjm6GetTimeIntervalFormatted.sql
**		Name: 	Pjm6GetTimeIntervalFormatted
**		Desc:	returns formatted time interval
*******************************************************************************
**		Date:		Author:		Description:
**		-----		-------		-------------
**    	16.04.09		SPV		first version
*******************************************************************************/

CREATE Procedure Pjm6GetTimeIntervalFormatted
	@PeriodStart datetime,
	@PeriodEnd datetime,
	@LangCode TCode3,
	@CapsFirstLetters bit = 0,
	@Result varchar(64) OUTPUT	
AS
	DECLARE
		@YearL varchar(16),
		@QuarterL varchar(16),
		@FromL varchar(16),
		@ToL varchar(16),
		@Year1L varchar(16),
		@MonthL varchar(16),
		@Month tinyint,
		@DateDif int,
		@StartDay tinyint,
		@IsLeapYear tinyint,
		@SysLang nvarchar(32)


	SET @SysLang = @@LANGUAGE	-- remember default language code

	IF(@LangCode IS NULL OR @LangCode NOT IN ('en', 'ru'))
		SET @LangCode = 'ru'	-- set to russian language as default

	SET @DateDif = DATEDIFF(day, @PeriodStart, @PeriodEnd)
	SET @Month = DATEPART(month, @PeriodStart)
	SET @StartDay = DATEPART(day, @PeriodStart)
	IF DATEPART(year, @PeriodStart) % 4 = 0
		SET @IsLeapYear = 1
	ELSE
		SET @IsLeapYear = 0

	IF(@LangCode = 'en')
	BEGIN
		SET @YearL = 'year'
		SET @QuarterL = 'quarter'
		IF @CapsFirstLetters = 1
			SET @FromL = 'From'
		ELSE
			SET @FromL = 'from'
		SET @ToL = 'to'
		SET @Year1L = 'year'
		SET LANGUAGE ENGLISH
		SET @MonthL = DATENAME(month, @PeriodStart)
	END
	ELSE IF(@LangCode = 'ru')
	BEGIN
		SET @YearL = ''
		SET @QuarterL = ''
		IF @CapsFirstLetters = 1
			SET @FromL = ''
		ELSE
			SET @FromL = ''
		SET @ToL = ''
		SET @Year1L = ''
		SET LANGUAGE RUSSIAN
		SET @MonthL = DATENAME(month, @PeriodStart)
	END

	SET LANGUAGE @SysLang	-- set back default language code
	
	IF @StartDay = 1
	BEGIN
		IF	-- months with 30 days
			( @DateDif = 29 AND @Month in (4, 6, 9, 11))
			OR	-- months with 31 days
			( @DateDif = 30 AND @Month in (1, 3, 5, 7, 8, 10, 12))
			OR	-- february leap year
			( @DateDif = 28 AND @Month = 2 AND @IsLeapYear = 1)
			OR	-- february
			( @DateDif = 27 AND @Month = 2 AND @IsLeapYear = 0)
			SET @Result = @MonthL + ', ' + DATENAME(year, @PeriodStart) + ' ' + @YearL
		-- by quarters
		ELSE IF	-- 1st quarter
			( @DateDif = 89 AND @Month = 1 AND @IsLeapYear = 0)
			OR -- 1st quarter leap year
			( @DateDif = 90 AND @Month = 1 AND @IsLeapYear = 1)
			OR	-- 2nd quarter
			( @DateDif = 90 AND @Month = 4)
			OR	-- 3rd and 4th quarters
			( @DateDif = 91 AND @Month in (7, 10))
			SET @Result = CAST(DATEPART(quarter, @PeriodStart) AS VARCHAR(2)) + ' '	+ @QuarterL + ' ' + DATENAME(year, @PeriodStart) + ' ' + @Year1L
		-- by years			
		ELSE IF
			 ( @DateDif = 364 AND @Month = 1 AND @IsLeapYear = 0)
			 OR -- leap year
			 ( @DateDif = 365 AND @Month = 1 AND @IsLeapYear = 1)
			SET @Result = DATENAME(year, @Periodstart) + ' ' + @YearL
		-- from date to date
		ELSE
			SET @Result = @FromL + ' ' + CONVERT(varchar, @PeriodStart, 4) + ' ' + @ToL + ' ' + CONVERT(varchar, @PeriodEnd, 4)
	END
	ELSE
	BEGIN
		SET @Result = @FromL + ' ' + CONVERT(varchar, @PeriodStart, 4) + ' ' + @ToL + ' ' + CONVERT(varchar, @PeriodEnd, 4)
	END
RETURN 0
GO

GRANT EXEC ON Pjm6GetTimeIntervalFormatted TO PUBLIC
GO
