IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'ReportMovementOfFinancialDocumentsHeader')
	BEGIN
		PRINT 'Dropping Procedure ReportMovementOfFinancialDocumentsHeader'
		DROP  Procedure  ReportMovementOfFinancialDocumentsHeader
	END
GO

PRINT 'Creating Procedure ReportMovementOfFinancialDocumentsHeader'
GO

/******************************************************************************
**		File: 
**		Name: 
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: MAN
**		Date: 26/08/2010 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		26.08.2010	MAN				First Version
*******************************************************************************/
CREATE PROCEDURE ReportMovementOfFinancialDocumentsHeader
	@CompanyID		  TIdentifier,
	@CostCenterID	  TIdentifier,
	@ProjectManagerID TIdentifier,
	@ProjectID		  TIdentifier,
	@PeriodName		  TName

AS
   SET NOCOUNT ON
   DECLARE
          @CompanyName TName,
          @Value TVariant,	
          @ManagerName TName,
          @CostCenterName TName,
          @ProjectName TName,
          @DatePeriod TName,
          @UserName TName,
          @UserID TIdentifier

   SET @CompanyName = '()'
   SET @ManagerName  = '()'
   SET @CostCenterName  = '()'
   SET @ProjectName  = '()'
   
   -- 
   -- 
   SELECT
      @CompanyName =  [Company].[FileAs]
   FROM
       [Company]
   WHERE 
		(
			@ProjectID IS NULL
			AND
			[Company].[ID] = @CompanyID
		)
		OR
		(
			@ProjectID IS NOT NULL
			AND
			EXISTS
			(
				SELECT
					[Project].[SupplierCompanyID]
				FROM
					[Project]
				WHERE
					[Project].[SupplierCompanyID] = [Company].[ID]
					AND
					[Project].[ID] = @ProjectID
			)
		)
	
	SELECT TOP(1)
		@ProjectName = [Project].[FileAs]
	FROM
		[Project]
	WHERE
		[Project].[ID] = @ProjectID
	
	--  
   SELECT TOP(1)
     @ManagerName = [Employee].[FileAs]
   FROM
         [Employee]
   WHERE
		(
			@ProjectID IS NULL
			AND
			[Employee].[ID] = @ProjectManagerID
		)
		OR
		(
			@ProjectID IS NOT NULL
			AND
			EXISTS
			(
				SELECT
					[Project].[ProjectManagerEmployeeID]
				FROM
					[Project]
				WHERE
					[Project].[ProjectManagerEmployeeID] = [Employee].[ID]
					AND
					[Project].[ID] = @ProjectID
			)
		)
  
  --  
   SELECT TOP(1)
	 @CostCenterName = [CostCenter].[FileAs]
   FROM
         [CostCenter]
   WHERE 
	( 
		@ProjectID IS NULL
		AND
        [CostCenter].[ID] = @CostCenterID
     )
     OR
		(
			@ProjectID IS NOT NULL
			AND
			EXISTS
			(
				SELECT
					[Project].[CostCenterID]
				FROM
					[Project]
				WHERE
					[Project].[CostCenterID] = [CostCenter].[ID]
					AND
					[Project].[ID] = @ProjectID
			)
		)
   
   SET @UserID = dbo.adfGetCurrentUserID()
	
	-- 
   SELECT TOP(1)
       @UserName =  [Trustee].[DisplayName]
   FROM 
         [Trustee]
   WHERE 
         [Trustee].[ID] = @UserID
 
   SELECT
         @CompanyName	 AS [CompanyName],
         @ManagerName	 AS [ManagerName],
         @ProjectName	 AS [ProjectName],
         @CostCenterName AS [CostCenterName],
         @PeriodName	 AS [DatePeriod],
         @UserName		 AS [UserName]

RETURN 0
GO

GRANT EXEC ON ReportMovementOfFinancialDocumentsHeader TO PUBLIC

GO
