IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'ReportPaymentsEfficiencyByClients')
	BEGIN
		PRINT 'Dropping Procedure ReportPaymentsEfficiencyByClients'
		DROP  Procedure  ReportPaymentsEfficiencyByClients
	END
GO

PRINT 'Creating Procedure ReportPaymentsEfficiencyByClients'
GO
/******************************************************************************
**		File: ReportPaymentsEfficiencyByClients.sql
**		Name: ReportPaymentsEfficiencyByClients
**
**         
**		Return values:
**
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: avgusta.bekthina
**		Date: 15.12.2014
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:		Description:
**		--------	--------	-------------------------------------------
**		16.12.14	SAP				
*******************************************************************************/
CREATE PROCEDURE [dbo].[ReportPaymentsEfficiencyByClients]
	@CompanyID TIdentifier,			-- ID 
    @CustomerID TIdentifier,		-- ID 
	@CurrencySelector int,			--  (0 - , 1 - )
	@SumByCompanyGroups int,		--  (0 - , 1 - )
	@CostCenterSelector int,		--   (0 -  , 1 -  )
	@CostCenterID TIdentifier,		-- ID  
	--@Displayed int,				--  (0 -   , 1 -   )
	--@ServiceTypeSelector int,		--   (0 -  , 1 -  , 2 -   )
	--@InvoiceSelector int,			--  (0 -  , 1 -  )
	@StateOfFinanceDocuments int,	--  .  (0 -  , 1 -  , 2 -  )	
	@PeriodSelector int,			--  (0 -  , 1 -  , 2 -  )
	@PeriodStart date,				--   
	@PeriodEnd date,				--   
	@PeriodLenghtSelector int,		--   (0 - , 1 - , 2 - )
	@ScaleSelector int				--   (0 - , 1 - , 2 - , 3 - )					

AS
	SET NOCOUNT ON
	
	--***********************************************************
	--  
	IF @CustomerID IS NULL
	BEGIN
		RAISERROR('#CustomerCompanyIsNotSelected', 16, -1)
		RETURN 1
	END

	--***********************************************************
	--  
	IF  dbo.adfHasAccessRightByName('ViewPaymentsEfficiencyByClientsReport') = 0 
	BEGIN
		  RAISERROR('#NotEnoughPermissions', 18, -1)
		  RETURN 1
	END


	--***********************************************************
	--  		
	DECLARE 
		@TmpDateSelector INT,
		@PeriodName TName
	SELECT
		@TmpDateSelector = 
			CASE @PeriodSelector --      
				WHEN 0 THEN 4
				WHEN 1 THEN 5
				WHEN 2 THEN 100									
			END					
	EXEC ReportGetPeriod 
		@TmpDateSelector,	--   
		1,					--      
		@PeriodStart OUTPUT,--  
		@PeriodEnd	 OUTPUT,--  
		@PeriodName  OUTPUT--  	



	DECLARE						
		@CompanyAverageEfficiency int,
		@Value TVariant

	EXEC adfParameterGet 'StandardEfficiency', 'Pjm7', @Value OUTPUT
	SET @CompanyAverageEfficiency = CAST(@Value as decimal(18,2)) * 100


	-- 
	DECLARE @MainTable TABLE
	(
		[Order]						INT,
		PeriodStartDate				date,
		PeriodEndDate				date,
		PeriodName					TName,
		Utilization					decimal(18,4),
		Realization					decimal(18,4),
		Efficiency					decimal(18,4),
		Deviation					decimal(18,4),
		PositiveDurationDeviation	decimal(18,4),
		NegativeDurationDeviation	decimal(18,4),
		Cost						decimal(18,4),
		PositiveCostDeviation		decimal(18,4),
		NegativeCostDeviation		decimal(18,4),
		AverageRate					decimal(18,4)
	)



	--***********************************************************
	--  
	INSERT INTO @MainTable
	SELECT
		0,
		PeriodStartDate,
		PeriodEndDate,
		PeriodName,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL	
	FROM
		[dbo].[fnReportPaymentsEfficiencyByClientsGetPeriod]
		(				
			@PeriodStart,
			@PeriodEnd,			
			@PeriodLenghtSelector
		)



	--**********************************************************
	--   
	UPDATE @MainTable
	SET
		[Utilization] = [Time].[Utilization],
		[Realization] = [Time].[Realization],
		[Efficiency] = [Time].[Efficiency],
		[Deviation] = [Time].[Deviation],
		[PositiveDurationDeviation] = [Time].[PositiveDurationDeviation],
		[NegativeDurationDeviation] = [Time].[NegativeDurationDeviation]
	FROM
		[dbo].[fnReportPaymentsEfficiencyByClientsGetTime]
		(
			@CompanyID,
			@CustomerID,
			@StateOfFinanceDocuments,
			@PeriodStart,
			@PeriodEnd,
			@CompanyAverageEfficiency,
			@PeriodLenghtSelector,
			@CostCenterSelector,
			@CostCenterID,
			@SumByCompanyGroups
		) as [Time]
		LEFT JOIN @MainTable
			ON [@MainTable].[PeriodStartDate] <= [Time].[TimeEntryStartDate] 
			AND [@MainTable].[PeriodEndDate] > [Time].[TimeEntryStartDate]



	--**********************************************************
	--    
	UPDATE @MainTable
	SET
		[Cost] = [Money].[Cost],
		[PositiveCostDeviation] = [Money].[PositiveCostDeviation],
		[NegativeCostDeviation] = [Money].[NegativeCostDeviation]
	FROM
		[dbo].[fnReportPaymentsEfficiencyByClientsGetMoney]
		(		
			@CompanyID,				
			@CustomerID,
			@CurrencySelector,					
			@StateOfFinanceDocuments,			
			@PeriodStart,						
			@PeriodEnd,
			@PeriodLenghtSelector,
			@CostCenterSelector,
			@CostCenterID,
			@SumByCompanyGroups
		) AS [Money]
		LEFT JOIN @MainTable
			ON [@MainTable].[PeriodStartDate] <= [Money].[TimeEntryStartDate] 
			AND [@MainTable].[PeriodEndDate] > [Money].[TimeEntryStartDate]

	UPDATE @MainTable
	SET 
		[AverageRate] = 
			CASE 
				WHEN ISNULL([MT].[Utilization],0) = 0 THEN NULL
				ELSE ROUND([MT].[Cost] / [MT].[Utilization],2)
			END		
	FROM 
		@MainTable AS [MT]


	--**********************************************************
	--,   
	
	INSERT INTO @MainTable
	SELECT
		1,
		NULL,
		NULL,
		':',		
		ROUND(SUM(ISNULL([MT].[Utilization],0))/(SELECT CASE COUNT(1) WHEN 0 THEN 1 ELSE COUNT(1) END FROM @MainTable WHERE [Order] = 0 AND [Utilization] IS NOT NULL),2),
		ROUND(SUM(ISNULL([MT].[Realization],0))/(SELECT CASE COUNT(1) WHEN 0 THEN 1 ELSE COUNT(1) END FROM @MainTable WHERE [Order] = 0 AND [Realization] IS NOT NULL),2),
		((SUM(ISNULL([MT].[Realization],0)) - SUM(ISNULL([MT].[PositiveDurationDeviation],0)))/CASE SUM(ISNULL([MT].[Utilization],0)) WHEN 0 THEN 1 ELSE SUM(ISNULL([MT].[Utilization],0)) END)*100,
		((SUM(ISNULL([MT].[Realization],0)) - SUM(ISNULL([MT].[PositiveDurationDeviation],0)))/CASE SUM(ISNULL([MT].[Utilization],0)) WHEN 0 THEN 1 ELSE SUM(ISNULL([MT].[Utilization],0)) END)*100 - @CompanyAverageEfficiency,
		NULL,
		NULL,
		ROUND(SUM(ISNULL([MT].[Cost],0))/(SELECT CASE COUNT(1) WHEN 0 THEN 1 ELSE COUNT(1) END FROM @MainTable WHERE [Order] = 0 AND [Cost] IS NOT NULL),2),
		NULL,
		NULL,
		ROUND(SUM(ISNULL([MT].[AverageRate],0))/(SELECT CASE COUNT(1) WHEN 0 THEN 1 ELSE COUNT(1) END FROM @MainTable WHERE [Order] = 0 AND [AverageRate] IS NOT NULL),2)
	FROM 
		@MainTable AS [MT]
	WHERE
		[MT].[Order] = 0

	INSERT INTO @MainTable
	SELECT
		2,
		NULL,
		NULL,
		':',
		SUM(ISNULL([MT].[Utilization],0)),
		SUM(ISNULL([MT].[Realization],0)),
		NULL,
		NULL,
		NULL,
		NULL,
		SUM(ISNULL([MT].[Cost],0)),
		NULL,
		NULL,
		NULL
	FROM 
		@MainTable AS [MT]
	WHERE
		[MT].[Order] = 0

	INSERT INTO @MainTable
	SELECT
		3,
		NULL,
		NULL,
		' :',
		NULL,
		SUM(ISNULL([MT].[PositiveDurationDeviation],0)),
		NULL,
		NULL,
		NULL,
		NULL,
		SUM(ISNULL([MT].[PositiveCostDeviation],0)),
		NULL,
		NULL,
		NULL
	FROM 
		@MainTable AS [MT]

	INSERT INTO @MainTable
	SELECT
		4,
		NULL,
		NULL,
		' :',
		NULL,
		SUM(ISNULL([MT].[NegativeDurationDeviation],0)),
		NULL,
		NULL,
		NULL,
		NULL,
		SUM(ISNULL([MT].[NegativeCostDeviation],0)),
		NULL,
		NULL,
		NULL
	FROM 
		@MainTable AS [MT]




	--**********************************************************
	--SmartValue	
	DECLARE 
		@Scale int,
		@MaxValue decimal(18,2)

	SELECT
		@Scale = [dbo].[fnReportCalculateSmartValue](@ScaleSelector,ISNULL(MAX([Cost]),0))		
	FROM
		@MainTable	
	

	UPDATE @MainTable
	SET [Cost] = [Cost]/@Scale
		
	DECLARE
		@AverageCost decimal(18,2),
		@AverageAverageRate DECIMAL (18,2),
		@PeriodsCount INT

	SELECT
		@PeriodsCount = COUNT(1)
	FROM
		@MainTable
	WHERE
		[Order] = 0

	SELECT
		@AverageCost = (SELECT [Cost] FROM @MainTable WHERE [Order] = 1),
		@AverageAverageRate = (SELECT [AverageRate] FROM @MainTable WHERE [Order] = 1)
	FROM
		@MainTable


	exec dbo.ReportPaymentsEfficiencyByClientsHeader @CompanyID, @CustomerID, @SumByCompanyGroups, 
		@CurrencySelector, @CostCenterSelector,
		@CostCenterID,@StateOfFinanceDocuments,@PeriodName,@PeriodLenghtSelector, @ScaleSelector, 
		@Scale, @CompanyAverageEfficiency, @AverageCost, @AverageAverageRate,@PeriodsCount

	SELECT * FROM @MainTable

RETURN 0
GO

GRANT EXEC ON ReportPaymentsEfficiencyByClients TO PUBLIC
GO

