IF EXISTS (SELECT * FROM sysobjects WHERE TYPE = 'P' AND name = 'ReportPaymentsEfficiencyByClientsHeader')
	BEGIN
		PRINT 'Dropping Procedure ReportPaymentsEfficiencyByClientsHeader'
		DROP  PROCEDURE  ReportPaymentsEfficiencyByClientsHeader
	END
GO
PRINT 'Creating Procedure ReportPaymentsEfficiencyByClientsHeader'
GO
/******************************************************************************
**		File: ReportPaymentsEfficiencyByClientsHeader.sql
**		Name: ReportPaymentsEfficiencyByClientsHeader
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: avgusta.bekhtina
**		Date: 15.12.2014 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:		Description:
**		--------	--------	-----------------------------------------------
**		16.12.14	SAP				
*******************************************************************************/
CREATE PROCEDURE [dbo].[ReportPaymentsEfficiencyByClientsHeader]	
	@CompanyID				TIdentifier,	-- ID 
    @CustomerID				TIdentifier,	-- ID 
	@SumByCompanyGroups		int,			--  (0 - , 1 - )	
	@CurrencySelector		int,			--  (0 - , 1 - )
	@CostCenterSelector		int,			--   (0 -  , 1 -  )
	@CostCenterID			TIdentifier,	-- ID  
	@StateOfFinanceDocuments int,			--  .  (0 -  , 1 -  , 2 -  )	
	@PeriodName				TName,			--  
	@PeriodLengthSelector	int,			--   (0 - , 1 - , 2 - )
	@ScaleSelector			INT,			--   (0 - , 1 - , 2 - , 3 - )						  
	@Scale					INT,
	@CompanyAverageEfficiency INT, 
	@AverageCost decimal(18,2),
	@AverageAverageRate DECIMAL (18,2),
	@PeriodsCount INT
AS
SET NOCOUNT ON

	DECLARE 
		@CompanyName				TName,
		@CustomerName				TName,
		@SumByCompany				nvarchar(max),
		@CurrencyCode				TCode3,	
		@CurrencyWithPrefix			TName,
		@CostCenterName				TName,		
		@PeriodLengthName			TName,
		@UserID						TIdentifier,
		@UserName					TName,		
		@value						TVariant,
		@StateOfFinanceDocumentsText NVARCHAR(100)

		
	--  	
	IF @CompanyID IS NULL
		SET @CompanyName = '()'
	ELSE
		SELECT
			@CompanyName = [Company].[FileAs]
		FROM
			[Company]
		WHERE
			[Company].[ID] = @CompanyID


	-- 	
	SELECT
		@CustomerName = [Company].[FileAs]
	FROM
		[Company]
	WHERE
		[Company].[ID] = @CustomerID

	
	--   
	SELECT
		@CurrencyCode = 
			CASE @CurrencySelector
				WHEN 0 THEN [dbo].[fnPjm5GetBaseCurrency]()
				WHEN 1 THEN [dbo].[fnPjm5GetNationalCurrency]()
			END
			
	SELECT
		@CurrencyWithPrefix =
			CASE @ScaleSelector
				WHEN 0 THEN @CurrencyCode
				WHEN 1 THEN '. ' + @CurrencyCode
				WHEN 2 THEN '. ' + @CurrencyCode
				WHEN 3 THEN
					CASE @Scale
						WHEN 1000 THEN '. ' + @CurrencyCode
						WHEN 1000000 THEN '. ' + @CurrencyCode
						WHEN 1000000000 THEN '. ' + @CurrencyCode
						ELSE @CurrencyCode
					END 
			END
		

	-- 
	IF @CostCenterSelector = 0
	BEGIN
		SET @CostCenterName = NULL
	END
	ELSE
	BEGIN
		IF @CostCenterID IS NULL
			SET @CostCenterName = '( )'
		ELSE
			SELECT 
				@CostCenterName = [CostCenter].[FileAs]		
			FROM
				[CostCenter]
			WHERE 
				[ID] = @CostCenterID		
	END

	-- . 
	SELECT
		@StateOfFinanceDocumentsText = 
			CASE @StateOfFinanceDocuments
				WHEN 0 THEN N'    '
				WHEN 1 THEN N'     '
				WHEN 2 THEN N'        '
			END

	
	SELECT		
		@PeriodLengthName = 
			CASE @PeriodLengthSelector
				WHEN 0 THEN ''
				WHEN 1 THEN ''
				WHEN 2 THEN ''
			END	

		
	SELECT
		@UserID = [Trustee].[ID],
		@UserName = [Trustee].[DisplayName]
	FROM
		[Trustee]
	WHERE
		[Trustee].[ID] = dbo.adfGetCurrentUserID()	



	SELECT
		@CompanyName AS [CompanyName],
		@CustomerName AS [CustomerName],		
		@SumByCompanyGroups AS [SumByCompanyGroups],		
		@CurrencyWithPrefix AS [CurrencyCode],
		@CostCenterSelector AS [CostCenterSelector],			
		@CostCenterName As [CostCenterName],
		@PeriodName AS [PeriodName],
		@PeriodLengthName AS [PeriodLengthName],
		@UserID AS [UserID],
		@UserName AS [UserName],
		@CompanyAverageEfficiency AS [CompanyAverageEfficiency],
		@AverageCost AS [AverageCost],
		@AverageAverageRate AS [AverageAverageRate],
		@StateOfFinanceDocumentsText AS [StateOfFinanceDocuments],
		@PeriodsCount AS [PeriodsCount]


RETURN 0
GO

GRANT EXEC ON ReportPaymentsEfficiencyByClientsHeader TO PUBLIC
GO