IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'VistaCalculationReport')
	BEGIN
		PRINT 'Dropping Procedure VistaCalculationReport'
		DROP  Procedure  VistaCalculationReport
	END

GO

PRINT 'Creating Procedure VistaCalculationReport'
GO

CREATE Procedure VistaCalculationReport
		@StateSelector INT,
		@DateSelector INT,
		@PeriodStart TDate,
		@PeriodEnd TDate

AS
	SET NOCOUNT ON

DECLARE
		@timeClass TClass,
		@expenseClass TClass,
		@phoneClass TClass,
		@requestClass TClass,
		@language TLanguage
		
SET @language = dbo.adfGetLanguage()	
SELECT
		@timeClass = dbo.adfGetClassByName('Pjm5.TimeEntry'),
		@expenseClass = dbo.adfGetClassByName('Pjm5.Expense'),
		@phoneClass = dbo.adfGetClassByName('Pjm5.PhoneCall'),
		@requestClass = dbo.adfGetClassByName('Pjm5.ServiceRequest')
		
		IF @DateSelector = 1
		BEGIN
			SELECT
				@PeriodStart = CAST(CONVERT(varchar(8), GETDATE(), 112) AS datetime),
				@PeriodStart = DATEADD(day, 1-DATEPART(day, @PeriodStart), @PeriodStart),
				@PeriodStart = DATEADD(month, -1, @PeriodStart),
				@PeriodEnd = DATEADD(day, -1, DATEADD(month, 1, @PeriodStart))
		END
		ELSE IF @DateSelector = 2 -- current month
			SELECT
				@PeriodStart = CAST(CONVERT(varchar(8), GETDATE(), 112) AS datetime),
				@PeriodStart = DATEADD(day, 1-DATEPART(day, @PeriodStart), @PeriodStart),
				@PeriodEnd = DATEADD(day, -1, DATEADD(month, 1, @PeriodStart))

	;
	WITH Detail([CalculationLineID], [Amount], [Price], [Total], [TotalWithVAT], [VAT], [Code], [ServiceType], [Employee], [Division], [VATState], [ServiceRequestNumber]) AS
	(
	SELECT
		[CalculationLine].[ID],
		[CalculationLine].[Amount],
		CASE [Project].[ApplyVAT] * 2 + [CalculationLine].[VATIsIncluded] 
			WHEN 0 --   , .  
				THEN [CalculationLine].[Price]
			WHEN 1 --   , .  
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT]) / [CalculationLine].[Amount]
			WHEN 2 --  c , .  
				THEN [CalculationLine].[Price]
			WHEN 3 --  c , . c 
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT]) / [CalculationLine].[Amount]
		END as [Price],
		CASE [Project].[ApplyVAT] * 2 + [CalculationLine].[VATIsIncluded] 
			WHEN 0 --   , .  
				THEN [CalculationLine].[Price] * [CalculationLine].[Amount]
			WHEN 1 --   , .  
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT])
			WHEN 2 --  c , .  
				THEN [CalculationLine].[Price] * [CalculationLine].[Amount]
			WHEN 3 --  c , . c 
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT])
		END as [Total],
		CASE [Project].[ApplyVAT] * 2 + [CalculationLine].[VATIsIncluded] 
			WHEN 0 --   , .  
				THEN [CalculationLine].[Price] * [CalculationLine].[Amount]
			WHEN 1 --   , .  
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT])
			WHEN 2 --  c , .  
				THEN [CalculationLine].[Price] * [CalculationLine].[Amount] * (1 + [Calculation].[VATRate])
			WHEN 3 --  c , . c 
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price])
		END as [TotalWithVAT],
		CASE [Project].[ApplyVAT] * 2 + [CalculationLine].[VATIsIncluded] 
			WHEN 0 --   , .  
				THEN 0
			WHEN 1 --   , .  
				THEN 0
			WHEN 2 --  c , .  
				THEN [CalculationLine].[Price] * [CalculationLine].[Amount] * [Calculation].[VATRate]
			WHEN 3 --  c , . c 
				THEN [CalculationLine].[VAT]
		END as [VAT],
		[ExpenseCode].[Code],
		NULL,
		[Employee].[Initials],
		[Division].[FileAs],
		[Project].[ApplyVAT] * 2 + [CalculationLine].[VATIsIncluded] as [VATState],
		NULL as [ServiceRequestNumber]
	 FROM
		[Calculation]
		INNER JOIN [CalculationLine]
			ON [Calculation].[ID] = [CalculationLine].[CalculationID]  AND 
				[Calculation].[DocumentClass] = @phoneClass
		INNER JOIN [Project]
			ON [Project].[ID]=[Calculation].[ProjectID]
		INNER JOIN [Expense]
			ON [CalculationLine].[DocumentClass] = [Expense].[Class] AND
				[Expense].[ID] = [CalculationLine].[DocumentID]
		INNER JOIN [PhoneCall]
			ON [PhoneCall].[ExpenseID] = [Expense].[ID]
		INNER JOIN [Employee]	
			ON [Employee].[ID] = [Expense].[EmployeeID]
		LEFT OUTER JOIN [ExpenseCode]
			ON [ExpenseCode].[ID] = [Expense].[ExpenseCodeID]
		LEFT OUTER JOIN [Division]
			ON [Expense].[DivisionID] = [Division].[ID]
	WHERE 
		@DateSelector = 0 OR [Calculation].[Date] BETWEEN @PeriodStart AND @PeriodEnd 

	UNION ALL

	SELECT
		[CalculationLine].[ID],
		[CalculationLine].[Amount],
		CASE [Project].[ApplyVAT] * 2 + [CalculationLine].[VATIsIncluded] 
			WHEN 0 --   , .  
				THEN [CalculationLine].[Price]
			WHEN 1 --   , .  
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT]) / [CalculationLine].[Amount]
			WHEN 2 --  c , .  
				THEN [CalculationLine].[Price]
			WHEN 3 --  c , . c 
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT]) / [CalculationLine].[Amount]
		END as [Price],
		CASE [Project].[ApplyVAT] * 2 + [CalculationLine].[VATIsIncluded] 
			WHEN 0 --   , .  
				THEN [CalculationLine].[Price] * [CalculationLine].[Amount]
			WHEN 1 --   , .  
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT])
			WHEN 2 --  c , .  
				THEN [CalculationLine].[Price] * [CalculationLine].[Amount]
			WHEN 3 --  c , . c 
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT])
		END as [Total],
		CASE [Project].[ApplyVAT] * 2 + [CalculationLine].[VATIsIncluded] 
			WHEN 0 --   , .  
				THEN [CalculationLine].[Price] * [CalculationLine].[Amount]
			WHEN 1 --   , .  
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT])
			WHEN 2 --  c , .  
				THEN [CalculationLine].[Price] * [CalculationLine].[Amount] * (1 + [Calculation].[VATRate])
			WHEN 3 --  c , . c 
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price])
		END as [TotalWithVAT],
		CASE [Project].[ApplyVAT] * 2 + [CalculationLine].[VATIsIncluded] 
			WHEN 0 --   , .  
				THEN 0
			WHEN 1 --   , .  
				THEN 0
			WHEN 2 --  c , .  
				THEN [CalculationLine].[Price] * [CalculationLine].[Amount] * [Calculation].[VATRate]
			WHEN 3 --  c , . c 
				THEN [CalculationLine].[VAT]
		END as [VAT],
		[ExpenseCode].[Code],
		NULL,
		[Employee].[Initials],
		[Division].[FileAs],
		[Project].[ApplyVAT] * 2 + [CalculationLine].[VATIsIncluded] as [VATState],
		NULL as [ServiceRequestNumber]
	 FROM
		[Calculation]
		INNER JOIN [CalculationLine]
			ON [Calculation].[ID]= [CalculationLine].[CalculationID] AND 
				[Calculation].[DocumentClass] = @expenseClass
		INNER JOIN [Project]
			ON [Project].[ID]=[Calculation].[ProjectID]
		INNER JOIN [Expense]
			ON [CalculationLine].[DocumentClass] = [Expense].[Class] AND
				[Expense].[ID] = [CalculationLine].[DocumentID]
		INNER JOIN [Employee]	
			ON [Employee].[ID] = [Expense].[EmployeeID]
		LEFT OUTER JOIN [ExpenseCode]
			ON [ExpenseCode].[ID] = [Expense].[ExpenseCodeID]
		LEFT OUTER JOIN [Division]
			ON [Expense].[DivisionID] = [Division].[ID]
	WHERE 
		@DateSelector = 0 OR [Calculation].[Date] BETWEEN @PeriodStart AND @PeriodEnd 
	
	UNION ALL

	SELECT
		MIN([CalculationLine].[ID]),
		SUM([CalculationLine].[Amount]),
		CASE [Project].[ApplyVAT] * 2 + [CalculationLine].[VATIsIncluded] 
			WHEN 0 --   , .  
				THEN [CalculationLine].[Price]
			WHEN 1 --   , .  
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT]) / [CalculationLine].[Amount]
			WHEN 2 --  c , .  
				THEN [CalculationLine].[Price]
			WHEN 3 --  c , . c 
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT]) / [CalculationLine].[Amount]
		END as [Price],
		SUM(CASE [Project].[ApplyVAT] * 2 + [CalculationLine].[VATIsIncluded] 
			WHEN 0 --   , .  
				THEN [CalculationLine].[Price] * [CalculationLine].[Amount]
			WHEN 1 --   , .  
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT])
			WHEN 2 --  c , .  
				THEN [CalculationLine].[Price] * [CalculationLine].[Amount]
			WHEN 3 --  c , . c 
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT])
		END) as [Total],
		SUM(CASE [Project].[ApplyVAT] * 2 + [CalculationLine].[VATIsIncluded] 
			WHEN 0 --   , .  
				THEN [CalculationLine].[Price] * [CalculationLine].[Amount]
			WHEN 1 --   , .  
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT])
			WHEN 2 --  c , .  
				THEN [CalculationLine].[Price] * [CalculationLine].[Amount] * (1 + [Calculation].[VATRate])
			WHEN 3 --  c , . c 
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price])
		END) as [TotalWithVAT],
		SUM(CASE [Project].[ApplyVAT] * 2 + [CalculationLine].[VATIsIncluded] 
			WHEN 0 --   , .  
				THEN 0
			WHEN 1 --   , .  
				THEN 0
			WHEN 2 --  c , .  
				THEN [CalculationLine].[Price] * [CalculationLine].[Amount] * [Calculation].[VATRate]
			WHEN 3 --  c , . c 
				THEN [CalculationLine].[VAT]
		END) as [VAT],
		NULL,
		NULL,
		[Employee].[Initials],
		[Division].[FileAs],
		[Project].[ApplyVAT] * 2 + [CalculationLine].[VATIsIncluded] as [VATState],
		NULL as [ServiceRequestNumber]
	FROM
		[Calculation]
		INNER JOIN [Project]
			ON [Project].[ID]=[Calculation].[ProjectID]
		INNER JOIN [CalculationLine]
			ON [Calculation].[ID]= [CalculationLine].[CalculationID] AND 
				[Calculation].[DocumentClass] = @timeClass
		INNER JOIN [TimeEntry]
			ON [CalculationLine].[DocumentClass] = [TimeEntry].[Class] AND
				[TimeEntry].[ID] = [CalculationLine].[DocumentID]
		INNER JOIN [Employee]	
			ON [Employee].[ID] = [TimeEntry].[EmployeeID]
		LEFT OUTER JOIN [Division]
			ON [Division].[ID] = [TimeEntry].[DivisionID]
	WHERE 
		@DateSelector = 0 OR [Calculation].[Date] BETWEEN @PeriodStart AND @PeriodEnd 
	GROUP BY
		[Project].[ApplyVAT] * 2 + [CalculationLine].[VATIsIncluded],
		CASE [Project].[ApplyVAT] * 2 + [CalculationLine].[VATIsIncluded] 
			WHEN 0 --   , .  
				THEN [CalculationLine].[Price]
			WHEN 1 --   , .  
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT]) / [CalculationLine].[Amount]
			WHEN 2 --  c , .  
				THEN [CalculationLine].[Price]
			WHEN 3 --  c , . c 
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT]) / [CalculationLine].[Amount]
		END,		
		[Calculation].[ID], 
		[Employee].[ID],
		[CalculationLine].[Price],
		[Division].[FileAs],
		[Employee].[Initials],
		[CalculationLine].[VATIsIncluded],
		[Calculation].[VATRate]
		
	UNION ALL

	SELECT
		[CalculationLine].[ID],
		[ServiceRequestLine].[FactAmount],
		CASE [Project].[ApplyVAT] * 2 + [ServiceRequest].[VATIsIncluded] 
			WHEN 0 --   , .  
				THEN [ServiceRequestLine].[Price]
			WHEN 1 --   , .  
				THEN ([ServiceRequestLine].[Price] / (1 + [Calculation].[VATRate]))
			WHEN 2 --  c , .  
				THEN [ServiceRequestLine].[Price]
			WHEN 3 --  c , . c 
				THEN ([ServiceRequestLine].[Price] / (1 + [Calculation].[VATRate]))
		END as [Price],
		CASE [Project].[ApplyVAT] * 2 + [ServiceRequest].[VATIsIncluded] 
			WHEN 0 --   , .  
				THEN [ServiceRequestLine].[Price] * [ServiceRequestLine].[FactAmount]
			WHEN 1 --   , .  
				THEN ([ServiceRequestLine].[Price] * [ServiceRequestLine].[FactAmount] / (1 + [Calculation].[VATRate]))
			WHEN 2 --  c , .  
				THEN [ServiceRequestLine].[Price] * [ServiceRequestLine].[FactAmount]
			WHEN 3 --  c , . c 
				THEN ([ServiceRequestLine].[Price] * [ServiceRequestLine].[FactAmount] / (1 + [Calculation].[VATRate]))
		END as [Total],
		CASE [Project].[ApplyVAT] * 2 + [ServiceRequest].[VATIsIncluded] 
			WHEN 0 --   , .  
				THEN [ServiceRequestLine].[Price] * [ServiceRequestLine].[FactAmount]
			WHEN 1 --   , .  
				THEN ([ServiceRequestLine].[Price] * [ServiceRequestLine].[FactAmount] / (1 + [Calculation].[VATRate]))
			WHEN 2 --  c , .  
				THEN [ServiceRequestLine].[Price] * [ServiceRequestLine].[FactAmount] * (1 + [Calculation].[VATRate])
			WHEN 3 --  c , . c 
				THEN ([ServiceRequestLine].[Price] * [ServiceRequestLine].[FactAmount])
		END as [TotalWithVAT],
		CASE [Project].[ApplyVAT] * 2 + [ServiceRequest].[VATIsIncluded] 
			WHEN 0 --   , .  
				THEN 0
			WHEN 1 --   , .  
				THEN 0
			WHEN 2 --  c , .  
				THEN [ServiceRequestLine].[Price] * [ServiceRequestLine].[FactAmount] * [Calculation].[VATRate]
			WHEN 3 --  c , . c 
				THEN [ServiceRequestLine].[Price] * [ServiceRequestLine].[FactAmount] * [Calculation].[VATRate] / ( 1 + [Calculation].[VATRate])
		END as [VAT],
		[Service].[Code],
		[ServiceType].[FileAs],
		[Employee].[Initials],
		[Division].[FileAs],
		[Project].[ApplyVAT] * 2 + [ServiceRequest].[VATIsIncluded] as [VATState],
		[ServiceRequest].[Number] as [ServiceRequestNumber]
	FROM
		[Calculation]
		INNER JOIN [Project]
				ON [Project].[ID]=[Calculation].[ProjectID]
		INNER JOIN [CalculationLine]
			ON [Calculation].[ID] = [CalculationLine].[CalculationID] AND 
				[Calculation].[DocumentClass] = @requestClass
		INNER JOIN [ServiceRequest]
			ON [ServiceRequest].[Class] = [CalculationLine].[DocumentClass] AND
				[ServiceRequest].[ID] = [CalculationLine].[DocumentID]
		INNER JOIN [ServiceRequestLine]	
			ON [ServiceRequestLine].[ServiceRequestID] = [ServiceRequest].[ID]
		INNER JOIN [ServiceAssignment]
			ON [ServiceAssignment].[ID] = [ServiceRequestLine].[ServiceAssignmentID]
		INNER JOIN [Service]
			ON [Service].[ID] = [ServiceAssignment].[ServiceID]
		INNER JOIN [ServiceType]
			ON [Service].[ServiceTypeID] = [ServiceType].[ID]
		INNER JOIN [Employee]
			ON [Employee].[ID] = [ServiceRequest].[EmployeeID]
		LEFT OUTER JOIN [Division]	
			ON [Division].[ID] = [ServiceRequest].[DivisionID]
	WHERE 
		@DateSelector = 0 OR [Calculation].[Date] BETWEEN @PeriodStart AND @PeriodEnd 
			
	)
	SELECT
		[Calculation].[Number],
		[Calculation].[Date],
		YEAR([Calculation].[Date]) AS [Year],
		MONTH([Calculation].[Date]) AS [Month],
		[SupplierCompany].[FileAs] AS [SupplierFileAs],
		[CustomerCompany].[FileAs] As [CustomerFileAs],
		[Project].[Number] AS [ProjectNumber],
		[Project].[FileAs] as [ProjectFileAs],
		CONVERT(datetime, CONVERT(varchar(8), [Project].[Date], 112), 112) as [ProjectDate],
		CASE 
			WHEN [Calculation].[DocumentClass] = @timeClass AND @language = 'ru' THEN ''
			WHEN [Calculation].[DocumentClass] = @expenseClass AND @language = 'ru' THEN ''
			WHEN [Calculation].[DocumentClass] = @phoneClass AND @language = 'ru' THEN ' '
			WHEN [Calculation].[DocumentClass] = @requestClass AND @language = 'ru' THEN ''
			WHEN [Calculation].[DocumentClass] = @timeClass AND @language = 'en' THEN 'Time'
			WHEN [Calculation].[DocumentClass] = @expenseClass AND @language = 'en' THEN 'Expenses'
			WHEN [Calculation].[DocumentClass] = @phoneClass AND @language = 'en' THEN 'Phone Calls'
			WHEN [Calculation].[DocumentClass] = @requestClass AND @language = 'en' THEN 'Orders'
			ELSE 'Unknown'
		END AS [CalculationType],
		[Calculation].[CurrencyCode],
		[Detail].[VAT],
		[Detail].[Amount],
		[Detail].[Price],
		[Detail].[Total],
		[Detail].[TotalWithVAT],
		[Detail].[ServiceType],
		[Detail].[Code],
		[Detail].[Employee],
		[Detail].[Division],
		[Detail].[ServiceRequestNumber],
		ROW_NUMBER() OVER(ORDER BY [Calculation].[Number]) as [ID],
		[Detail].[VATState],
		[Calculation].[ID] as [CalculationID],
		ISNULL(
			(SELECT
				MAX([Payment].[Date])
			FROM
				[Payment]
				INNER JOIN [PaymentDetail]
					ON [PaymentDetail].[PaymentID] = [Payment].[ID]
				INNER JOIN [Invoice]
					ON [Invoice].[ID] = [PaymentDetail].[InvoiceID]
				INNER JOIN [InvoiceLine]
					ON [InvoiceLine].[InvoiceID] = [Invoice].[ID]
			WHERE
				[InvoiceLine].[DocumentClass] = [Calculation].[Class] AND
				[InvoiceLine].[DocumentID] = [Calculation].[ID])
		,
			(SELECT 
				MAX([Payment].[Date])
			 FROM
				[Payment]
				INNER JOIN [PaymentDetail]
					ON [PaymentDetail].[PaymentID] = [Payment].[ID]
				INNER JOIN [StatementAdvanceInvoice]
					ON [StatementAdvanceInvoice].[AdvanceInvoiceID] = [PaymentDetail].[InvoiceID]
				INNER JOIN [Statement]
					ON [Statement].[ID] = [StatementAdvanceInvoice].[StatementID]
				INNER JOIN [StatementLine]
					ON [StatementLine].[DocumentClass] = [Calculation].[Class]
						AND [StatementLine].[DocumentID] = [Calculation].[ID] 
						AND [Statement].[ID] = [StatementLine].[StatementID]
					)
			) AS [PaymentDate]
	FROM
		[Calculation]
		INNER JOIN [Project]
			ON [Calculation].[ProjectID]=[Project].[ID]
		INNER JOIN [Company] AS [SupplierCompany]
			ON [Project].[SupplierCompanyID] = [SupplierCompany].[ID]
		INNER JOIN [Company] AS [CustomerCompany]
			ON [Project].[CustomerCompanyID] = [CustomerCompany].[ID]
		INNER JOIN [CalculationLine]
			ON [CalculationLine].[CalculationID] = [Calculation].[ID]
		INNER JOIN [Detail] 
			ON [CalculationLine].[ID] = [Detail].[CalculationLineID]
	WHERE 
		(@StateSelector = 0) OR 
		(@StateSelector = 5 AND [Calculation].[State] != 4) OR
		(@StateSelector = 6 AND [Calculation].[State] = 3 AND
			EXISTS(
				SELECT
					[Invoice].[ID]
				FROM
					[Invoice]
					INNER JOIN [InvoiceLine]
						ON [InvoiceLine].[InvoiceID] = [Invoice].[ID]
				WHERE
					[Invoice].[State] = 7 AND --  
					[InvoiceLine].[DocumentClass] = [Calculation].[Class] AND
					[InvoiceLine].[DocumentID] = [Calculation].[ID])
		) OR
		(@StateSelector != 0 AND [Calculation].[State] = @StateSelector)

	RETURN 0
GO

GRANT EXEC ON VistaCalculationReport TO PUBLIC
GO
