IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'VistaWordStatement')
	BEGIN
		PRINT 'Dropping Procedure VistaWordStatement'
		DROP  Procedure  VistaWordStatement
	END

GO

PRINT 'Creating Procedure VistaWordStatement'
GO

--------------------------------------------------------------------------------------
-- Copyright (c) 2003 by Avicom
--------------------------------------------------------------------------------------
-- $Archive: $
-- $Author: $
-- $Revision: $
-- $Modtime: $
-- $Date: $
--
-- Description: this stored procedure ...........
-- Input:	.........
-- Output:      0 - success, negative number - otherwise
--
-- Notes:       no
--
--
/* $Log: $ 
   $NoKeywords: $ 
--------------------------------------------------------------------------------------*/

CREATE Procedure VistaWordStatement
	@ID int,
	@Version TName,
	@Language TName = NULL
AS
	SET NOCOUNT ON
	DECLARE
		@Total money,
		@TotalWithVAT money,
		@VAT money	,
		@TotalCount int,
		@VATRate decimal(18,4),
		@VATRateCount int,
		@Currency TName,
		@AdvanceValue TMoney,
		@AdvanceVAT TMoney,
		@PaymentValue TMoney,
		@PaymentVAT TMoney,
		@Class TClass,
		@Code TName,
		@CompanyClass TClass,
		@SupplierID TIdentifier,
		@CustomerID TIdentifier,
		@CurrencyCode TCode
		
	SET @CompanyClass = dbo.adfGetClassByName('Adf.Company')
	SET @Code = CASE WHEN @Language = 'en' THEN 'Act; ' ELSE '; ' END
	SELECT
		@Currency = CASE [Project].[StatementCurrency] WHEN 0 THEN 'Project' ELSE 'Payment' END,
		@CurrencyCode = CASE [Project].[StatementCurrency] WHEN 0 THEN [Project].[CurrencyCode] ELSE [Statement].[CurrencyCode] END,
		@Class = [Statement].[Class],
		@Total = CASE [Project].[StatementCurrency]
			WHEN 0 THEN [Statement].[TotalPC]
			ELSE [Statement].[Total]
		END,
		@VAT = CASE [Project].[StatementCurrency]
			WHEN 0 THEN [Statement].[VATPC]
			ELSE [Statement].[VAT]
		END,
		@TotalWithVAT = CASE [Project].[StatementCurrency]
			WHEN 0 THEN [Statement].[TotalWithVATPC]
			ELSE [Statement].[TotalWithVAT]
		END,
		@SupplierID = [Statement].[SupplierCompanyID],
		@CustomerID = [Statement].[CustomerCompanyID]
	FROM
		[Statement]
		INNER JOIN [Project]
			ON [Statement].[ProjectID] = [Project].[ID]
	WHERE
		[Statement].[ID] = @ID	
		
	SELECT 
		@AdvanceValue = SUM(ISNULL(
			CASE
				WHEN @Currency = 'Payment'
					THEN [Invoice].[TotalWithVAT]
				WHEN @Currency = 'Project'
					THEN [Invoice].[TotalWithVATPC]
			END, 0.0)),
		@AdvanceVAT = SUM(ISNULL(
			CASE
				WHEN @Currency = 'Payment'
					THEN [Invoice].[VAT]
				WHEN @Currency = 'Project'
					THEN [Invoice].[VATPC]
			END, 0.0))
	FROM
		[StatementAdvanceInvoice] 
		INNER JOIN [Invoice]
			ON [Invoice].[ID] = [StatementAdvanceInvoice].[AdvanceInvoiceID]
	WHERE
		[StatementAdvanceInvoice].[StatementID] = @ID
		
	SELECT 
		@PaymentValue = SUM(ISNULL(
			CASE
				WHEN @Currency = 'Payment'
					THEN [StatementAdvanceInvoice].[StatementPaymentValue]
				WHEN @Currency = 'Project'
					THEN [StatementAdvanceInvoice].[StatementPaymentValuePC]
			END, 0.0)),
		@PaymentVAT = SUM(ISNULL(
			CASE
				WHEN @Currency = 'Payment'
					THEN [StatementAdvanceInvoice].[StatementPaymentVAT]
				WHEN @Currency = 'Project'
					THEN [StatementAdvanceInvoice].[StatementPaymentVATPC]
			END, 0.0))
	FROM
		[StatementAdvanceInvoice] 
	WHERE
		[StatementAdvanceInvoice].[StatementID] = @ID
	
	DECLARE @t TABLE(
		[ID] TIdentifier,
		[Description] TNotes,
		[Amount] TAmount,
		[Price] TMoney,
		[Total] TMoney,
		[VATRate] TMoney,
		[VAT] TMoney,
		[PriceWithVAT] TMoney,
		[TotalWithVAT] TMoney,
		[UOMID] TIdentifier,
		[ServiceTypeID] TIdentifier,
		[ServiceTypeFileAs] TName,
		[CalculationNumber] TCode,
		[CalculationDate] TDate		
	)
	IF @Currency = 'Payment'
	BEGIN
		INSERT INTO @t
			([ID],
			[Description],
			[Amount],[Price],[Total],
			[VATRate],[VAT],
			[PriceWithVAT],[TotalWithVAT],
			[UOMID],[ServiceTypeID],[ServiceTypeFileAs],
			[CalculationNumber], [CalculationDate])
		SELECT
			[StatementLine].[ID],
			[StatementLine].[Description],
			[StatementLine].[Amount],
			CASE 
				WHEN [StatementLine].[VATIsIncluded] = 1 AND [StatementLine].[Amount] != 0
					THEN ([StatementLine].[Price] * [StatementLine].[Amount] - [StatementLine].[VAT]) / [StatementLine].[Amount]
				ELSE [StatementLine].[Price]
			END,
			[StatementLine].[Amount] * 	
				CASE 
					WHEN [StatementLine].[VATIsIncluded] = 1 AND [StatementLine].[Amount] != 0
						THEN ([StatementLine].[Price] * [StatementLine].[Amount] - [StatementLine].[VAT]) / [StatementLine].[Amount]
					ELSE [StatementLine].[Price]
				END,
			[StatementLine].[VATRate],
			[StatementLine].[VAT],		
			CASE 
				WHEN [StatementLine].[VATIsIncluded] = 0 AND [StatementLine].[Amount] != 0
					THEN ([StatementLine].[Price] * [StatementLine].[Amount] + [StatementLine].[VAT]) / [StatementLine].[Amount]
				ELSE [StatementLine].[Price]
			END,
			[StatementLine].[Amount] * 	
				CASE 
					WHEN [StatementLine].[VATIsIncluded] = 0 AND [StatementLine].[Amount] != 0
						THEN ([StatementLine].[Price] * [StatementLine].[Amount] + [StatementLine].[VAT]) / [StatementLine].[Amount]
					ELSE [StatementLine].[Price]
				END,
			[StatementLine].[UOMID],
			[ServiceType].[ID],
			[ServiceType].[FileAs],
			[Calculation].[Number],
			[Calculation].[Date]
		FROM
			[StatementLine]
			LEFT OUTER JOIN [ServiceType]
				ON [StatementLine].[ServiceTypeID] = [ServiceType].[ID]
			LEFT OUTER JOIN [Calculation]
				ON [StatementLine].[DocumentClass] = [Calculation].[Class] AND
					[StatementLine].[DocumentID] = [Calculation].[ID]
			WHERE 
			[StatementLine].[StatementID] = @ID	
	END
	ELSE
	BEGIN
		INSERT INTO @t
			([ID],
			[Description],
			[Amount],[Price],[Total],
			[VATRate],[VAT],
			[PriceWithVAT],[TotalWithVAT],
			[UOMID],[ServiceTypeID],[ServiceTypeFileAs],
			[CalculationNumber], [CalculationDate])
		SELECT
			[StatementLine].[ID],
			[StatementLine].[Description],
			[StatementLine].[Amount],
			CASE 
				WHEN [StatementLine].[VATIsIncluded] = 1 AND [StatementLine].[Amount] != 0
					THEN ([StatementLine].[PricePC] * [StatementLine].[Amount] - [StatementLine].[VATPC]) / [StatementLine].[Amount]	
				ELSE [StatementLine].[PricePC] 	 
			END,
			[StatementLine].[Amount] * 	
				CASE 
					WHEN [StatementLine].[VATIsIncluded] = 1 AND [StatementLine].[Amount] != 0
						THEN ([StatementLine].[PricePC] * [StatementLine].[Amount] - [StatementLine].[VATPC]) / [StatementLine].[Amount]	
					ELSE [StatementLine].[PricePC] 
				END,
			[StatementLine].[VATRate],
			[StatementLine].[VATPC],
			CASE 
				WHEN [StatementLine].[VATIsIncluded] = 0 AND [StatementLine].[Amount] != 0
					THEN ([StatementLine].[PricePC] * [StatementLine].[Amount] + [StatementLine].[VATPC]) / [StatementLine].[Amount]	
				ELSE [StatementLine].[PricePC] 	 
			END,
			[StatementLine].[Amount] * 	
				CASE 
					WHEN [StatementLine].[VATIsIncluded] = 0 AND [StatementLine].[Amount] != 0
						THEN ([StatementLine].[PricePC] * [StatementLine].[Amount] + [StatementLine].[VATPC]) / [StatementLine].[Amount]	
					ELSE [StatementLine].[PricePC] 
				END,
			[StatementLine].[UOMID],
			[ServiceType].[ID],
			[ServiceType].[FileAs],
			[Calculation].[Number],
			[Calculation].[Date]
		FROM
			[StatementLine]
			LEFT OUTER JOIN [ServiceType]
				ON [StatementLine].[ServiceTypeID] = [ServiceType].[ID]
			LEFT OUTER JOIN [Calculation]
				ON [StatementLine].[DocumentClass] = [Calculation].[Class] AND
					[StatementLine].[DocumentID] = [Calculation].[ID]
		WHERE 
			[StatementLine].[StatementID] = @ID	
	END	
	SELECT
		@TotalCount = COUNT(*),
		@VATRate = MAX([VATRate]),
		@VATRateCount = COUNT(DISTINCT [VATRate])
	FROM
		@t

	SELECT
		[Statement].[ID],
		[Statement].[FileAs],
		[Statement].[Number],
		[Statement].[Date],
		[Statement].[PeriodStart],
		[Statement].[PeriodEnd],
		
		[Statement].[ProjectID],
		[Project].[FileAs] AS [ProjectFileAs],
		[Project].[Number] AS [ProjectNumber],
		[Project].[ContractNumber] AS [ContractNumber],
		[Project].[ContractDate] AS [ContractDate],
		[Statement].[SupplierCompanyID],
		[Statement].[CustomerCompanyID],

		[CustomerCompany].[LegalName] AS [CustomerCompanyLegalName],
		[SupplierCompany].[LegalName] AS [SupplierCompanyLegalName],
		
		[SupplierCompany].[LegalCode] AS [SupplierCompanyLegalCode],
		[SupplierCompany].[KPP] AS [SupplierCompanyKPP],

		[CustomerCompany].[LegalCode] AS [CustomerCompanyLegalCode],
		[CustomerCompany].[KPP] AS [CustomerCompanyKPP],

		[CustomerCompany].[ExecutiveEmployeeID] AS [CustomerExecutiveEmployeeID],
		[CustomerExecutive].[LastName] + 
			ISNULL(' ' + [CustomerExecutive].[FirstName], '') + 
				ISNULL(' ' + [CustomerExecutive].[MiddleName], '') AS [CustomerExecutiveName],
		[CustomerExecutive].[FileAs] AS [CustomerExecutiveFileAs],
		[CustomerExecutivePosition].[ID] AS [CustomerExecutivePositionID],
		[CustomerExecutivePosition].[FileAs] AS [CustomerExecutivePositionFileAs],

		[SupplierCompany].[ExecutiveEmployeeID] AS [SupplierExecutiveEmployeeID],
		[SupplierExecutive].[LastName] + 
			ISNULL(' ' + [SupplierExecutive].[FirstName], '') + 
				ISNULL(' ' + [SupplierExecutive].[MiddleName], '') AS [SupplierExecutiveName],
		[SupplierExecutive].[FileAs] AS [SupplierExecutiveFileAs],
		[SupplierExecutivePosition].[ID] AS [SupplierExecutivePositionID],
		[SupplierExecutivePosition].[FileAs] AS [SupplierExecutivePositionFileAs],

		@Total AS [Total],
		@VAT AS [VAT],
		CASE
			WHEN [Project].[ApplyVAT] = 0 THEN @Total
			ELSE @Total + @VAT
		END AS [TotalWithVAT],
		CASE
			WHEN @Total IS NULL THEN 0
			ELSE 1
		END	AS [TotalFlag],
		
		dbo.fnPjm5CurrencyToString(@Total, @CurrencyCode) AS [TotalCurrencyCode],
		dbo.fnPjm5CurrencyToString(@VAT, @CurrencyCode) AS [VATCurrencyCode],
		CASE
			WHEN [Project].[ApplyVAT] = 0 THEN dbo.fnPjm5CurrencyToString(@Total, @CurrencyCode)
			ELSE dbo.fnPjm5CurrencyToString((@Total + @VAT), @CurrencyCode)
		END AS [TotalWithVATCurrencyCode],

		@AdvanceValue AS [AdvanceValue],
		@AdvanceVAT AS [AdvanceVAT],
		CASE
			WHEN ISNULL(@AdvanceValue, 0) = 0 THEN 0
			ELSE 1 
		END	AS [AdvanceFlag],
		dbo.fnPjm5CurrencyToString(@AdvanceValue, @CurrencyCode) AS [AdvanceValueCurrencyCode],
		dbo.fnPjm5CurrencyToString(@AdvanceVAT, @CurrencyCode) AS [AdvanceVATCurrencyCode],

		@PaymentValue AS [PaymentValue],
		@PaymentVAT AS [PaymentVAT],
		CASE
			WHEN ISNULL(@PaymentValue, 0) = 0 THEN 0
			ELSE 1 
		END	AS [PaymentFlag],
		dbo.fnPjm5CurrencyToString(ISNULL(@PaymentValue, 0.0), @CurrencyCode) AS [PaymentValueCurrencyCode],
		dbo.fnPjm5CurrencyToString(ISNULL(@PaymentVAT, 0.0), @CurrencyCode) AS [PaymentVATCurrencyCode],

		CASE 
			WHEN @TotalWithVAT > ISNULL(@PaymentValue, 0.0) THEN @TotalWithVAT - ISNULL(@PaymentValue, 0.0) 
			ELSE 0.0
		END AS [RemainValue],
		CASE
			WHEN @VAT > ISNULL(@PaymentVAT, 0.0) THEN @VAT - ISNULL(@PaymentVAT, 0.0) 
			ELSE 0.0
		END AS [RemainVAT],
		CASE
			WHEN ISNULL(@PaymentValue, 0.0) > 0 AND ISNULL(@PaymentValue, 0.0) < @TotalWithVAT THEN 1
			WHEN ISNULL(@PaymentValue, 0.0) > 0 AND ISNULL(@PaymentValue, 0.0) >= @TotalWithVAT THEN 2
			ELSE 0
		END	AS [RemainFlag],
		CASE
			WHEN @TotalWithVAT > ISNULL(@PaymentValue, 0.0) THEN dbo.fnPjm5CurrencyToString(@TotalWithVAT - ISNULL(@PaymentValue, 0.0), @CurrencyCode) 
			ELSE dbo.fnPjm5CurrencyToString(0.0, @CurrencyCode) 
		END AS [RemainValueCurrencyCode],
		CASE
			WHEN @VAT > ISNULL(@PaymentVAT, 0.0) THEN dbo.fnPjm5CurrencyToString(@VAT - ISNULL(@PaymentVAT, 0.0), @CurrencyCode)
			ELSE dbo.fnPjm5CurrencyToString(0.0, @CurrencyCode)
		END AS [RemainVATCurrencyCode],

		CASE
			WHEN [Project].[ApplyVAT] = 0 THEN 0
			WHEN [Project].[ApplyVAT] = 1 AND @VATRateCount = 1 THEN (@VATRate * 100)
			ELSE NULL
		END AS [VATRate],
		CASE
			WHEN [Project].[ApplyVAT] = 1 AND @VATRateCount = 1 THEN 1
			ELSE 0
		END AS [VATFlag],
		CASE
			WHEN [Project].[ApplyVAT] = 0 THEN 0
			ELSE 1
		END	AS [VATRateFlag],

		CASE @Currency WHEN 'Payment' THEN [Currency].[ID] ELSE [ProjectCurrency].[ID] END AS [CurrencyID],
		CASE @Currency WHEN 'Payment' THEN [Currency].[FileAs] ELSE [ProjectCurrency].[FileAs] END AS [CurrencyFileAs],
		CASE @Currency WHEN 'Payment' THEN [Currency].[Code] ELSE [ProjectCurrency].[Code] END AS [CurrencyCode],
		CASE 
			WHEN @Currency = 'Payment' AND
				dbo.adfPropertyValueGetText([Currency].[Class], [Currency].[ID], '00000001-0000-0000-0000-000000000000') = 'False' 
				THEN 0
			WHEN @Currency != 'Payment' AND
				dbo.adfPropertyValueGetText([ProjectCurrency].[Class], [ProjectCurrency].[ID], '00000001-0000-0000-0000-000000000000') = 'False' 
				THEN 0
			ELSE 1 
		END AS [CurrencyMale]

	FROM
		[Statement]
		INNER JOIN [Project]
			ON [Statement].[ProjectID] = [Project].[ID]
		INNER JOIN [Currency]
			ON [Statement].[CurrencyCode] = [Currency].[Code]
		INNER JOIN [Currency] AS [ProjectCurrency]
			ON [Project].[CurrencyCode] = [ProjectCurrency].[Code]
			
		INNER JOIN [Company] AS [SupplierCompany]	
			ON [Statement].[SupplierCompanyID] = [SupplierCompany].[ID]
		INNER JOIN [Company] AS [CustomerCompany]	
			ON [Statement].[CustomerCompanyID] = [CustomerCompany].[ID]
			
		LEFT OUTER JOIN [Employee] AS [SupplierExecutive]
			ON [SupplierCompany].[ExecutiveEmployeeID] = [SupplierExecutive].[ID]
		LEFT OUTER JOIN [Position] AS [SupplierExecutivePosition]
			ON [SupplierExecutive].[PositionID] = [SupplierExecutivePosition].[ID]

		LEFT OUTER JOIN [Employee] AS [CustomerExecutive]
			ON [CustomerCompany].[ExecutiveEmployeeID] = [CustomerExecutive].[ID]
		LEFT OUTER JOIN [Position] AS [CustomerExecutivePosition]
			ON [CustomerExecutive].[PositionID] = [CustomerExecutivePosition].[ID]
	WHERE 
		[Statement].[ID] = @ID	

	SELECT
		[ID],
		[Description],
		[Amount],
		[Price],
		[Total],
		[VAT],
		[PriceWithVAT],
		[TotalWithVAT],
		[UOMID],
		[ServiceTypeID],
		[ServiceTypeFileAs],
		[CalculationNumber],
		[CalculationDate]
	FROM
		@t
	ORDER BY
		[ServiceTypeFileAs] ASC,
		[Description] ASC 
	
	SELECT
		@SupplierID AS [SupplierCompanyID],
		[Address].[AddressTypeID],
		[Address].[Street],
		[Address].[City],
		[Address].[Region],
		[Address].[PostalCode],
		[Address].[Country],
		[Address].[Mail],
		dbo.fnPjm5AddressAsString([Address].[Street], [Address].[City],[Address].[Region],[Address].[PostalCode],[Address].[Country]) AS [AsString]
	FROM
		[Address]
	WHERE
		[Address].[BusinessCardID] = @SupplierID AND 
		[Address].[BusinessCardClass] = @CompanyClass  	
		
	SELECT
		@CustomerID AS [CustomerCompanyID],
		[Address].[AddressTypeID],
		[Address].[Street],
		[Address].[City],
		[Address].[Region],
		[Address].[PostalCode],
		[Address].[Country],
		[Address].[Mail],
		dbo.fnPjm5AddressAsString([Address].[Street], [Address].[City],[Address].[Region],[Address].[PostalCode],[Address].[Country]) AS [AsString]
	FROM
		[Address]
	WHERE
		[Address].[BusinessCardID] = @CustomerID AND 
		[Address].[BusinessCardClass] = @CompanyClass 

	
	DECLARE @author TName
	SELECT @author = [FileAs] FROM [Employee] WHERE [ID] = dbo.adfGetCurrentEmployeeID()
	SELECT
      [Statement].[FileAs] AS [Title],
      [Project].[FileAs] AS [Subject],
      @author AS [Author],
      [Employee].[FileAs] AS [Manager],
      [Company].[FileAs] AS [Company],
      dbo.fnPjm5GetCategoryList(@ID, @Class) AS [Category],
      @Code + [Project].[Number] AS [Keywords]
     FROM
		[Statement]
		INNER JOIN [Project]
			ON [Statement].[ProjectID] = [Project].[ID]
		INNER JOIN [Company]
			ON [Project].[SupplierCompanyID] = [Company].[ID]
		INNER JOIN [Employee]
			ON [Employee].[ID] = [Project].[ProjectManagerEmployeeID]
     WHERE
		[Statement].[ID] = @ID

		
RETURN 0
GO

GRANT EXEC ON VistaWordStatement TO PUBLIC

GO

