 IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm5TaskRequestBrowseByEmployee]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm5TaskRequestBrowseByEmployee'
		DROP PROCEDURE [dbo].[pjm5TaskRequestBrowseByEmployee]
	END
GO

PRINT 'Creating procedure pjm5TaskRequestBrowseByEmployee'
GO

/******************************************************************************
**		File: pjm5TaskRequestBrowseByEmployee.sql
**		Name: pjm5TaskRequestBrowseByEmployee
**		Desc: this procedure automatically generated by Avicom.Adf.Generator
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: Dmitry.Kolchev
**		Date: 04/03/2006 13:49:21
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		9.04.10		KOV				 @DateSelector  WHERE   
**									(     )
**
**		15.04.10	KOV				  " "  ""
**      13.10.10    BVY                 ""  ""
**		01.04.13	SOV				   . -
**		08.11.13	SAP				   ,    
**		02.03.15	KUZ				   EndTime
*******************************************************************************/
CREATE PROCEDURE [pjm5TaskRequestBrowseByEmployee]
	@Presentation varchar(32),
	@StateSelector	INT,
	@AuthorID TIdentifier,
	@EmployeeID TIdentifier,
	@DateSelector int,
	@PeriodStart TDateTime,
	@PeriodEnd TDateTime,
	@AcceptedSelector INT
AS
	SET NOCOUNT ON

	DECLARE @CurrentEmployeeID TIdentifier,
		@CurrentUserID TIdentifier,
		@CreatedBy TIdentifier,
		@Now datetime,
		@IsAssistant TBool,
		@AccessToPublicFolders TBool
				
	SET @AccessToPublicFolders = dbo.adfHasAccessRight(dbo.adfGetRightID('AccessToPublicFolders'))
	SET @IsAssistant = dbo.adfHasAccessRight(dbo.adfGetRightID('Assistant'))
	SET @CurrentEmployeeID = dbo.adfGetCurrentEmployeeID()
	SET @CurrentUserID = dbo.adfGetCurrentUserID()
	SET @Now = GETDATE()
	IF @AuthorID IS NOT NULL
		SELECT @CreatedBy = ISNULL([UserID], -1) FROM [Employee] WHERE [ID] = @AuthorID

	IF @DateSelector = 1
		SET @PeriodStart = CAST(CAST(YEAR(@Now) AS varchar(4)) + '0101' AS datetime)
	ELSE IF @DateSelector = 2
		SET @PeriodStart = CAST(CAST(YEAR(@Now)+1 AS varchar(4)) + '0101' AS datetime)
	ELSE IF @DateSelector = 3
		SET @PeriodStart = DATEADD(day, 1-DATEPART(dayofyear, @PeriodStart), @PeriodStart)
	SET @PeriodStart = CONVERT(datetime, CONVERT(varchar(8), @PeriodStart, 112), 112)
	SET @PeriodEnd = DATEADD(year, 1, @PeriodStart)

	SELECT
		[TaskRequest].[Class],
		[TaskRequest].[ID],
		[TaskRequest].[State],
		[TaskRequest].[Flag],
		[TaskRequest].[FileAs],
		[TaskRequest].[ProjectID],
		[TaskRequest].[PriorityID],
		[TaskRequest].[DueTime],
		[TaskRequest].[StartTime],
		[TaskRequest].[EndTime],
		[TaskRequest].[SendStatusReport],
		[TaskRequest].[TaskRequestStatusID],
		[TaskRequest].[PercentComplete],
		(CAST([TaskRequest].[PercentComplete] * 100 as int) / 25) * 25 as PC,
		[TaskRequest].[DocumentCardID],
		[TaskRequest].[SupervisorEmployeeID],
		[DocumentCard].[FileAs] AS [DocumentCardFileAs],
		[DocumentCard].[CardOwnerID],
		[TaskRequest].[Comments],
		[TaskRequest].[ExternalID],
		[TaskRequest].[Created],
		[TaskRequest].[CreatedBy],
		[TaskRequest].[Modified],
		[TaskRequest].[ModifiedBy],
		[TaskRequest].[Timestamp],
		[TaskRequestAssignment].[EmployeeID],
		ISNULL([TaskRequestAssignment].[EmployeeID], -1) as [EmployeeIDKey],
		[TaskRequest].[ActualBudget],
		[TaskRequest].[ActualManHour],
		[TaskRequest].[Budget],
		[TaskRequest].[ManHour]
	FROM
		[TaskRequest] 
		LEFT OUTER JOIN [DocumentCard]
			ON [DocumentCard].[ID] = [TaskRequest].[DocumentCardID]
		LEFT OUTER JOIN [TaskRequestAssignment]
			ON ([TaskRequest].[ID] = [TaskRequestAssignment].[TaskRequestID])
	WHERE
		(
			@AccessToPublicFolders = 1 OR
			[TaskRequest].[DocumentCardID] IS NULL
			
		) AND		
		(
			@CreatedBy IS NULL OR [TaskRequest].[CreatedBy] = @CreatedBy
		) AND
		(
			@DateSelector = 0 OR 
			(
				COALESCE([TaskRequest].[DueTime], [TaskRequest].[Created]) >= @PeriodStart
				AND
				COALESCE([TaskRequest].[StartTime], @Now) < @PeriodEnd
			)
		) AND
		(
			@EmployeeID IS NULL OR 
			EXISTS(
				SELECT * 
				FROM [TaskRequestAssignment] 
				WHERE 
					[TaskRequestAssignment].[TaskRequestID] = [TaskRequest].[ID] AND 
					[TaskRequestAssignment].[EmployeeID]=@EmployeeID
				)
		) AND
		(
			(@Presentation = 'Own' -- 
				AND 
				((@StateSelector = 0 AND [TaskRequest].[State] <> 6) OR (@StateSelector = 1 AND [TaskRequest].[State] IN (1, 2))
									OR (@StateSelector = 2 AND [TaskRequest].[State] = 3)
									OR (@StateSelector = 3 AND [TaskRequest].[State] = 4)
									OR (@StateSelector = 4 AND [TaskRequest].[State] = 5))
				AND
				[TaskRequest].[CreatedBy] = @CurrentUserID
				--    ""  ""
				AND
				(@AcceptedSelector=0 OR (@AcceptedSelector=1 AND [TaskRequestAssignment].[Accepted]=0))
			)
			OR
			(@Presentation = 'Control' -- 
				AND 
				((@StateSelector = 0 AND [TaskRequest].[State] <> 6) OR (@StateSelector = 1 AND [TaskRequest].[State] IN (1, 2))
									OR (@StateSelector = 2 AND [TaskRequest].[State] = 3)
									OR (@StateSelector = 3 AND [TaskRequest].[State] = 4)
									OR (@StateSelector = 4 AND [TaskRequest].[State] = 5))
				AND
				[TaskRequest].[SupervisorEmployeeID] = @CurrentEmployeeID
			)
			OR
			(@Presentation = 'All' AND -- 
				((@StateSelector = 0 AND [TaskRequest].[State] <> 6) OR (@StateSelector != 0 AND [TaskRequest].[State] = @StateSelector))
				AND
				(
					@IsAssistant = 1 OR
					[TaskRequest].[CreatedBy] = @CurrentUserID OR
					[TaskRequest].[SupervisorEmployeeID] = @CurrentEmployeeID OR
					dbo.fnPjm5IsProjectMember([TaskRequest].[ProjectID], @CurrentEmployeeID) = 1 OR
					dbo.fnPjm5IsProjectManager([TaskRequest].[ProjectID], @CurrentEmployeeID) = 1 OR
					dbo.fnPjm7IsManagerAssistant([TaskRequest].[ProjectID], @CurrentEmployeeID) = 1 OR
					((EXISTS(SELECT * FROM [TaskRequestAssignment] WHERE [TaskRequestAssignment].[TaskRequestID] = [TaskRequest].[ID] AND [TaskRequestAssignment].[EmployeeID]=@CurrentEmployeeID)) AND [TaskRequest].[ProjectID] IS NULL)
				)
			)
		)	
	RETURN 0
GO

GRANT EXECUTE ON [pjm5TaskRequestBrowseByEmployee] TO PUBLIC
GO

 IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm5TaskRequestDocumentBrowseByEmployee]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm5TaskRequestDocumentBrowseByEmployee'
		DROP PROCEDURE [dbo].[pjm5TaskRequestDocumentBrowseByEmployee]
	END
GO