IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'TimeEntryAllBrowse')
	BEGIN
		PRINT 'Dropping Procedure TimeEntryAllBrowse'
		DROP  Procedure  TimeEntryAllBrowse
	END

GO

PRINT 'Creating Procedure TimeEntryAllBrowse'
GO
/******************************************************************************
**		File: 
**		Name: TimeEntryAllBrowse
**		Desc: 
**
**		This template can be customized:
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		05.04.13	SOV				. -       
*******************************************************************************/
CREATE Procedure TimeEntryAllBrowse
	@PeriodStart TDate,
	@PeriodEnd TDate
AS
	SET NOCOUNT ON
	DECLARE
		@ViewAllFinance TBool,
		@CurrentEmployeeID TIdentifier,
		@UserID TIdentifier,
		@ViewAllProjects TBool, 
		@Assistant TBool
		
	SET @ViewAllFinance = dbo.adfHasAccessRight(dbo.adfGetRightID('ViewAllFinance'))
	SET @CurrentEmployeeID = dbo.adfGetCurrentEmployeeID()
	SET @UserID = dbo.adfGetCurrentUserID()
	
	SET @ViewAllProjects = dbo.adfHasAccessRight(dbo.adfGetRightID('ViewAllProjects')) 
	SET @Assistant = dbo.adfHasAccessRight(dbo.adfGetRightID('Assistant'))
	
	;
	WITH [MemberOfProject] ([ID]) AS
	(
		SELECT 
			[ProjectID] 
		FROM 
			[vwAssignment] 
		WHERE 
			[EmployeeID] = @CurrentEmployeeID AND 
			[State] = 1
	),
	[ManagerOfProject] ([ID]) AS (SELECT [ID] FROM [Project] WHERE [ProjectManagerEmployeeID]=@CurrentEmployeeID),
	[UsedEmployee] ([ID]) AS (SELECT [EmployeeID] FROM [CalendarEmployee] WHERE [UserID]=@UserID),
	[AllTimeEntries] ([Class], [ID], [State], [StartTime], [EndTime],  [EmployeeID], [ProjectID], [Color], [Timestamp], [Guid]) AS 
	(
		SELECT 
			[TimeEntry].[Class],
			[TimeEntry].[ID], 
			[TimeEntry].[State],
			[TimeEntry].[StartTime], 
			DATEADD(minute, [TimeEntry].[Duration], [TimeEntry].[StartTime]) as [EndTime],
			[TimeEntry].[EmployeeID],
			[TimeEntry].[ProjectID],
			[TimeEntry].[Color],
			[TimeEntry].[Timestamp],
			[Class].[Guid]
		FROM
			[TimeEntry]
			INNER JOIN [Class]
				ON ([TimeEntry].[Class] = [Class].[Class])
			INNER JOIN [UsedEmployee]
				ON [UsedEmployee].[ID] = [TimeEntry].[EmployeeID]
		WHERE
			[TimeEntry].[StartTime] BETWEEN @PeriodStart AND @PeriodEnd
	),
	[AllMeetingAttendees] ([Class], [ID], [State], [StartTime], [EndTime],  [EmployeeID], [ProjectID], [Color], [Timestamp], [Guid]) AS 
	(
		SELECT 
			[MeetingRequest].[Class],
			[MeetingRequest].[ID], 
			[MeetingRequest].[State],
			[MeetingRequest].[StartTime], 
			DATEADD(minute, [MeetingRequest].[Duration], [MeetingRequest].[StartTime]) as [EndTime],
			[Attendee].[EmployeeID],
			[MeetingRequest].[ProjectID],
			[MeetingRequest].[Color],
			[MeetingRequest].[Timestamp],
			[Class].[Guid]
		FROM
			[MeetingRequest]
			INNER JOIN [Class]
				ON ([MeetingRequest].[Class] = [Class].[Class])
			INNER JOIN [Attendee]
				ON [Attendee].[MeetingRequestID] = [MeetingRequest].[ID] AND
				(
					[Attendee].[ResponseType] IS NULL OR
					[Attendee].[ResponseType] IN ('Accept', 'Tentative')
				)
			INNER JOIN [UsedEmployee]
				ON [UsedEmployee].[ID] = [Attendee].[EmployeeID]
		WHERE
			([MeetingRequest].[StartTime] BETWEEN @PeriodStart AND @PeriodEnd) AND 
			([MeetingRequest].[State] = 2)
	),
	[AllMeetingOrigins] ([Class], [ID], [State], [StartTime], [EndTime],  [EmployeeID], [ProjectID], [Color], [Timestamp], [Guid]) AS 
	(
		SELECT 
			[MeetingRequest].[Class],
			[MeetingRequest].[ID], 
			[MeetingRequest].[State],
			[MeetingRequest].[StartTime], 
			DATEADD(minute, [MeetingRequest].[Duration], [MeetingRequest].[StartTime]) as [EndTime],
			[MeetingRequest].[OriginatorEmployeeID],
			[MeetingRequest].[ProjectID],
			[MeetingRequest].[Color],
			[MeetingRequest].[Timestamp],
			[Class].[Guid]
		FROM
			[MeetingRequest]
			INNER JOIN [Class]
				ON ([MeetingRequest].[Class] = [Class].[Class])
			INNER JOIN [UsedEmployee]
				ON [UsedEmployee].[ID] = [MeetingRequest].[OriginatorEmployeeID]
		WHERE
			([MeetingRequest].[State] NOT IN (3, 4))
	)
	SELECT
		[AllTimeEntries].[Class],
		[AllTimeEntries].[ID],
		[AllTimeEntries].[State],
		[TimeEntry].[FileAs],
		[AllTimeEntries].[StartTime], 
		[AllTimeEntries].[EndTime],
		[TimeEntry].[ActivityTypeID],
		[AllTimeEntries].[EmployeeID],
		[TimeEntry].[ProjectID],
		[Project].[Number] as [ProjectCode],
		[TimeEntry].[TaskID],
		[Task].[FileAs] as [TaskName],
		[TimeEntry].[EmployeeID],
		[TimeEntry].[Billable],
		[AllTimeEntries].[Color],
		[TimeEntry].[Comments],
		[TimeEntry].[Created],
		[TimeEntry].[Modified],
		[TimeEntry].[CreatedBy],
		[TimeEntry].[ModifiedBy],
		[AllTimeEntries].[Timestamp],
		[AllTimeEntries].[Guid]
	FROM
		[AllTimeEntries]
		LEFT OUTER JOIN [MemberOfProject]
			ON [MemberOfProject].[ID] = [AllTimeEntries].[ProjectID]
		LEFT OUTER JOIN [ManagerOfProject]
			ON [ManagerOfProject].[ID] = [AllTimeEntries].[ProjectID]
		LEFT OUTER JOIN [TimeEntry]
			ON [AllTimeEntries].[ID] = [TimeEntry].[ID]
			AND ( 
				(@ViewAllFinance = 1) OR 
				(@ViewAllProjects = 1 AND @Assistant = 1) OR
				([TimeEntry].[ProjectID] IS NOT NULL AND 
					dbo.fnPjm7IsManagerAssistant([TimeEntry].[ProjectID], @CurrentEmployeeID) = 1) OR
				([TimeEntry].[ProjectID] IS NULL AND [TimeEntry].[EmployeeID] = @CurrentEmployeeID) OR
				ISNULL([MemberOfProject].[ID], [ManagerOfProject].[ID]) = [AllTimeEntries].[ProjectID]
			)
		LEFT OUTER JOIN [Project]
			ON [Project].[ID] = [TimeEntry].[ProjectID]
		LEFT OUTER JOIN [Task]
			ON [TimeEntry].[TaskID] = [Task].[ID] 
	
	UNION ALL
	
	SELECT
		[AllMeetingAttendees].[Class],
		[AllMeetingAttendees].[ID],
		[AllMeetingAttendees].[State],
		[MeetingRequest].[FileAs],
		[AllMeetingAttendees].[StartTime],
		[AllMeetingAttendees].[EndTime],
		[MeetingRequest].[ActivityTypeID],
		[AllMeetingAttendees].[EmployeeID],
		[MeetingRequest].[ProjectID],
		[Project].[Number] as [ProjectCode],
		NULL,
		[MeetingRequest].[Location],
		[MeetingRequest].[OriginatorEmployeeID],
		0,
		[AllMeetingAttendees].[Color],
		[MeetingRequest].[Comments],
		[MeetingRequest].[Created],
		[MeetingRequest].[Modified],
		[MeetingRequest].[CreatedBy],
		[MeetingRequest].[ModifiedBy],
		[AllMeetingAttendees].[Timestamp],
		[AllMeetingAttendees].[Guid]
	FROM
		[AllMeetingAttendees] 
		LEFT OUTER JOIN [MemberOfProject]
			ON [MemberOfProject].[ID] = [AllMeetingAttendees].[ProjectID]
		LEFT OUTER JOIN [ManagerOfProject]
			ON [ManagerOfProject].[ID] = [AllMeetingAttendees].[ProjectID]		
		LEFT OUTER JOIN [MeetingRequest]
			ON [MeetingRequest].[ID] = [AllMeetingAttendees].[ID] 
			AND ( 
				(@ViewAllFinance = 1) OR 
				(@ViewAllProjects = 1 AND @Assistant = 1) OR
				([AllMeetingAttendees].[ProjectID] IS NOT NULL AND 
					dbo.fnPjm7IsManagerAssistant([AllMeetingAttendees].[ProjectID], @CurrentEmployeeID) = 1) OR
				([AllMeetingAttendees].[ProjectID] is null) OR
				([AllMeetingAttendees].[EmployeeID] = @CurrentEmployeeID) OR
				(EXISTS(SELECT *FROM [AllMeetingAttendees] WHERE [ID] = [MeetingRequest].[ID] 
						AND [AllMeetingAttendees].[EmployeeID] = @CurrentEmployeeID))OR
				ISNULL([MemberOfProject].[ID], [ManagerOfProject].[ID]) = [AllMeetingAttendees].[ProjectID]
			)
		LEFT OUTER JOIN [Project]
			ON [MeetingRequest].[ProjectID] = [Project].[ID]

	UNION ALL
	
	SELECT
		[AllMeetingOrigins].[Class],
		[AllMeetingOrigins].[ID],
		[AllMeetingOrigins].[State],
		[MeetingRequest].[FileAs],
		[AllMeetingOrigins].[StartTime],
		[AllMeetingOrigins].[EndTime],
		[MeetingRequest].[ActivityTypeID],
		[AllMeetingOrigins].[EmployeeID],
		[MeetingRequest].[ProjectID],
		[Project].[Number] as [ProjectCode],
		NULL,
		[MeetingRequest].[Location],
		[MeetingRequest].[OriginatorEmployeeID],
		0,
		[MeetingRequest].[Color],
		[MeetingRequest].[Comments],
		[MeetingRequest].[Created],
		[MeetingRequest].[Modified],
		[MeetingRequest].[CreatedBy],
		[MeetingRequest].[ModifiedBy],
		[MeetingRequest].[Timestamp],
		[AllMeetingOrigins].[Guid]
	FROM
		[AllMeetingOrigins] 
		LEFT OUTER JOIN [Attendee]
			ON [Attendee].[MeetingRequestID] = [AllMeetingOrigins].[ID]
			AND [Attendee].[EmployeeID] = @CurrentEmployeeID
		LEFT OUTER JOIN [MemberOfProject]
			ON [MemberOfProject].[ID] = [AllMeetingOrigins].[ProjectID]
		LEFT OUTER JOIN [ManagerOfProject]
			ON [ManagerOfProject].[ID] = [AllMeetingOrigins].[ProjectID]		
		LEFT OUTER JOIN [MeetingRequest]
			ON [MeetingRequest].[ID] = [AllMeetingOrigins].[ID] 
			AND ( 
				(@ViewAllFinance = 1) OR 
				(@ViewAllProjects = 1 AND @Assistant = 1) OR
				([AllMeetingOrigins].[ProjectID] IS NOT NULL AND 
					dbo.fnPjm7IsManagerAssistant([AllMeetingOrigins].[ProjectID], @CurrentEmployeeID) = 1) OR
				([AllMeetingOrigins].[ProjectID] is null) OR
				([Attendee].[EmployeeID] is not null) OR
				ISNULL([MemberOfProject].[ID], [ManagerOfProject].[ID]) = [AllMeetingOrigins].[ProjectID]				
				)
		LEFT OUTER JOIN [Project]
			ON [MeetingRequest].[ProjectID] = [Project].[ID]

	RETURN 0
GO

GRANT EXEC ON TimeEntryAllBrowse TO PUBLIC
GO