IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm6trTimeEntryForTfaDelete')
	BEGIN
		PRINT 'Dropping Trigger pjm6trTimeEntryForTfaDelete'
		DROP  Trigger pjm6trTimeEntryForTfaDelete
	END
GO

/******************************************************************************
**		File:	pjm6trTimeEntryForTfaDelete.sql
**		Name:	pjm6trTimeEntryForTfaDelete
**		Desc:	    
**				     
**
**		This template can be customized:
**              
**
**		Auth: pavel.svintsov
**		Date: 02.06.2009
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**		02.06.09	SPV				 
**    
*******************************************************************************/

PRINT 'Creating Trigger pjm6trTimeEntryForTfaDelete'
GO
CREATE TRIGGER pjm6trTimeEntryForTfaDelete 
ON [TimeEntry] FOR DELETE
AS
BEGIN
    IF @@rowcount = 0
       RETURN
	SET NOCOUNT ON

	DECLARE
		@ServiceRequestClass TClass,
		@TaskRequestClass TClass,
		@IssueClass TClass,
		@ServiceAssignmentClass TClass,
		@invalid int

	SET @ServiceRequestClass = dbo.adfGetClassByName('Pjm5.ServiceRequest')
	SET @TaskRequestClass = dbo.adfGetClassByName('Pjm5.TaskRequest')
	SET @IssueClass = dbo.adfGetClassByName('Pjm5.Issue')
	SET @ServiceAssignmentClass = dbo.adfGetClassByName('Pjm5.ServiceAssignment')
	
	SET @invalid = 0
	
		DECLARE @forSubtract TABLE
	(
		[ID] TIdentifier,
		[DocumentClass] TClass,
		[DocumentID] TIdentifier,
		[Duration] TCount,
		[Cost] TMoney
	)

	INSERT INTO @forSubtract
	([ID], [DocumentClass], [DocumentId], [Duration], [Cost])
	SELECT
		[deleted].[ID],
		[deleted].[DocumentClass],
		[deleted].[DocumentID],
		[deleted].[Duration],
		(CASE WHEN [deleted].[Billable] = 1 THEN [deleted].[Duration] * [deleted].[Rate] / 60 ELSE 0 END)
	FROM
		[deleted]
	WHERE
	(	
		[deleted].[State] in (2, 3, 6)
	) AND
	(
		[deleted].[DocumentClass] in (@TaskRequestClass, @ServiceRequestClass, @IssueClass, @ServiceAssignmentClass)
	)


	------------------------- Task Request ----------------------
	/*SELECT @invalid = count(*)
	FROM
		[TaskRequest]
		INNER JOIN [deleted] AS [Add]
			ON [TaskRequest].[ID] = [Add].[DocumentID] AND
				[TaskRequest].[Class] = [Add].[DocumentClass] AND
				[TaskRequest].[State] not in (1, 2)
		IF @@error != 0
			BEGIN
				ROLLBACK TRANSACTION
				RETURN
			END

	IF @invalid > 0
	BEGIN
		RAISERROR('#tfaDeleteInvalidJobState', 18, -1)
		ROLLBACK TRANSACTION
	END
	ELSE*/
	BEGIN
		UPDATE [TaskRequest]
			SET
				[TaskRequest].[ActualManHour] = [TaskRequest].[ActualManHour] - [Subtract].[Duration],
				[TaskRequest].[ActualBudget] = [TaskRequest].[ActualBudget] - [Subtract].[Cost]
			FROM
				[TaskRequest]
				INNER JOIN @ForSubtract AS [Subtract]
					ON [TaskRequest].[ID] = [Subtract].[DocumentID] AND
						 [TaskRequest].[Class] = [Subtract].[DocumentClass]
				IF @@error != 0
					BEGIN
						ROLLBACK TRANSACTION
						RETURN
					END
	END
	

	---------------------- Service Request ------------------------------
	/*SELECT @invalid = count(*)
	FROM
		[ServiceRequest]
		INNER JOIN [deleted] AS [Add]
			ON [ServiceRequest].[ID] = [Add].[DocumentID] AND
				[ServiceRequest].[Class] = [Add].[DocumentClass] AND
				[ServiceRequest].[State] not in (1, 2, 3)
		IF @@error != 0
			BEGIN
				ROLLBACK TRANSACTION
				RETURN
			END

	IF @invalid > 0
	BEGIN
		RAISERROR('#tfaDeleteInvalidJobState', 18, -1)
		ROLLBACK TRANSACTION
	END
	ELSE*/
	BEGIN
		UPDATE [ServiceRequest]
			SET
				[ServiceRequest].[ActualManHour] = [ServiceRequest].[ActualManHour] - [Subtract].[Duration],
				[ServiceRequest].[ActualBudget] = [ServiceRequest].[ActualBudget] - [Subtract].[Cost]
			FROM
				[ServiceRequest]
				INNER JOIN @ForSubtract AS [Subtract]
					ON [ServiceRequest].[ID] = [Subtract].[DocumentID] AND
						 [ServiceRequest].[Class] = [Subtract].[DocumentClass]
				IF @@error != 0
					BEGIN
						ROLLBACK TRANSACTION
						RETURN
					END
	END
	------------------------- Issue ----------------------
	/*SELECT @invalid = count(*)
	FROM
		[Issue]
		INNER JOIN [deleted] AS [Add]
			ON [Issue].[ID] = [Add].[DocumentID] AND
				[Issue].[Class] = [Add].[DocumentClass] AND
				[Issue].[State] not in (1, 2, 3, 4, 6)
		IF @@error != 0
			BEGIN
				ROLLBACK TRANSACTION
				RETURN
			END

	IF @invalid > 0
	BEGIN
		RAISERROR('#tfaDeleteInvalidJobState', 18, -1)
		ROLLBACK TRANSACTION
	END
	ELSE*/
	BEGIN
		UPDATE [Issue]
			SET
				[Issue].[ActualManHour] = [Issue].[ActualManHour] - [Subtract].[Duration],
				[Issue].[ActualBudget] = [Issue].[ActualBudget] - [Subtract].[Cost]
			FROM
				[Issue]
				INNER JOIN @ForSubtract AS [Subtract]
					ON [Issue].[ID] = [Subtract].[DocumentID] AND
						 [Issue].[Class] = [Subtract].[DocumentClass]
				IF @@error != 0
					BEGIN
						ROLLBACK TRANSACTION
						RETURN
					END
	END
	---------------------- Service Assignment ------------------------------
	/*SELECT @invalid = count(*)
	FROM
		[ServiceAssignment]
		INNER JOIN [deleted] AS [Add]
			ON [ServiceAssignment].[ID] = [Add].[DocumentID] AND
				[ServiceAssignment].[Class] = [Add].[DocumentClass] AND
				[ServiceAssignment].[State] not in (1)
		IF @@error != 0
			BEGIN
				ROLLBACK TRANSACTION
				RETURN
			END

	IF @invalid > 0
	BEGIN
		RAISERROR('#tfaDeleteInvalidJobState', 18, -1)
		ROLLBACK TRANSACTION
	END
	ELSE*/
	BEGIN
		UPDATE [ServiceAssignment]
			SET
				[ServiceAssignment].[ActualManHour] = [ServiceAssignment].[ActualManHour] - [Subtract].[Duration],
				[ServiceAssignment].[ActualBudget] = [ServiceAssignment].[ActualBudget] - [Subtract].[Cost]
			FROM
				[ServiceAssignment]
				INNER JOIN @ForSubtract AS [Subtract]
					ON [ServiceAssignment].[ID] = [Subtract].[DocumentID] AND
						 [ServiceAssignment].[Class] = [Subtract].[DocumentClass]
				IF @@error != 0
					BEGIN
						ROLLBACK TRANSACTION
						RETURN
					END
	END
END
GO
