 IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm5vwReportCalculationLine]') and OBJECTPROPERTY(id, N'IsView') = 1)
	BEGIN
		PRINT 'Dropping view pjm5vwReportCalculationLine'
		DROP VIEW [dbo].[pjm5vwReportCalculationLine]
	END
GO

PRINT 'Creating view pjm5vwReportCalculationLine'
GO


CREATE VIEW [pjm5vwReportCalculationLine]
AS
-- by time
	SELECT
		[CalculationLine].[CalculationID],
		[CalculationLine].[ID],
		[CalculationLine].[DocumentClass],
		[CalculationLine].[DocumentID],
		
		[TimeEntry].[State] AS [DocumentState],
		[TimeEntry].[StartTime] AS [Date],

		[TimeEntry].[EmployeeID],

		[Assignment].[QualificationID] AS [QualificationID],
		[TimeEntry].[ActivityTypeID] AS [TypeID],
		[ActivityType].[Code] AS [TypeCode],
		[ActivityType].[FileAs] AS [TypeFileAs],

		[CalculationLine].[Description],
		[CalculationLine].[Amount],
		[CalculationLine].[Price],
		[CalculationLine].[Amount] * [CalculationLine].[Price] AS [Total],
		[CalculationLine].[VATIsIncluded],
		[CalculationLine].[VAT]
	FROM
		[CalculationLine]
		INNER JOIN [TimeEntry]
			ON [TimeEntry].[Class] = [CalculationLine].[DocumentClass] AND
				[TimeEntry].[ID] = [CalculationLine].[DocumentID]
		LEFT OUTER JOIN [ActivityType]
			ON [TimeEntry].[ActivityTypeID] = [ActivityType].[ID]
		LEFT OUTER JOIN [Assignment]
			ON [TimeEntry].[AssignmentID] = [Assignment].[ID]
	UNION
-- by expense
	SELECT
		[CalculationLine].[CalculationID],
		[CalculationLine].[ID],
		[CalculationLine].[DocumentClass],
		[CalculationLine].[DocumentID],

		[Expense].[State] AS [DocumentState],
		[Expense].[Date],
		[Expense].[EmployeeID],
		[Assignment].[QualificationID] AS [QualificationID],
		[Expense].[ExpenseCodeID] AS [TypeID],
		[ExpenseCode].[Code] AS [TypeCode],
		[ExpenseCode].[FileAs] AS [TypeFileAs],

		[CalculationLine].[Description],
		[CalculationLine].[Amount],
		CASE
			WHEN [CalculationLine].[VATIsIncluded] = 1 AND [CalculationLine].[Amount] != 0
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT]) / [CalculationLine].[Amount]
			ELSE [CalculationLine].[Price]
		END AS [Price],
		CASE
			WHEN [CalculationLine].[VATIsIncluded] = 1
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT])
			ELSE [CalculationLine].[Amount] * [CalculationLine].[Price]
		END AS [Total],
		[CalculationLine].[VATIsIncluded],
		[CalculationLine].[VAT]
	FROM
		(
			[CalculationLine]
			INNER JOIN [Expense]
				ON [Expense].[Class] = [CalculationLine].[DocumentClass] AND
					[Expense].[ID] = [CalculationLine].[DocumentID]
		)	
		LEFT OUTER JOIN
		(
			[Assignment]
			INNER JOIN [AssignmentEntry]
				ON [Assignment].[ID] = [AssignmentEntry].[AssignmentID]
		)
			ON [Expense].[EmployeeID] = [AssignmentEntry].[EmployeeID]
			AND [Expense].[ProjectID] = [Assignment].[ProjectID]
		LEFT OUTER JOIN [ExpenseCode]
			ON [Expense].[ExpenseCodeID] = [ExpenseCode].[ID]
	WHERE
		[Assignment].[State] = 1 
		OR
		(
			[Assignment].[State] = 2
			AND
			-- 
			[Assignment].[Modified] = 
			(
				SELECT 
					MAX([Modified]) 
				FROM
					[Assignment]
					INNER JOIN [AssignmentEntry]
						ON [Assignment].[ID] = [AssignmentEntry].[AssignmentID]
				WHERE
					[Expense].[EmployeeID] = [AssignmentEntry].[EmployeeID]
					AND 
					[Expense].[ProjectID] = [Assignment].[ProjectID]
			) 
		)	
	UNION
-- by phonecall
	SELECT
		[CalculationLine].[CalculationID],
		[CalculationLine].[ID],
		[PhoneCall].[Class],
		[PhoneCall].[ID],

		[Expense].[State] AS [DocumentState],
		[Expense].[Date],
		[Expense].[EmployeeID],
		[Assignment].[QualificationID] AS [QualificationID],
		[PhoneCall].[PhoneCallCodeID] AS [TypeID],
		[PhoneCallCode].[Code] AS [TypeCode],
		[PhoneCallCode].[FileAs] AS [TypeFileAs],

		[CalculationLine].[Description],
		[CalculationLine].[Amount],
		CASE
			WHEN [CalculationLine].[VATIsIncluded] = 1 AND [CalculationLine].[Amount] != 0
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT]) / [CalculationLine].[Amount]
			ELSE [CalculationLine].[Price]
		END AS [Price],
		CASE
			WHEN [CalculationLine].[VATIsIncluded] = 1
				THEN ([CalculationLine].[Amount] * [CalculationLine].[Price] - [CalculationLine].[VAT])
			ELSE [CalculationLine].[Amount] * [CalculationLine].[Price]
		END AS [Total],
		[CalculationLine].[VATIsIncluded],
		[CalculationLine].[VAT]
	FROM
		(
			[CalculationLine]
			INNER JOIN [Expense]
				ON [Expense].[Class] = [CalculationLine].[DocumentClass]
				AND [Expense].[ID] = [CalculationLine].[DocumentID]
			INNER JOIN [PhoneCall]
				ON [PhoneCall].[ExpenseID] = [Expense].[ID]
			INNER JOIN [PhoneCallCode]
				ON [PhoneCall].[PhoneCallCodeID] = [PhoneCallCode].[ID]
		)	
		LEFT OUTER JOIN
		(
			[Assignment]
			INNER JOIN [AssignmentEntry]
				ON [Assignment].[ID] = [AssignmentEntry].[AssignmentID]
		)
			ON [Expense].[EmployeeID] = [AssignmentEntry].[EmployeeID]
			AND [Expense].[ProjectID] = [Assignment].[ProjectID]
		LEFT OUTER JOIN [ExpenseCode]
				ON [Expense].[ExpenseCodeID] = [ExpenseCode].[ID]
	WHERE
		[Assignment].[State] = 1 
		OR
		(
			[Assignment].[State] = 2
			AND
			-- 
			[Assignment].[Modified] = 
			(
				SELECT 
					MAX([Modified]) 
				FROM
					[Assignment]
					INNER JOIN [AssignmentEntry]
						ON [Assignment].[ID] = [AssignmentEntry].[AssignmentID]
				WHERE
					[Expense].[EmployeeID] = [AssignmentEntry].[EmployeeID]
					AND 
					[Expense].[ProjectID] = [Assignment].[ProjectID]
			) 
		)	
	UNION
-- by service request
	SELECT
		[CalculationLine].[CalculationID],
		[CalculationLine].[ID],
		[CalculationLine].[DocumentClass],
		[CalculationLine].[DocumentID],
		CASE [ServiceRequest].[State]
			WHEN 5 THEN 6
			ELSE [ServiceRequest].[State] - 1
		END AS [DocumentState],
		[ServiceRequest].[Date],
		[ServiceRequest].[EmployeeID],
		[Assignment].[QualificationID] AS [QualificationID],
		[ServiceAssignment].[ID] AS [TypeID],
		[Service].[Code] AS [TypeCode],
		COALESCE(
				dbo.adfPropertyValueGetTextByName([ServiceAssignment].[Class], [ServiceAssignment].[ID], 
					'    '),
				dbo.adfPropertyValueGetTextByName([Service].[Class], [Service].[ID], 
					'    '),
				[Service].[FileAs]),
		ISNULL([ServiceRequestLine].[Description], [ServiceRequest].[FileAs]),
		[ServiceRequestLine].[FactAmount],
		CASE 
				WHEN [ServiceRequest].[VATIsIncluded] = 1 
					THEN [ServiceRequestLine].[Price]/(1 + [Calculation].[VATRate])
				ELSE 
			[ServiceRequestLine].[Price]
			END AS [Price],
		CASE 
				WHEN [ServiceRequest].[VATIsIncluded] = 1 
					THEN [ServiceRequestLine].[FactAmount] * [ServiceRequestLine].[Price]/(1+ [Calculation].[VATRate])
				ELSE 
				[ServiceRequestLine].[FactAmount] * [ServiceRequestLine].[Price]			
			END AS [Total],
		[CalculationLine].[VATIsIncluded],
		[CalculationLine].[VAT]
	FROM
		(
			[CalculationLine]
			INNER JOIN [Calculation]
				ON [CalculationLine].[CalculationID] = [Calculation].[ID]
			INNER JOIN [ServiceRequest]
				ON [ServiceRequest].[Class] = [CalculationLine].[DocumentClass]
				AND	[ServiceRequest].[ID] = [CalculationLine].[DocumentID]
			INNER JOIN [ServiceRequestLine]	
				ON [ServiceRequestLine].[ServiceRequestID] = [ServiceRequest].[ID]
			INNER JOIN [ServiceAssignment]
				ON [ServiceAssignment].[ID] = [ServiceRequestLine].[ServiceAssignmentID]
			INNER JOIN [Service]
				ON [Service].[ID] = [ServiceAssignment].[ServiceID]
		)
		LEFT OUTER JOIN
		(
			[Assignment]
			INNER JOIN [AssignmentEntry]
				ON [Assignment].[ID] = [AssignmentEntry].[AssignmentID]
		)
			ON [ServiceRequest].[EmployeeID] = [AssignmentEntry].[EmployeeID]
			AND [ServiceRequest].[ProjectID] = [Assignment].[ProjectID]
		LEFT OUTER JOIN [ServiceType]
			ON [ServiceRequest].[ServiceTypeID] = [ServiceType].[ID]
	WHERE
		[Assignment].[State] = 1 
		OR
		(
			[Assignment].[State] = 2
			AND
			-- 
			[Assignment].[Modified] = 
			(
				SELECT 
					MAX([Modified]) 
				FROM
					[Assignment]
					INNER JOIN [AssignmentEntry]
						ON [Assignment].[ID] = [AssignmentEntry].[AssignmentID]
				WHERE
					[ServiceRequest].[EmployeeID] = [AssignmentEntry].[EmployeeID]
					AND 
					[ServiceRequest].[ProjectID] = [Assignment].[ProjectID]
			) 
		)	
GO